/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.net;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Vector;

import org.apache.commons.net.ftp.FTPReply;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTP;



/**
 * <p>Title: FTP-Client</p>
 * <p>Description: a wrapper around the Jakarta Commons FTP</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: SOS GmbH</p>
 * @author <a href="mailto:ghassan.beydoun@sos-berlin.com">Ghassan Beydoun</a>
 * @version $Id: SOSFTP.java 5650 2009-10-29 11:31:35Z mo $
 * @resource commons-net-1.3.0.jar, 
 *           oro.jar fr den Aufruf FTPFile (=download=http://jakarta.apache.org/site/downloads/downloads_oro.cgi)
 */


public class SOSFTP extends FTPClient implements SOSFileTransfer{
	
	private Vector fileListNames = new Vector();
	
	//private int    proxyPort     = 1080;
	
	//private String proxyHost     = null;;
	
	public SOSFTP() {}
	
	/**
	 * SOSFTP constructor: there is no need to call the connect-Method
	 * @param host the remote ftp server
	 * @throws java.net.SocketException
	 * @throws java.io.IOException
	 * @throws java.net.UnknownHostException
	 */
	public SOSFTP ( String host ) throws SocketException,IOException,UnknownHostException {
		connect( host);
	}
	
	/**
	 * SOSFTP constructor: there is no need to call the connect-Method
	 * @param host the remote ftp server
	 * @param port the port number of the remote server
	 * @throws java.net.SocketException
	 * @throws java.io.IOException
	 * @throws java.net.UnknownHostException
	 */
	public SOSFTP ( String host, int port ) throws SocketException,IOException,UnknownHostException {
		connect( host, port);
	}
	
	public SOSFTP(String ftpHost, 
			int ftpPort, 
			String proxyHost, 
			int proxyPort) throws SocketException, IOException, UnknownHostException {
		//this.setProxyHost(proxyHost);
		//this.setProxyPort(proxyPort);
		this.connect(ftpHost, ftpPort);
	}
	
	/*
	 * @param host the remote ftp server
	 * @param port the port number of the remote server
	 * @throws java.net.SocketException
	 * @throws java.io.IOException
	 * @throws java.net.UnknownHostException
	 * @see org.apache.commons.net.SocketClient#connect(java.lang.String, int)
	 */
	public void connect(String host, int port) throws SocketException,IOException,UnknownHostException {
	
		

		if ( !isConnected()) {			
			super.connect(host, port);
		}
	}
	
	/*
	 * @param host the remote ftp server
	 * @param port the port number of the remote server
	 * @throws java.net.SocketException
	 * @throws java.io.IOException
	 * @throws java.net.UnknownHostException
	 * @see org.apache.commons.net.SocketClient#connect(java.lang.String)
	 */
	public void connect(String hostname) throws SocketException, IOException,UnknownHostException {
		
		if ( !isConnected())			
			super.connect(hostname);
		
		
	}
	
	
	/**
	 * Creates a new subdirectory on the FTP server in the current directory .
	 * @param pathname The pathname of the directory to create.
	 * @return True if successfully completed, false if not.
	 * @throws java.lang.IOException
	 */
	public boolean mkdir( String pathname ) throws IOException {
		return makeDirectory(pathname);
	}
	
	/**
	 * Removes a directory on the FTP server (if empty).
	 * @param pathname The pathname of the directory to remove.
	 * @return True if successfully completed, false if not.
	 * @throws java.lang.IOException
	 */
	public boolean rmdir( String pathname ) throws IOException {
		return removeDirectory( pathname );
	}
	
	/**
	 * turn passive transfer mode on.
	 *
	 * @return The reply code received from the server.
	 */
	public int passive() throws IOException {
		return pasv();
	}
	
	
	/**
	 * return a listing of the contents of a directory in short format on
	 * the remote machine
	 * @param pathname on remote machine
	 * @return a listing of the contents of a directory on the remote machine
	 *
	 * @exception Exception
	 * @see #dir()
	 */
	public Vector nList(String pathname) throws Exception {
		return getFilenames(pathname);		
	} // nList
	
	
	/**
	 * return a listing of the contents of a directory in short format on
	 * the remote machine (without subdirectory)
	 * 
	 * @param pathname on remote machine
	 * @return a listing of the contents of a directory on the remote machine
	 *
	 * @exception Exception
	 * @see #dir()
	 */
	private Vector getFilenames(String pathname) throws Exception {
		String[] fileList = null;
		Vector fileListNames = new Vector();
		fileList = listNames(pathname);
		if ( fileList == null )
			return fileListNames;
		for(int i=0;i<fileList.length;i++) {			
			int x = cd( fileList[i]);
			if(getReplyCode() > 300) {
				pwd(); //um den Fehlercode zu lschen
				fileListNames.add( fileList[i] );				  
			} else {
				cdup();				
			}
		}
		return fileListNames;
		
	} // nList
	
	/**
	 * return a listing of the contents of a directory in short format on
	 * the remote machine
	 * @param pathname on remote machine
	 * @return a listing of the contents of a directory on the remote machine
	 *
	 * @exception Exception
	 * @see #dir()
	 */	  
	public Vector nList(String pathname, boolean recursive) throws Exception {
		
		if(!recursive) {
			//return nList(pathname);
			return getFilenames(pathname);
		} else {
			
			String[] fileList = null;			
			fileList = listNames(pathname);
			if ( fileList == null )
				return fileListNames;
			for(int i=0;i<fileList.length;i++) {			
				int x = cd( fileList[i]);
				if(getReplyCode() > 300) {
					pwd(); //Fehlercode lschen						
					//FTPServer arbeiten anders. Einmal liefert die Methode listNames mit pathname und einmal wird nur die Dateiname mitgeliefert. 
					if(fileList[i].startsWith(pathname))
						fileListNames.add(fileList[i] );
					else
						fileListNames.add(pathname +"/"+  fileList[i] );
				} else {
					String[] split = fileList[i].split("/");//Anzahl der Unterverzeichnisse
					for(int s = 0; s < split.length; s++) 
						cdup();
					nList(fileList[i], recursive);			    	  
				}
			}
			return fileListNames;
			
			
		}
	} // nList
	
	/**
	 * return a listing of the contents of a directory in short format on
	 * the remote machine
	 *
	 * @return a listing of the contents of a directory on the remote machine
	 *
	 * @exception Exception
	 * @see #nList( String )
	 * @see #dir()
	 * @see #dir( String )
	 */
	public Vector nList() throws Exception {
		
		return getFilenames();
		
	} // nList
	
	/**
	 * return a listing of the contents of a directory in short format on
	 * the remote machine (without subdirectory)
	 *
	 * @return a listing of the contents of a directory on the remote machine
	 *
	 * @exception Exception
	 * @see #nList( String )
	 * @see #dir()
	 * @see #dir( String )
	 */
	private Vector getFilenames() throws Exception {
		String[] fileList = null;
		Vector fileListNames = new Vector();
		fileList = listNames();
		if ( fileList == null )
			return fileListNames;
		for(int i=0;i<fileList.length;i++) {			
			int x = cd( fileList[i]);
			if(getReplyCode() > 300) {
				pwd();//um den Fehlercode zu unterdrcken
				fileListNames.add( fileList[i] );				  
			} else {
				cdup();
				//System.out.println("es ist ein Verzeichnis: " + fileList[i]);
			}
		}
		return fileListNames;		
	} // nList
	/**
	 * return a listing of the contents of a directory in short format on
	 * the remote machine
	 *
	 * @return a listing of the contents of a directory on the remote machine
	 *
	 * @exception Exception
	 * @see #nList( String )
	 * @see #dir()
	 * @see #dir( String )
	 */
	
	public Vector nList(boolean recursive) throws Exception {
		if(!recursive) {			
			return getFilenames();
		} else {
			String[] fileList = null;
			//fileListNames = new Vector();
			fileList = listNames();
			if ( fileList == null )
				return fileListNames;
			for(int i=0;i<fileList.length;i++) {			
				int x = cd( fileList[i]);
				if(getReplyCode() > 300) {
					pwd(); //Fehlercode lschen
					fileListNames.add( fileList[i] );				  
				} else {
					cdup();
					nList(fileList[i], recursive);
				}
			}
			return fileListNames;
			
		}
		
	} // nList
	/**
	 * return a listing of the files in a directory in long format on
	 * the remote machine
	 * @param pathname on remote machine
	 * @return a listing of the contents of a directory on the remote machine
	 * @exception Exception
	 * @see #nList()
	 * @see #nList( String )
	 * @see #dir()
	 */
	public Vector dir( String pathname ) throws Exception {
		return getFilenames(pathname);		
		/*Vector fileList = new Vector();
		 try {
		 
		 FTPFile[] listFiles  = listFiles( pathname );
		 for(int i=0;i<listFiles.length;i++) {
		 if (listFiles[i].isFile()) {
		 fileList.add( listFiles[i].toString() );
		 }
		 }
		 } catch (Exception e) {
		 throw new Exception("error in dir(): " + e.getMessage());
		 }
		 return fileList;
		 */
	}
	
	
	/**
	 * return a listing of a directory in long format on
	 * the remote machine
	 * @param pathname on remote machine
	 * @return a listing of the contents of a directory on the remote machine
	 * @exception Exception
	 * @see #nList()
	 * @see #nList( String )
	 * @see #dir()
	 * @deprecated
	 */
	public Vector dir( String pathname, int flag ) throws Exception {
		
		Vector fileList = new Vector();
		FTPFile[] listFiles  = listFiles( pathname );
		for(int i=0;i<listFiles.length;i++) {
			if (flag > 0 && listFiles[i].isDirectory()) {
				fileList.addAll(this.dir(pathname + "/" + listFiles[i].toString(), ((flag >= 1024) ? flag : flag+1024)));   
			} else {
				if (flag >= 1024) {
					fileList.add( pathname + "/" + listFiles[i].toString() );
				} else {
					fileList.add( listFiles[i].toString() );
				}
			}
		}
		return fileList;
	}
	
	
	/**
	 * return a listing of the files of the current directory in long format on
	 * the remote machine
	 * @return a listing of the contents of the current directory on the remote machine
	 * @exception Exception
	 * @see #nList()
	 * @see #nList( String )
	 * @see #dir( String )
	 */
	public Vector dir() throws Exception {
		return dir( "." );
	}
	
	/**
	 * @return The entire text from the last FTP response as a String.
	 */
	public String getResponse() {
		return this.getReplyString();
	}
	
	/**
	 * return the size of remote-file on the remote machine on success, otherwise -1
	 * @param remoteFile the file on remote machine
	 * @return the size of remote-file on remote machine
	 */
	public long size(String remoteFile ) throws Exception {
		
		this.sendCommand("SIZE " + remoteFile);
		if ( this.getReplyCode() == FTPReply.CODE_213 )
			return Long.parseLong( trimResponseCode( this.getReplyString()) );
		else
			return -1L;
	}
	
	/**
	 * trim the response code at the beginning
	 * @param response
	 * @return the response string without response code
	 * @throws Exception
	 */
	private String trimResponseCode(String response) throws Exception
	{
		if (response.length() < 5) return response;
		return response.substring(4).trim();
	}
	
	/**
	 * Retrieves a named file from the ftp server.
	 *
	 * @param localFile The name of the local file.
	 * @param remoteFile The name of the remote file.
	 * @exception Exception
	 * @see #getFile( String, String )
	 */
	public boolean get(String remoteFile, String localFile) throws Exception {
		FileOutputStream out = null;
		boolean rc = false;
		try {
			out = new FileOutputStream( localFile );
			rc = retrieveFile( remoteFile, out );
			return rc;
		}
		catch (IOException e) {
			throw e;
		}
		finally {
			try { if ( out != null) out.close(); } catch (Exception e) {}
		}
	} // get
	
	
	/**
	 * Retrieves a named file from the ftp server.
	 *
	 * @param localFile The name of the local file.
	 * @param remoteFile The name of the remote file.
	 * @return  The total number of bytes retrieved.
	 * @see #get( String, String )
	 * @exception Exception
	 */
	public long getFile(String remoteFile, String localFile) throws Exception {
		return getFile(remoteFile, localFile, false);
	}
	
	/**
	 * Retrieves a named file from the ftp server.
	 *
	 * @param localFile The name of the local file.
	 * @param remoteFile The name of the remote file.
	 * @param append Appends the remote file to the local file.
	 * @return  The total number of bytes retrieved.
	 * @see #get( String, String )
	 * @exception Exception
	 */
	public long getFile(String remoteFile, String localFile, boolean append) throws Exception {
		InputStream in = null;
		OutputStream out = null;
		long totalBytes = 0;
		try {
			in = retrieveFileStream(remoteFile ); 
			//boolean test = retrieveFile(remoteFile, new FileOutputStream( new File(localFile+"test"), append ));						
		 	
			if ( in == null) {				
				throw new Exception ("Could not open stream for "+remoteFile+". Perhaps the file does not exist. Reply from ftp server: "+getReplyString());
			}
			
			if (getReplyCode() > 300) {
				throw new Exception("..error occurred in getFile() [retrieveFileStream] on the FTP server for file [" + remoteFile+ "]: "  + getReplyString());
			}
			
			byte[] buffer = new byte[4096];
			out = new FileOutputStream( new File(localFile), append );
			int bytes_read = 0;
			synchronized (this) {
				while ( (bytes_read = in.read(buffer)) != -1) {
					out.write(buffer, 0, bytes_read);
					out.flush();
					totalBytes += bytes_read;
				}
			}
			in.close(); // we must close the stream: connection hang up :-(
			out.close();
			if(!completePendingCommand()) {				
				logout();
				disconnect();
				throw (new Exception("File transfer failed."));
			}
			if (getReplyCode() > 300) {
				throw new Exception("..error occurred in getFile() on the FTP server for file [" + remoteFile+ "]: "  + getReplyString());
			}
			if ( totalBytes > 0 )
				return totalBytes;
			else 
				return -1L;
		} catch( IOException e ) {
			throw e;
		}
		finally {
			try { if ( in != null) in.close(); } catch (Exception e) {}
			try { out.flush(); } catch (Exception e) {}
			try { if ( out != null)  out.close(); } catch (Exception e) {}
		}
	}
	
	/**
	 * Stores a file on the server using the given name.
	 * @param localFile The name of the local file.
	 * @param remoteFile The name of the remote file.
	 * @return True if successfully completed, false if not.
	 * @exception Exception
	 * @see #putFile( String, String )
	 */
	public boolean put(String localFile, String remoteFile) throws Exception {
		FileInputStream in = null;
		boolean rc = false;
		try {
			in = new FileInputStream( localFile );
			rc = storeFile( remoteFile, in);
			return rc;
		} catch( Exception e ) {
			throw e;
		}
		finally {
			try { if ( in != null) in.close(); } catch (Exception e) {}
		}
	}
	
	
	/**
	 * Stores a file on the server using the given name.
	 * @param localFile The name of the local file.
	 * @param remoteFile The name of the remote file.
	 * @return The total number of bytes written.
	 * @exception Exception
	 * @see #put( String, String )
	 */
	public long putFile(String localFile, String remoteFile) throws Exception {
		java.io.OutputStream outputStream = storeFileStream(remoteFile);
		if (getReplyCode() > 300) {
			throw new Exception("..error occurred in storeFileStream() on the FTP server for file [" + remoteFile+ "]: "  + getReplyString());
		} 
		return putFile(localFile,outputStream); 
	} // putFile
	
	
	/**
	 *
	 * @param localfile The name of the local file.
	 * @param an OutputStream through which data can be 
	 * written to store a file on the server using the given name.
	 * @return The total number of bytes written.
	 * @exception Exception
	 */
	public long putFile(String localFile, OutputStream out) throws Exception {
		FileInputStream in = null;
		long totalBytes  = 0;
		try {
			
			if ( out == null)
				throw (new Exception("output stream has null value."));
			
			byte[] buffer = new byte[4096];
			in = new FileInputStream( new File(localFile));
			int bytesWritten;
			synchronized (this) {
				while ( (bytesWritten = in.read(buffer)) != -1) {
					out.write(buffer, 0, bytesWritten);
					totalBytes += bytesWritten;
				}
			}
			
			in.close(); // we must close the stream: connection hang up :-(
			out.close();
			if(!completePendingCommand()) {
				logout();
				disconnect();
				throw (new Exception("File transfer failed."));
			}
			if (getReplyCode() > 300) {
				throw new Exception("..error occurred in putFile() on the FTP server for file [" + localFile+ "]: "  + getReplyString());
			}
			return totalBytes;
		} catch( Exception e ) {
			throw e;
		}		
		finally {
			try { if ( in != null) in.close(); } catch (Exception e) {}
			try { out.flush(); } catch (Exception e) {}
			try { if ( out != null) out.close(); } catch (Exception e) {}
		}
	} // putFile
	
	
	/**
	 * append a local file to the remote one on the server
	 * @param localFile The name of the local file.
	 * @param remoteFile The name of the remote file.
	 * @return The total number of bytes appended.
	 * @exception Exception
	 * @see #put( String, String )
	 * @see #putFile( String, String )
	 */
	
	public long appendFile(String localFile, String remoteFile) throws Exception {
		return putFile(localFile,appendFileStream( remoteFile ));
	} // appendFile
	
	
	
	/**
	 * Using ASCII mode for file transfers
	 * @return True if successfully completed, false if not.
	 * @throws IOException If an I/O error occurs while either sending a
	 * command to the server or receiving a reply from the server.
	 */
	public boolean ascii() throws IOException {
		return setFileType( FTP.ASCII_FILE_TYPE);
	}
	
	/**
	 * Using Binary mode for file transfers
	 * @return True if successfully completed, false if not.
	 * @throws IOException If an I/O error occurs while either sending a
	 * command to the server or receiving a reply from the server.
	 */
	public boolean binary() throws IOException {
		return setFileType( FTP.BINARY_FILE_TYPE);
	}
	
	/**
	 *
	 * @param directory The new working directory.
	 * @return The reply code received from the server.
	 * @throws IOException If an I/O error occurs while either sending a
	 * command to the server or receiving a reply from the server.
	 */
	public int cd(String directory) throws IOException {
		return cwd( directory );
	}
	
	/**
	 * Deletes a file on the FTP server.
	 * @param The pathname of the file to be deleted.
	 * @return True if successfully completed, false if not.
	 * @throws IOException If an I/O error occurs while either sending a
	 * command to the server or receiving a reply from the server.
	 */
	public boolean delete(String pathname)
	throws IOException {
		return deleteFile(pathname);
	}

	/*public void setProxyHost(String proxyHost_) {
		System.getProperties().put( "socksProxyHost", (proxyHost != null ? proxyHost : ""));
		proxyHost = proxyHost_;
	}
	
	public String getProxyHost() {
		//System.getProperties().put( "socksProxyHost", proxyHost);
		return proxyHost;
	}
	
	public void setProxyPort(int proxyPort_)  {		
		System.getProperties().put( "socksProxyPort", String.valueOf(proxyPort));
		proxyPort = proxyPort_;
	}

	public int getProxyPort()  {		
		 //System.getProperties().put( "socksProxyPort", String.valueOf(proxyPort));
		return proxyPort;
	}
	
	public void setProxyUser(String user) {

	}

	public void setProxyPassword(String password) {
		//setSocketFactory
		
	}
*/
	/*private void initProxy()  {

		if ( System.getProperty("socksProxyHost") != null && System.getProperty("socksProxyHost").length() > 0 )
			setProxyHost(System.getProperty("socksProxyHost"));

		if ( System.getProperty("socksProxyPort") != null && System.getProperty("socksProxyPort").length() > 0) {
			try {
				this.setProxyPort( Integer.parseInt(System.getProperty("socksProxyPort")));
			} catch (Exception ex) {
				throw new NumberFormatException("Non-numeric value is set [socksProxyPort]: " + System.getProperty("socksProxyPort"));
			}
		}
	}
*/	
	/**
	 * Testprogram
	 * @param args
	 */
	public static void main(String[] args) throws Exception{
		SOSFTP sosftp =null;
		try {
			//SOSFTP sosftp = new SOSFTP("wilma", 21);
			//sosftp.login("test", "12345");
			//System.getProperties().put("socksProxySet", "true");
			/*System.getProperties().put("ftp.proxySet", "true");			
			System.getProperties().put("socksProxyHost", "proxy.sos");
			System.getProperties().put("socksProxyPort", "1080");
			System.getProperties().put("socksProxyUserName", "sos");
			System.getProperties().put("socksProxyPassword", "sos");
*/
			
			/*HttpClient client = new HttpClient();
			client.getHostConfiguration().setProxy(proxyHost, port);
			*/
			
			/*System.getProperties().setProperty("proxyHost", "proxy.sos");
			System.getProperties().setProperty("proxyPort","3128" );
			System.getProperties().setProperty("proxySet", "true");
			*/
/*
			System.getProperties().setProperty("http.proxyHost", "proxy.sos");
			System.getProperties().setProperty("http.proxyPort","3128" );
			System.getProperties().setProperty("http.proxySet", "true");
*/

			//sosftp = new SOSFTP("WILMA", 21, "", 123);
			sosftp = new SOSFTP("www.sos-berlin.com", 21);
			
			
			
			//sosftp = new SOSFTP("192.11.0.61", 21);
			sosftp.login("anonymous", "info@sos-berlin.com");
			sosftp.nList("/home/test/temp");
			
			System.out.println("~~~~~~~~~~~~~Test mit nlist~~~~~~~~~~~~~~~~~~~");
			/*sosftp = new SOSFTP("WILMA", 21, "", 123);
			//sosftp.login("test", "12345");
			
			sosftp.login("test", "12345");
		
			
			long s1 = System.currentTimeMillis();
			Vector va = sosftp.nList("/home/test/temp");
			long e1 = System.currentTimeMillis();
			long r1 = e1-s1;
			
			//System.out.println("1 is :" + r1);
			//for(int i = 0; i < va.size(); i++) {				
			//	System.out.println("VA File: " + va.get(i));				
			//}
			
			long e11 = System.currentTimeMillis();
			long r11 = e11-s1;
			System.out.println("--> :" + r11);
			*/
			
			
			System.out.println("~~~~~~~~~~~~~Test mit FTPFILE~~~~~~~~~~~~~~~~~~~");
			long s2 = System.currentTimeMillis();
			FTPFile[] v = sosftp.listFiles("/home/test/temp");
			long e2 = System.currentTimeMillis();
			long r2 = e2 - s2;
			/*System.out.println("2 is :" + r2);
			for(int i = 0; i < v.length; i++) {
				if(v[i].isFile())
					System.out.println("File: "+ v[i].getName());				
			}
			*/
			long e22 = System.currentTimeMillis();
			long r22 = e22-s2;
				
			System.out.println("--> :" + r22);
			
			
			
			
			/*
			Vector v = sosftp.dir("/home/test/temp", 1);
			for(int i = 0; i < v.size(); i++)
				System.out.println(i + " " + v.get(i));
			*/
			
	/*		Vector v1 = sosftp.getFilenames();
			for(int i = 0; i < v1.size() && i < 10; i++)
				System.out.println(i + " " + v1.get(i));
		*/	
		} catch (Exception e) {
			//System.err.println(e.toString());
			e.printStackTrace();
		} finally {
			if(sosftp != null ) sosftp.disconnect();
		}
	}
	
}

