/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.stresstest;


import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;

import java.util.Iterator;
import java.util.LinkedHashMap;

import sos.connection.SOSConnection;


/**
 * @author Ron - Gerrit Vahle <rv@sos-berlin.com>
 * @version 1.0
 * @author Andreas Pschel  <andreas.pueschel@sos-berlin.com>
 * @version 1.1
 * @since 2004-11-13
 *
 *SOSStresstestSession liest SQL-Statements aus den Eingabedateien aus und sendet diese an die
 *Datenbank. Jede Eingabedatei wird als Anweisungskette verarbeitet, was bedeutet, das
 *nicht die Dauer jeder einzelnen Anweisung protokolliert wird, sondern die Ausfhrungszeit
 *der gesamten Eingabedatei.
 *Aufgerufen wird SOSStresstestSession vom StresstestThread ,der ber den Klassennamen auf
 *die SOSStresstestSession als Instanz der Basisklasse StresstestSession zugreift.
 *
 */

public class SOSStresstestSession extends StresstestSession 
{
    String _param = null;
    String _execute = null;
    String _input_file = null;
    String _current_file = null; 
    SOSConnection _connection = null;
    private static long _current_value = 0;
    private long _start;
    private long _stop;
    
    /**
     * Konstruktor
     * 
     * @param stresstest_thread Thread, der die Session instantiiert
     */
    public SOSStresstestSession( StresstestThread stresstest_thread ) throws Exception 
    {
        super( stresstest_thread );
        _param = stresstest_thread.get_stresstest().get_param();
        _execute = stresstest_thread.get_stresstest().get_execute();
        _input_file = stresstest_thread.get_stresstest().get_input_file();
        _connection = stresstest_thread.getConnection();
        _start = Stresstest._start;
        _stop = Stresstest._stop;       
    }
    
    
    /**
     * Destruktor
     */
    public void destruct()
    {
    }
    

	/** 
	 * Liefert den Wert des Parameters "param" zurck
	 * @return _param Wert des Parameters "param" 
	 */
	public String get_param() {
		return _param;
	}
	
	/** 
	 * Liefert den Wert des Parameters "execute" zurck
	 * @return _execute Wert des Parameters "execute" 
	 */
	public String get_execute() {
		return _execute;
	}
	
	/** 
	 * Liefert den Namen der aktuellen Eingabedatei zurck
	 * @return _current_file Name der Eingabedatei
	 */
	public String get_current_file() {
		return _current_file;
	}
	
	
	/**
	 * Setzt den Namen der aktuellen Eingabedatei
	 * @param _current_file Name der Eingabedatei
	 */
	public void set_current_file(String _current_file) {
		this._current_file = _current_file;
	}

    
	/**
	 * Liefert den aktuellen Wert des Zhlers zurck, dessen Wertebereich mit den Startparametern -start und -stop vereinbart wurde
	 * @return aktueller Wert des Zhlers.
	 */
	public static long get_current_value() {
		
		return Stresstest.get_current_value();
	}

	/**
	 * Liefert den inkrementierten Wert des Zhlers zurck, dessen Wertebereich mit den Startparametern -start und -stop vereinbart wurde.
	 * Ist der Wert des inkrementierten Zhlers grer als der beim Startparameter -stop angegebene, dann wird der Zhler auf 0 gesetzt 
	 * bzw. auf den Wert des Startparameters -start sofern vorhanden.  
	 * @return inkrementierter Wert des Zhlers oder Startwert.
	 */
	public static long get_next_value() {
		
		return Stresstest.get_next_value();
	}
	
	/**
     * Liest die SQL-Anweisungen aus der Eingabedatei aus und erzeugt eine Job-Kette aus der Eingabedateie 
     */
	public void get_statements() throws Exception {
		
    	LinkedHashMap test = new LinkedHashMap();
    	String data  = "";
		String datas = null;
		
		try
		{
			File file = new File(get_current_file());
			if(!file.exists()){
				throw new Exception("file not found: " + get_current_file());
			}
			FileInputStream fstream = new FileInputStream(get_current_file());
			DataInputStream in = new DataInputStream(fstream);

			while (in.available() != 0)
			{
				datas = in.readLine();
				if(datas.startsWith("#") || datas.startsWith(";")) {					
				} else {
					if (datas.length() == 0) {
						if (data.length() == 0) {
						} else {
							test.put(new Integer(test.size()), data);
							data = "";
						}
					} else {
						data = data +" "+ datas;
					}
				}
			}
			if(data.length() != 0 ){
				test.put(new Integer(test.size()), data);
			}
			in.close();
		} 
		catch (Exception e){
			throw new Exception("get_statements: " + e.getMessage());
		}

		createChain(test);
    }
    
	
    /**
     * Arbeitet die SQL-Anweisungen ab
     * @param hash HashMap mit den SQL-Statements
     */
    private void createChain(LinkedHashMap hash) throws Exception{
    	
        LinkedHashMap chain = hash;
        Iterator it = chain.values().iterator();
        String stmt = "";
       
        try{
        	while (it.hasNext()) {
        		stmt = it.next().toString();
        		stmt = stmt.trim();

        		if (stmt.indexOf("<%=nextval%>") != -1) {
        			stmt = stmt.replaceAll("<%=nextval%>", "" + get_next_value());
        		}
        	
        		if(stmt.indexOf("<%=curval%>") !=0 ) {
        			stmt = stmt.replaceAll("<%=curval%>", "" + get_current_value());
        		}  
        	
        		if(stmt.toLowerCase().startsWith("select")){
        			if (this.getConnection() != null) this.getConnection().executeQuery(stmt);
        		} else {
        			if (this.getConnection() != null) this.getConnection().execute(stmt);
        		}	
        	}
        } catch (Exception e) {
        	throw new Exception(e.getMessage() + ", from file: " + _current_file + "  in statement: " + stmt);
        }
    }
    
    /**
     * Fhrt eine Aktionskette aus
     */
    
    public void executeChain()throws Exception {
    	
    	if (_input_file !=null && _input_file.length() > 0){	
    		String[] _input_files = _input_file.split("[+]");
    		for(int i=0; i<_input_files.length; i++){
    			set_current_file(_input_files[i]);
  	  			execute( "get_statements", _input_files[i]);
    		}
    	}
    	if (this.getConnection() != null) getConnection().rollback();
    }
    
}
