/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.stresstest;

import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Vector;

import sos.connection.SOSConnection;
import sos.util.SOSDate;

/**
 * @author Ron - Gerrit Vahle  <rv@sos-berlin.com>
 * @version 1.0
 * @author Andreas Pschel  <andreas.pueschel@sos-berlin.com>
 * @version 1.1
 * @since 2004-11-13
 *
 *StresstestReport liest die letzten Ergebnisse der Tests,die mit StresstestImport
 *in eine Datenbank importiert worden sind und vergleicht die Ergebnisse miteinander.
 *Wieviele verglichen werden ist ber den Parameter -compare= steuerbar.
 *Man kann sich alle ausgefhrten Funktionen der letzten Tests ansehen, 
 *sowie sich spezielle Funktionen herausgreifen
 *
 */

public class StresstestReport {
  
    private static String _configuration_file;
    private static SOSConnection _connection;
    private int _last_tests;
    private String _function; 
    private float[] _changes = new float[6];
    private Vector _results = null;
    private Hashtable _valueChanges = new Hashtable();
    private String space = "";
    
    
    /**
     *	Konstruktor - Stellt bei Aufruf die Verbindung zur Datenbank her
     *
     * @param configuration_file Name einer Konfigurationsdatei mit Parametern zum Datenbankverbindungsaufbau 
     * @param compare Anzahl zurkliegender Testreihen, die bercksichtigt werden sollen
     * @param functions Funktion die ausgewertet werden soll. Bei mehreren Funktionen mssen deren Namen durch + getrennt werden
     * @throws Exception falls die Verbindung mit der Datenbank fehl schlgt
     */

    public StresstestReport(String configuration_file, int compare, String functions)throws Exception{
		_configuration_file = configuration_file;
		_last_tests = compare;
		_function = functions;
		_connection = SOSConnection.createInstance(_configuration_file);
		_connection.connect();
	}
    
	
	/**
	 * Liefert die aktuell bestehende Datenbankverbindung
	 * @return die aktuelle Datenbankverbindung
	 */
	private static SOSConnection get_connection() {
	    return _connection;
	}


    /**
	 * Formatiert die Ausgabe
     * @param in unformatierte Ausgabe  
     * @param maxlength Grsste Lnge eines Eingabestrings der selben Art
     * @return der formartierte String
     */
    private String getSpaces(String in, int maxlength) {
    	
    	String in_spaces = in;
    	if(in_spaces.length() > 11){
    		space = in_spaces.substring(11, in_spaces.length());
    		in_spaces = in_spaces.substring(0, 11) + " ";
    	}else{
    		for(int i=0; i<maxlength-in.length()+2; i++){
    			in_spaces = in_spaces + " ";
    		}
    	}
     	return in_spaces +"|";
    }

    /**
     * Formatiert die Ausgabe
     * @param in unformatierte Ausgabe
     * @param field_range die fr die Ausgabe zur Verfgung stehende Feldgrsse
     * @return der formatierte String
     */
    private String getSpacesN(String in, int field_range) {
    	
    	String _temp = in.trim();
    	int i = _temp.indexOf(".");
    	if( i!= -1){
    		_temp = _temp.substring(0,i+2);
    	}
    	while(_temp.length() < field_range){
       		_temp = " "+_temp;
       		}
    	return _temp+" |";
    }
    
    /** Errechnet die max. Lnge eines Feldes der Ausgabe
     * @param field zu berechnendes Feld der Ausgabe
     * @return max. Lnge des Feldes in der Ausgabe
     */
    private int checkLength(String field) {
    
    	int longest = 0; 
    	for(int i = 0; i<this._results.size();i++){
			HashMap _result = (HashMap)this._results.elementAt(i);
			String _helper = _result.get(field).toString(); 
			if(_helper.length()>longest)longest = _helper.length();
    	}
    	return longest;
    }

    /** Errechnet die Anzahl der Threads, die beim Stresstest verwendet wurden
     * @param id Angabe aus welcher Testreihe die Threadanzahl bestimmt werden soll 
     * @return Anzahl der Threads
     * @throws Exception falls Aktion fehl schlgt
     */
    private int getThreadCount(String id) throws Exception {
    	
    	Integer _temp = new Integer(get_connection().getSingleValue("SELECT MAX(\"THREAD_ID\") FROM STRESSTEST WHERE \"ID\" = %timestamp_iso('"+ id +"')"));
    	return _temp.intValue();
	}
    
	private String idFormat(String id) throws Exception {
		
    	SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		return SOSDate.getTimeAsString( dateFormat.parse(id));
	}
	
    /** Bestimmt die nderungen zwischen den einzelnen Testreihen
     * @param min Minimum des Tests
     * @param avg Durchschnittswert des Tests
     * @param max Maximum des Tests
     * @param fun zugehrige Funktion 
     */
    private void calculateChanges(String min, String avg, String max, String fun) {
    	
        String _help = min+"+"+max+"+"+avg;
    	Float _min1 = new Float(min);
    	Float _max1 = new Float(max);
    	Float _avg1 = new Float(avg);
    	String _helpChange = "";
       		 if(_valueChanges.containsKey(fun)){
       		  	_helpChange = (String)_valueChanges.get(fun);
    			String[] _valueChange = _helpChange.split("[+]");
    			Float _temp1_ = new Float(_valueChange[0]);
    			Float _temp2_ = new Float(_valueChange[1]);
    			Float _temp3_ = new Float(_valueChange[2]);
				//Prozentwert = Grundwert * (Prozentsatz/100)
				_changes[0] = _min1.floatValue() - _temp1_.floatValue();
				_changes[1] = _max1.floatValue() - _temp2_.floatValue();
				_changes[2] =  _avg1.floatValue() - _temp3_.floatValue();
				_changes[3] = (_temp1_.floatValue() == 0) ? 0 : ((_min1.floatValue() /_temp1_.floatValue())* 100)-100;
				_changes[4] = ((_max1.floatValue() /_temp2_.floatValue())* 100)-100;
				_changes[5] = ((_avg1.floatValue() /_temp3_.floatValue())* 100)-100;
				_valueChanges.remove(fun);
       			_valueChanges.put(fun,_help);
   		  	}else{
   		  		for(int j = 0;j < 6;j++){
   		  			_changes[j] =0;
	  			}
      		  	_valueChanges.put(fun,_help);
       		 }
		}
    
	
    /** Liest die Testergebnisse der gewnschten Funktionen aus der Datenbank
     * @throws Exception falls Aktion fehl schlgt
     */
    private void getLastTestsByName() throws Exception {
     
     String[] _functions = _function.split("[+]");
     for(int j = 0 ; j< _functions.length ; j++) {
     	_results = get_connection().getArrayAsVector("SELECT \"ID\", \"FUNCTION\", COUNT(*) AS \"CALLS\", MIN(\"DURATION\") AS \"MIN_DURATION\", AVG(\"DURATION\") AS \"AVG_DURATION\", MAX(\"DURATION\") AS \"MAX_DURATION\""
	  			+ " FROM STRESSTEST "
				+ " WHERE \"REPLAY\" > 0 AND \"FUNCTION\" = '" +_functions[j]+"'"
				+ " GROUP BY \"ID\",\"FUNCTION\" "
				+ " ORDER BY \"ID\" DESC");
     	printer();
     }
    }
    

    /** Holt die Testergebnisse aller Funktionen aus der Datenbank
     * @throws Exception falls Aktion fehl schlgt
     */
    public void getLastTestsAll() throws Exception {

   	 _results = get_connection().getArrayAsVector("SELECT \"ID\", \"FUNCTION\", COUNT(*) AS \"CALLS\", MIN(\"DURATION\") AS \"MIN_DURATION\", AVG(\"DURATION\") AS \"AVG_DURATION\", MAX(\"DURATION\") AS \"MAX_DURATION\""
	    		+ " FROM STRESSTEST "
				+ " WHERE \"REPLAY\" > 0 "
				+ " GROUP BY \"ID\",\"FUNCTION\" "
				+ " ORDER BY \"ID\" DESC");	
   	 printer();
    }


    /** Gibt die Ergebnisse nach stdout aus
     * @throws Exception falls Aktion fehl schlgt
     */
    public void printer()throws Exception {
    	
          	System.out.println();
    		int _max_distinct_records = _last_tests;
	    	int _cur_distinct_records = 0;
	   	    int _thread_count = 0;
			
	    	String _indent  = "";
	    	String _last_id = new String("");
	    	System.out.println();
	   	    String topic = "| time                | function    |  min ms |  max ms |  avg ms | calls | threads |  +- min |  +- max |  +- avg | % min | % max | % avg |";
	   	    String barrier = "";
	   	    for(int j=0; j<topic.length(); j++) { barrier = barrier + "-"; }
	   	    System.out.println(barrier);
	    	System.out.println(topic);
	    	System.out.println(barrier);
	    	
	    	for(int i = 0; i<_results.size();i++){
				HashMap _result = (HashMap)_results.elementAt(i);
				if (_last_id.equals(_result.get("id").toString()) ) {
			    	_indent = "";
				} else {
					_indent =  _result.get("id").toString();
					_last_id =  _result.get("id").toString();
			    	_cur_distinct_records++;
			   		if( _cur_distinct_records > _max_distinct_records ) break;
					}
				if(_indent.equals("")){
					this.calculateChanges(_result.get("min_duration").toString(),_result.get("avg_duration").toString(), _result.get("max_duration").toString(),_result.get("function").toString());
					Double help = new Double(_result.get("avg_duration").toString());
					System.out.println("|"+getSpacesN("",20)+" "
							+ getSpaces( _result.get("function").toString(),10)  
							+ getSpacesN(_result.get("min_duration").toString(),8)
							+ getSpacesN(_result.get("max_duration").toString(),8) 
							+ getSpacesN(""+help.doubleValue(),8)
							+ getSpacesN(_result.get("calls").toString(),6) 
							+ getSpacesN(""+_thread_count,8)
							+ getSpacesN(""+_changes[0],8)
							+ getSpacesN(""+_changes[1],8)
							+ getSpacesN(""+_changes[2],8)
							+ getSpacesN(""+_changes[3],6)
							+ getSpacesN(""+_changes[4],6)
							+ getSpacesN(""+_changes[5],6)
					);
					if(space != ""){
						System.out.println(getSpacesN("", 21) + " " + getSpaces(space, 10));
					}
				}
				else{
					// System.out.println();
					_indent = this.idFormat(_indent);
					_thread_count = getThreadCount(_indent)+1;
					this.calculateChanges(_result.get("min_duration").toString(),_result.get("avg_duration").toString(), _result.get("max_duration").toString(),_result.get("function").toString());
					Double help = new Double(_result.get("avg_duration").toString());
					System.out.println("| " +_indent+" | " 
							+ getSpaces(_result.get("function").toString(),10)  
							+ getSpacesN(_result.get("min_duration").toString(),8) 
							+ getSpacesN(_result.get("max_duration").toString(),8) 
							+ getSpacesN(""+help.doubleValue(),8)
							+ getSpacesN(_result.get("calls").toString(),6) 
							+ getSpacesN(""+_thread_count,8)
							+ getSpacesN(""+_changes[0],8)
							+ getSpacesN(""+_changes[1],8)
							+ getSpacesN(""+_changes[2],8)
							+ getSpacesN(""+_changes[3],6)
							+ getSpacesN(""+_changes[4],6)
							+ getSpacesN(""+_changes[5],6)
					 );
					if(space != ""){
						System.out.println(getSpacesN("", 21) + " " + getSpaces(space, 10));
					}
				}
	    	}
	    	System.out.println(barrier);
	    	System.out.println();
    }
 
   /** Entscheidet anhand des -=functions= Parameters darber, ob nach allen Funktionen gesucht werden soll oder nur nach spezifischen
 	* @throws Exception falls Aktion fehl schlgt
 	*/
	private void printReport() throws Exception {
			      		
		if(_function.equals("")){
      		getLastTestsAll();
   		} else {
   			getLastTestsByName();
      	}
   	}
//--------------------------------------------------------------------------------Main	
	/**
	 * main.
	 * <br/>
	 * Aufruf:
	 * 
	 * <p>
	 * <b>-config=</b>Konfigurationsdatei
	 * <br/>
	 * Ort der und Name der Konfigurationsdatei mit den Einzelheiten der Datenbankverbindung
	 * Standard: config/sos_setting.ini
	 * 
	 * <p>
	 * <b>-functions=</b>Funktionen
	 * <br/>
	 * Funktionen die mit Vorgngerversion verglichen werden sollen
	 * Standard: Alle Funktionen der letzten Testreihe
	 * <p>
	 * 
	 * <b>-compare=</b>Vergleiche
	 * <br/>
	 * Anzahl der vorhergehenden Testreihen, die miteinander verglichen werden sollen
	 * Standard: 1 
	 * 
	 * Beispiel Unix-Startscript:
	 * <br/>
	 * java -cp $CLASSPATH:./lib/msbase.jar:./lib/mssqlserver.jar:./lib/msutil.jar:./lib/mysql-connector-java-3.0.15-ga-bin.jar:./lib/ojdbc14.jar:./lib/sos.connection.jar:./lib/sos.util.jar:./lib/sos.stresstest.jar  sos.stresstest.StresstestReport -compare=3 -functions=stress1.sql+stress2.sql
	 */
	public static void main(String[] args)throws Exception {

		Arguments arguments = new Arguments( args );

		StresstestReport report = new StresstestReport
        ( 
                new String(      arguments.as_string( "-config=",    "config/sos_settings.ini" ) ),
                				 arguments.as_int   ( "-compare=",   1),
				new String(      arguments.as_string( "-functions=", ""))
        );
		
		arguments.check_all_used();
		report.printReport();
		_connection.disconnect();
	}

}
