/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
/*
 * Created on 16.12.2004
 * 
 * Ein Checkpoint in einem Take. Fhrt alle Steps des Checkpoints aus. Fhrt
 * Methoden der Checkklasse aus Prft HTML-Checks
 *  
 */
package sos.stresstest.dialogtest;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.w3c.dom.CDATASection;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import sos.util.SOSClassUtil;
import sos.util.SOSLogger;
import java.net.URL;

/**
 * @author ur
 */
public class DialogCheckpoint extends DialogTesterBasic {
	
	private String name = "";
	
	//private ArrayList listOfSteps; // Enthlt die Steps des Checkpoints
	
	private Node node;
	
	private ArrayList listOfDialogSteps;
	
	private DialogTester caller;
	
	private String take;
	
	private ArrayList listOfChecks;	
	
	protected DialogCheckpoint(SOSLogger sosLogger_, String name_) {
		
		name = name_;
		sosLogger = sosLogger_;
		listOfDialogSteps = new ArrayList();
		listOfChecks = new ArrayList();
	}
	
	private void addCheck(DialogTesterChecks s) throws Exception {
		
		try {
			
			s.setUsed(true);
			listOfChecks.add(s);
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
	}
	
	private String invokeMethod(Object checkClassObject, String methodName) throws Exception {
		
		if (!caller.isEnableChecks() || checkClassObject == null) return null;
		
		methodName = methodName.toLowerCase();
		methodName = methodName.replace(' ', '_');
		
		try {
			
			Method method = checkClassObject.getClass().getMethod(methodName, null);
			
			String returnType = method.getReturnType().getName();
			
			if (!returnType.equals("java.lang.String")) {
				log_warn("method [" + methodName + "] must return a String but returns " + returnType);
			} else {
				Object returnValue = method.invoke(checkClassObject, null);
				if (returnValue != null && !returnValue.equals("")) {
					log_warn(methodName + " returns:" + returnValue);
					return returnValue.toString();
				}
			}
			
		} catch (NoSuchMethodException e) {
			log_debug5("method " + methodName + " is not implemented in " + checkClassObject.getClass());
			// hier darf keine Exception geworfen werden
		} catch (IllegalArgumentException e) {
			log_warn("method " + methodName + " must called without any parameters");
			throw new Exception("method " + methodName + " must called without any parameters");
		} catch (IllegalAccessException e) {
			log_warn("object of class " + checkClassObject.getClass() + " has no access to method "+ methodName);
			throw new Exception("object of class " + checkClassObject.getClass() + " has no access to method "+ methodName);
		} catch (InvocationTargetException e) {
			log_warn("method " + methodName + "-->" + e);
			throw new Exception("method " + methodName + "-->" + e);
		}
		
		return null;
	}
	
	
	private void addDialogStep(int nr, String request_method, HashMap r, HashMap h, HashMap d) throws Exception {
		
		DialogTesterTags tag = null;
		//DialogTesterChecks check = null;
		String k;
		String v;
		
		try {
			
			if ( !caller.isSession_clear() ) {
				//  r.put("clear_session","1"); // Die Session wird gelscht.
				caller.setSession_clear(true);
			}
			
			if ( !caller.isList() ) {
				
				log_debug5("Changing Tag-Values....");
				for (int i = 0; i < caller.getTagValues().size(); i++) {
					
					Iterator requestKeys = r.keySet().iterator();
					Iterator requestVals = r.values().iterator();
					
					tag = (DialogTesterTags) caller.getTagValues().get(i);
					tag.setGlobals( caller.getGlobalVars() );
					tag.setChecksEnabled( caller.isEnableChecks() );
					
					if ( tag.isActive() ) {
						
						log_debug5("tag" + tag.getTagName() + " is activ!");
						
						while ( requestKeys.hasNext() ) {
							
							k = requestKeys.next().toString();
							v = requestVals.next().toString();
							
							/*   log_debug5( tag.getTagName() + "=" + k + "\n" +
															 tag.getNewValue()+ "<>" + r.get(k) + "\n" +
															 tag.getOldValue()+ "<>" + v + "\n" );
							*/
							
							if (tag.getTagName().equals(k) && !tag.getNewValueParsed().equals(r.get(k))
									&& (tag.getOldValue().equals(v) || tag.getOldValue().equals(""))) {
								
								if ( !tag.getNewValue().equals(tag.getNewValueParsed()) )
									log_debug4("translate \""+tag.getNewValue()+"\" to \""+tag.getNewValueParsed()+"\"");
								
								log_debug3("..Checkpoint: " + name + " --> Using  " + tag.getNewValueParsed() + " for " + k
														+ " instead of " + r.get(k));
								r.put(k, tag.getNewValueParsed());
								
								// Tags in Url ndern
								String aktUrl = d.get("url").toString();
								String oldUrl = d.get("url").toString();
								aktUrl = aktUrl.replaceAll(tag.getTagName() + "=" + v,tag.getTagName() + "=" + tag.getNewValueParsed());
								d.put("url",aktUrl);
								
								if (!oldUrl.equals(aktUrl))
									log_debug5("Url: " + oldUrl + "\n verndert nach -->\n" + aktUrl);
							}
						}
					}
				}
			}
			
			DialogStep step = new DialogStep(sosLogger, d.get("url").toString(), nr, r, h, d);
			step.setDebugLevel(getDebugLevel());
			step.setRequestMethod(request_method);
			step.setCaller(caller);
			step.setCheckpoint(this);
			listOfDialogSteps.add(step);
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
			throw new Exception(err_msg + e.getMessage());
		}
	}
	
	
	private String getTagValue(Node node) throws Exception {
		
		NodeList list = node.getChildNodes();
		Node n = null;
		StringBuffer ret = new StringBuffer();
		
		try {
			
			if (list != null) {
				
				for (int i = 0; i < list.getLength(); i++) {
					
					n = list.item(i);
					if (n.getNodeType() == Node.TEXT_NODE)
						ret.append(n.getNodeValue().trim());
					
					if (n.getNodeType() == Node.CDATA_SECTION_NODE) {
						CDATASection cs = (CDATASection) n;
						ret.append(cs.getData());
					}
				}
			}
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
		
		return ret.toString();
	}
	
	private void parseNode(Node n, HashMap h) throws Exception {
		
		Node r;
		
		try {
			
			n = n.getFirstChild();
			
			while (n != null) {
				
				if (n.hasChildNodes()) {
					r = n.getFirstChild();
					if (r.getNodeName().equals("#cdata-section")) {
						h.put(n.getNodeName(), getTagValue(n));
						log_debug5("-->" + n.getNodeName() + "=" + getTagValue(n));
					
					}
					
				} else {
					if ( !n.getNodeName().equals("#text") ) {
						h.put(n.getNodeName(), "");
						log_debug5("-->" + n.getNodeName() + "=<empty>");
					}
				}
			
				n = n.getNextSibling();
			}
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
	}
	
	private void parseDialogSteps() throws Exception {
		
		HashMap dialogstep = new HashMap();
		
		boolean add = false;
		String request_method = "";
		
		try {
			
			NodeList nl = node.getChildNodes();
			Node n, r;
			
//		Jeden Dialogstep parsieren
			for (int i = 0; i < nl.getLength(); i++) {
				
				n = nl.item(i);
				
				if (n.getAttributes() != null) {
					dialogstep.put("nr", n.getAttributes().getNamedItem("nr").getNodeValue());
					
					if (n.getAttributes().getNamedItem("request_method") == null) {
						request_method = "POST";
					} else {
						request_method = n.getAttributes().getNamedItem("request_method").getNodeValue();
					}
				
					add = true;
				}
				
				n = n.getFirstChild();
				HashMap request = new HashMap();
				HashMap header = new HashMap();
				
				while (n != null) {
					
					if (n.hasChildNodes()) {
						r = n.getFirstChild();
						
						if (r.getNodeName().equals("#cdata-section")) {
							dialogstep.put(n.getNodeName(), getTagValue(n));
							log_debug5("-->" + n.getNodeName() + "=" + getTagValue(n));
							
						} else {
							if (n.getNodeName().equals("request"))
								parseNode(n, request);
							
							if (n.getNodeName().equals("header"))
								parseNode(n, header);
						}
					}
					
					n = n.getNextSibling();
				}
				
				
				// Den Dialogstep in den Dialog aufnehmen
				if ( add && (true || header.containsKey("Referer")
						 && !header.get("Referer").toString().equalsIgnoreCase("NULL"))) {
					
					addDialogStep(Integer.parseInt(dialogstep.get("nr").toString()), request_method, request, header, dialogstep);
					add = false;
				}
			}
			
		} catch (Exception e) {
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
  		throw new Exception(err_msg + e.getMessage());
		}
	}
	
	protected boolean execute() throws Exception {
		
		int i;
		int nr = 0;
		DialogStep step = null;
		DialogTesterChecks check;
		String invokedResult = null;
		
		try {
			
			listOfDialogSteps.clear();
			
			parseDialogSteps();
			
			//Html-Checks hinzufgen.
			for (i = 0; i < caller.getHtmlCheckValues().size(); i++) {
				
				check = (DialogTesterChecks) caller.getHtmlCheckValues().get(i);
				if (check.getDialogCheckpoint().equalsIgnoreCase(name) && (check.getDialogTake().equalsIgnoreCase(take))) {
					log_debug3("Checkpoint: '" + take + "." + name + "' --> will check for  " + check.getSearchString());
					addCheck(check);
				}
			}
			
			if ( caller.isEnableChecks() ) {
				
				caller.setSosLogger( sosLogger );
	        	
	        		/*
	            if (caller.getCheckClassObject() != null) {
	                try {
	                    Class clazz = caller.getCheckClassObject().getClass();
	                    Method mtd = clazz.getMethod("setSosLogger", new Class[] { sosLogger.getClass()});
	
	                    mtd.invoke(caller.getCheckClassObject(), new Object[] { sosLogger});
	                } catch (Exception e) {
	                    log_warn("Methode setSosLogger in Klasse " + caller.getCheckClassObject().getClass().getName()
	                            + " konnte nicht gerufen werden");
	                }
	            }
							*/
			}
			
			if ( !caller.isList() ) {
				
				// ausfhren der PRE-Methode
				invokedResult = invokeMethod(caller.getCheckClassObject(), take + "_" + name + "_pre");
				
				if ( !(invokedResult==null) ) {
					String error = "Take: '"+ take + "', Checkpoint: '" + name + "' has failed in Precheck: [class "
												+ caller.getCheckClassObject().getClass().getName() + "] public String " + take + "_" + name
												+ "_pre() returns: "+invokedResult; 
					throw new Exception(error);
				}
			}
			
			if (listOfDialogSteps.size() == 1)
				log_info("      -> Starting Checkpoint '" + name + "' with one Step");
			else
				log_info("      -> Starting Checkpoint '" + name + "' with " + listOfDialogSteps.size()+ " Steps");
			
			
			for (i = 0; i < listOfDialogSteps.size(); i++) {
				
				step = (DialogStep) listOfDialogSteps.get(i);
				
				nr = step.getNr();
				
				if ( !caller.isList() ) {
					
					log_debug5("Step:" + step.getNr());
					log_debug5("Url: --> " + step.getUrl().toString());
					
					// hier wird die URL verndert, falls die ini-Eintrge url_search und url_replace gesetzt sind
					if ( caller.getURL_search()!=null ) {
						
						if ( caller.getURL_replace()==null ) {
							this.log_warn("\"url_search\" is defined as \""+caller.getURL_search()+"\" but \"url_replace\" is not defined");
							
						} else {
							String url = step.getUrl().toString();
		
							Pattern pattern = Pattern.compile( caller.getURL_search() );
							Matcher matcher = pattern.matcher(url);							
							
							if ( matcher.find() ) {
								
								// feststellen ob der Ausdruck im Laufe des Recordings gefunden wurde,
								// um spter ein Zusammenfassendes Ergebnis zu melden, falls er NIE gefunden wurde
								if ( !caller.isURL_replaced() )
									caller.setURL_replaced( true );
								
								// den Ausdruck ersetzen
								log_debug4("replace in URL \""+caller.getURL_search()+"\" with \""+caller.getURL_replace()+"\"");
								url = url.replaceAll(caller.getURL_search(), caller.getURL_replace());
								
								URL newURL = new URL(url);
								step.setUrl(newURL);
							}
						}
					}
				}
				
				if ( caller.isShowStep() ) {
					
					log_info("Url:" + step.getUrl().toString());
					log_info("nr:" + nr);
					log_info("---------------------------");
				}
				
				if ( caller.isPrintHeader() )
					step.printHeader();
				
				if ( caller.isPrintRequest() )
					step.printRequest();
				
				if ( !caller.isList() )
					
					step.execute();
				else
					log_debug4("           Step nr:" + nr);
				
				if (caller.xmlReport!=null) caller.xmlReport.resultAddStep(step.getResponseTime());
			}
			
			// html-Check und evtl. POST-Check nur wenn das Recording ausgefhrt wird
			if ( !caller.isList() ) {
				
				for (int k = 0; k < listOfChecks.size(); k++) {
					
					check = (DialogTesterChecks) listOfChecks.get(k);
					
					// expected but was not found!
					if (!check.isFound() && check.isTrueIfExists()) {
						String error = "Recording "+ caller.getNr() +", Take '" + check.getDialogTake() + "', Checkpoint '" + check.getDialogCheckpoint() + "': \""+ check.getEvalVarString() +"\" expected but was not found!"; 
						throw new Exception(error);
					}
					
					// not expected but was found!
					if (check.isFound() && !check.isTrueIfExists()) {
						String error = "Recording "+ caller.getNr() +", Take '" + check.getDialogTake() + "', Checkpoint '" + check.getDialogCheckpoint() + "': \""+ check.getEvalVarString() +"\" not expected but was found!";
						throw new Exception(error);
					}
				}
				
				// ausfhren der POST-Methode
				invokedResult = invokeMethod(caller.getCheckClassObject(), take + "_" + name + "_post");
				
				if ( !(invokedResult==null) ) {
					String error = "Checkpoint: " + take + "." + name + " has failed in Postcheck: [class "
												+ caller.getCheckClassObject().getClass().getName() + "] public String " + take + "_" + name
												+ "_pre() returns: "+invokedResult; 
					throw new Exception(error);
				}
			}
			
		} catch (Exception e) {
			
			if (caller.xmlReport!=null) caller.xmlReport.setError(e.getClass().getName(), e.getMessage());
			
			String err_msg = "";
			if (this.getDebugLevel()>2)
				err_msg = "\nerror in "+SOSClassUtil.getMethodName()+": ";
			throw new Exception(err_msg + e.getMessage());
		
		} finally {
			
			if (caller.xmlReport!=null)
				if ( caller.xmlReport.resultIsStepNotAdded(nr) ) {
					if (step != null) {
						caller.xmlReport.resultAddStep(step.getResponseTime());
					} else
						caller.xmlReport.resultAddStep(1);
				}
		}
		
		return true;
	}
	
    /**
     * @param node
     *            The node to set.
     */
    protected void setNode(Node node) {
        this.node = node;
    }

    /**
     * @param caller
     *            The caller to set.
     */
    protected void setCaller(DialogTester caller) {
        this.caller = caller;
    }

    /**
     * @param take
     *            The take to set.
     */
    protected void setTake(String take) {
        this.take = take;
    }

    /**
     * @return Returns the name.
     */
    protected String getName() {
        return name;
    }

    /**
     * @return Returns the listOfChecks.
     */
    protected ArrayList getListOfChecks() {
        return listOfChecks;
    }

    /**
     * @return Returns the take.
     */
    protected String getTake() {
        return take;
    }

}
