/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.stresstest.dialogtest;

import java.io.File;
import java.util.Date;

import sos.settings.SOSProfileSettings;
import sos.stresstest.SOSStresstestSession;
import sos.stresstest.StresstestExceptionMessage;
import sos.stresstest.StresstestThread;

import sos.util.SOSLogger;

/**
 * stresstest session implementation<br>
 * 
 * Die von SOSDialogTest zu verarbeitende ini-Datei mu dem Stresstest ber den Parameter "-param" bergeben werden.<br>
 * Bsp.: -param=sos_dialogtest.ini
 * 
 * @author Florian Schreiber <florian.schreiber@sos-berlin.com>
 * @since 2005-04-11
 * @version 1.0
 */

public class DialogtestStresstestSession  extends SOSStresstestSession {
		
		public static Date startTime = null;
		
    /**
     * Constructor
     * 
     * @param stresstest_thread Thread Identifier
     */
		public DialogtestStresstestSession( StresstestThread stresstest_thread ) throws Exception {
			
			super( stresstest_thread );
		}
    
    /**
     * Initialization on startup: method is called once per stresstest
     */
		public void init() throws Exception {
			
			startTime = new Date();
		}
    
    /**
     * runs the action chain
     */
		public void executeChain() throws Exception {
			
			try {

				execute( "dialogTest" );

			} catch (Exception e) {
				throw new Exception("..error in executeChain(): " + StresstestExceptionMessage.getExceptionMessage(e));
			}
		}
		
		/**
		 * runs the action
		 */
		public void dialogTest() throws Exception {
			
			SOSDialogTest test = new SOSDialogTest((SOSLogger)this.get_logger(), this.get_thread_index());
			
//		die bergebene ini-Datei
			String iniFile = this.get_param();
			
//		hier muss geprft werden, ob es die Datei gibt
			if ( iniFile==null || iniFile.equals("") )
				throw new Exception("no configuration file was given");
			
			File f = new File(iniFile);
			if ( !f.canRead() )
				throw new Exception("configuration file not found or not readable: " + f.getAbsolutePath());
			
			SOSProfileSettings profile = new SOSProfileSettings( f.getAbsolutePath() );
			
			test.initialize(profile);
			test.execute();
    }
}
