/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.stresstest.dialogtest;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.CDATASection;

import sos.util.SOSClassUtil;
import sos.util.SOSDate;
import sos.xml.SOSXMLValidator;


public class XMLReport {
	
	private String xmlFile = null;
	
	private DocumentBuilderFactory docFactory = null;                
	private DocumentBuilder docBuilder = null;                
	private Document report = null;
	
	private Element configSettingsElement = null;
	private ArrayList allConfigRecordings = null;
	private ArrayList allConfigRecordingNrs = null;
	
	private Element result = null;
	private long startTime = 0;
	private long endTime = 0;
	
	private Element resultRecording = null;
	private Element resultTake = null;
	private Element resultCheckpoint = null;
	
	private long recordingDuration = 0;
	private long takeDuration = 0;
	private long checkpointDuration = 0;
	private int stepCounter = 0;
	
	private Element error = null;
	private boolean errorOccurred = false;
	
	private Element parameter;
	private CDATASection cdataSection;
	private Element configRecording = null;
	
	
	private XMLReport() throws Exception {
		
		try {
			this.docFactory = DocumentBuilderFactory.newInstance();                
			this.docBuilder = this.docFactory.newDocumentBuilder();
			this.report = docBuilder.newDocument();
			
			// frhestens jetzt wird die Zeit gestoppt, automatisch,
			// kann spter mit resultmanuell berschrieben werden
			this.startTime = System.currentTimeMillis();
			
		} catch (Exception e) {
			throw new Exception("cannot instantiate XMLReport: "+e.getMessage());
		}
	}
	
	protected XMLReport(String xmlfile) throws Exception {
		
		this();
		this.xmlFile = xmlfile;
	}
	
	protected void configAddSettingsParameter(String name, String value) throws Exception {
		
		try {
		
			if ( name==null ) throw new Exception("XMLReport: cannot add settings parameter: 'name' is null");
			if ( value==null ) throw new Exception("XMLReport: cannot add settings parameter for "+name+": value is null");
			
			if ( this.configSettingsElement==null ) this.configSettingsElement = this.report.createElement("settings");
			
			parameter = this.report.createElement("parameter");
			parameter.setAttribute("name", name);
			cdataSection = this.report.createCDATASection(value);
			parameter.appendChild(cdataSection);
			
			this.configSettingsElement.appendChild(parameter);
			
		} catch(Exception e) {
			throw new Exception("error in "+SOSClassUtil.getMethodName()+": "+e.getMessage());
		}
	}
	
	// ffnet eine neues Recording-Element
	protected void configOpenRecording(int nr) throws Exception {
		
		try {
			if ( this.allConfigRecordingNrs == null ) this.allConfigRecordingNrs = new ArrayList();
			this.allConfigRecordingNrs.add(String.valueOf(nr));
			this.configRecording = this.report.createElement("recording");
		} catch(Exception e) {
			throw new Exception("error in "+SOSClassUtil.getMethodName()+": "+e.getMessage());
		}
	}
	
	// fgt einen Parameter zum aktuellen Recording-Element hinzu
	protected void configAddRecordingParameter(String name, String value) throws Exception {
		
		try {
			if ( name==null || value==null ) throw new Exception("XMLReport: null entry in configAddRecordingParameter: '"+name+"->"+value+"'");
			if ( this.configRecording==null ) throw new Exception("XMLReport: cannot add parameter - recording element is not open");
			
			parameter = this.report.createElement("parameter");
			parameter.setAttribute("name", name);
			cdataSection = this.report.createCDATASection(value);
			parameter.appendChild(cdataSection);
			
			this.configRecording.appendChild(parameter);
		} catch(Exception e) {
			throw new Exception("error in "+SOSClassUtil.getMethodName()+": "+e.getMessage());
		}
	}
	
	protected boolean configIsRecordingOpen() {
		
		return ( this.configRecording!=null );
	}
	
	// schliet das aktuelle Recording-Element
	protected void configCloseRecording() throws Exception {
		
		try {
			if ( this.configRecording != null ) {
			
				if ( this.allConfigRecordings==null ) this.allConfigRecordings = new ArrayList();
				
				this.allConfigRecordings.add(this.configRecording);
				this.configRecording = null; // wieder freigeben
			}
		} catch(Exception e) {
			throw new Exception("error in "+SOSClassUtil.getMethodName()+": "+e.getMessage());
		}
	}
	
	protected void resultOpenRecording(int nr, String inputfile, String logfile) throws Exception {
		
		try {
			if ( inputfile==null || inputfile.equals("") ) throw new Exception("XMLReport: no inputfile specified for recording");
			if ( this.result==null ) this.result = this.report.createElement("result");
			
			this.resultRecording = this.report.createElement("recording");
			this.resultRecording.setAttribute("nr", String.valueOf(nr));
			this.resultRecording.setAttribute("inputfile", inputfile);
			
			if ( logfile!=null && !logfile.equals("") )
				this.resultRecording.setAttribute("logfile", logfile);
			
			this.recordingDuration = System.currentTimeMillis();
		} catch(Exception e) {
			throw new Exception("error in "+SOSClassUtil.getMethodName()+": "+e.getMessage());
		}
	}
	
	protected void resultOpenTake(String name) throws Exception {
		
		try {
			this.resultTake = this.report.createElement("take");
			this.resultTake.setAttribute("name", name);
			this.takeDuration = System.currentTimeMillis();
		} catch(Exception e) {
			throw new Exception("error in "+SOSClassUtil.getMethodName()+": "+e.getMessage());
		}
	}
	
	protected void resultOpenCheckpoint(String name) throws Exception {
		
		try {
			this.resultCheckpoint = this.report.createElement("checkpoint");
			this.resultCheckpoint.setAttribute("name", name);
			this.checkpointDuration = System.currentTimeMillis();
		} catch(Exception e) {
			throw new Exception("error in "+SOSClassUtil.getMethodName()+": "+e.getMessage());
		}
	}
	
	protected boolean resultIsStepNotAdded(int nr) {
		
		return ( nr > this.stepCounter );
	}
	
	protected void resultAddStep(long responseTime) throws Exception {
		
		try {
			Element step = this.report.createElement("step");
			step.setAttribute("nr", String.valueOf(++this.stepCounter));
			step.setAttribute("responseTime", String.valueOf(responseTime));
			
			if ( this.error != null ) {
				step.appendChild(this.error);
				this.error = null;
			}
			
			this.resultCheckpoint.appendChild(step);
		} catch(Exception e) {
			throw new Exception("error in "+SOSClassUtil.getMethodName()+": "+e.getMessage());
		}
	}
	
	protected boolean resultIsCheckpointOpen() {
		
		return ( this.resultCheckpoint != null );
	}
	
	protected void resultCloseCheckpoint() throws Exception {
		
		try {
			if ( this.resultCheckpoint!=null && this.resultTake!=null ) {
				
				long duration = System.currentTimeMillis()-this.checkpointDuration;
				if ( !(duration>0) ) duration = 1;
				
				this.resultCheckpoint.setAttribute("duration", String.valueOf(duration));
				
				if ( this.error != null ) {
					this.resultCheckpoint.appendChild(this.error);
					this.error = null;
				}
				
				this.resultTake.appendChild(this.resultCheckpoint);
				this.stepCounter = 0;
				this.resultCheckpoint = null;
			}
		} catch(Exception e) {
			throw new Exception("error in "+SOSClassUtil.getMethodName()+": "+e.getMessage());
		}
	}
	
	protected boolean resultIsTakeOpen() {
		
		return ( this.resultTake != null );
	}
	
	protected void resultCloseTake() throws Exception {
		
		try {
			if ( this.resultTake!=null && this.resultRecording!=null ) {
				
				long duration = System.currentTimeMillis()-this.takeDuration;
				if ( !(duration>0) ) duration = 1;
				
				this.resultTake.setAttribute("duration", String.valueOf(duration));
				this.resultRecording.appendChild(this.resultTake);
				this.resultTake = null;
			}
		} catch(Exception e) {
			throw new Exception("error in "+SOSClassUtil.getMethodName()+": "+e.getMessage());
		}
	}
	
	protected boolean resultIsRecordingOpen() {
		
		return ( this.resultRecording != null );
	}
	
	protected void resultCloseRecording() throws Exception {
		
		try {
			if ( this.resultRecording != null ) {
				
				long duration = System.currentTimeMillis()-this.recordingDuration;
				if ( !(duration>0) ) duration = 1;
				
				this.resultRecording.setAttribute("duration", String.valueOf(duration));
				this.result.appendChild(this.resultRecording);
				this.resultRecording = null;
			}
		} catch(Exception e) {
			throw new Exception("error in "+SOSClassUtil.getMethodName()+": "+e.getMessage());
		}
	}
	
	protected void setError(String type, String message) throws Exception {
		
		// nur der erste Fehler
		if ( !this.errorOccurred ) {
			
			if ( type==null ) throw new Exception("XMLReport: cannot add error element: 'type' is null");
			if ( message==null ) throw new Exception("XMLReport: cannot add error element: 'message' is null");
			
			CDATASection cdata;
			
			this.error = this.report.createElement("error");
			
			Element exception = this.report.createElement("exception");
			cdata = this.report.createCDATASection(type);
			exception.appendChild(cdata);
			this.error.appendChild(exception);
			
			Element msg = this.report.createElement("message");
			cdata = this.report.createCDATASection(message);
			msg.appendChild(cdata);
			this.error.appendChild(msg);
			
			this.errorOccurred = true;
		}
	}
	
	// erzeugt die XML-Datei
	protected void createReport() throws Exception {
		
		try {
			
			if ( this.allConfigRecordingNrs==null || this.allConfigRecordings==null)
				throw new Exception("no recording config added");
				
			if ( this.allConfigRecordingNrs.size() != this.allConfigRecordings.size() )
				throw new Exception("cannot create Report: not unique recording buffer size");
			
			// sptestens jetzt wird die Zeit gestoppt
			if ( this.endTime==0 ) this.endTime = System.currentTimeMillis();
			
			Element reportElement = this.report.createElement("report");
			reportElement.setAttribute("xmlns", "http://www.sos-berlin.com/schema/dialogtest_report_v1.0");
			reportElement.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
			reportElement.setAttribute("xsi:schemaLocation", "http://www.sos-berlin.com/schema/dialogtest_report_v1.0 http://www.sos-berlin.com/schema/dialogtest_report_v1.0.xsd");
			
			Element config = this.report.createElement("config");
			
			// wenn <settings> noch nicht exisitert, wird eine leeres Element erzeugt
			// andernfalls wird das bestehende settings-Element verwendet
			if ( this.configSettingsElement==null ) this.configSettingsElement = this.report.createElement("settings");
			config.appendChild(this.configSettingsElement);
			
			for (int i=0; i<this.allConfigRecordings.size(); i++) {
				this.configRecording = (Element)this.allConfigRecordings.get(i);
				this.configRecording.setAttribute("nr", (String)this.allConfigRecordingNrs.get(i));
				config.appendChild(this.configRecording);
			}
			
			reportElement.appendChild(config);
			
			this.result.setAttribute("startTime", this.resultGetStartTime());
			this.result.setAttribute("endTime", this.resultGetEndTime());
			
			String duration = "1";
			if ( this.endTime>0 && this.startTime>0 && this.endTime>this.startTime )
				duration = (new Long(this.endTime-this.startTime)).toString();
			
			this.result.setAttribute("duration", duration);
			this.result.setAttribute("successful", ((this.errorOccurred) ? "false" : "true") );
			
			reportElement.appendChild(this.result);
			
			this.report.appendChild(reportElement);
			
	// xml-stylesheet angeben
			this.report.insertBefore(
					this.report.createProcessingInstruction(
							"xml-stylesheet", "href=\"dialogtest_report.xslt\" type=\"text/xsl\""
					),
					this.report.getDocumentElement()
			);
			
	// Dokument erzeugen
			Transformer transformer = TransformerFactory.newInstance().newTransformer();
			
			transformer.setOutputProperty(OutputKeys.METHOD, "xml");
			//transformer.setOutputProperty(OutputKeys.ENCODING, "ISO-8859-1");
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
			
			// create string from xml tree
			File outputFile = new File(this.xmlFile);
			StreamResult streamResult = new StreamResult(outputFile);
			DOMSource source = new DOMSource(this.report);
			transformer.transform(source, streamResult);
			
			// Dokument validieren, Schema URL steht im Dokument
			SOSXMLValidator.validate(this.xmlFile);
			
		/*
			StringWriter sw = new StringWriter();
      StreamResult result = new StreamResult(sw);
      DOMSource source = new DOMSource(this.report);
      trans.transform(source, result);
      
      String xmlString = sw.toString();
      System.out.println(xmlString);
		*/
			
		} catch (Exception e) {
			throw new Exception("error in "+SOSClassUtil.getMethodName()+": "+e.getMessage());
		}
	}
	
	/**
	 * @param startTime Set the current time as startTime.
	 */
	protected void setStartTime() {
		
		this.startTime = System.currentTimeMillis();
	}
	
	/**
	 * @return Returns the startTime in yyyy-MM-dd'T'HH:mm:ss format.
	 */
	private String resultGetStartTime() throws Exception {
		
		String dtime = "";
		if ( this.startTime == 0 ) throw new Exception("XMLReport: startTime is not set");
		
		try {
			Date date = new Date(this.startTime);
			dtime = SOSDate.getDateAsString(date, "yyyy-MM-dd'T'HH:mm:ss");
		} catch (Exception e) {
			throw new Exception("XMLReport: cannot evaluate startTime");
		}
		
		return dtime;
	}
	
	/**
	 * @param endTime Set the current time as endTime.
	 */
	protected void setEndTime() {
		
		this.endTime = System.currentTimeMillis();
	}
	
	/**
	 * @return Returns the endTime in yyyy-MM-dd'T'HH:mm:ss format.
	 */
	private String resultGetEndTime() throws Exception {
		
		String dtime = "";
		if ( this.endTime == 0 ) throw new Exception("XMLReport: endTime is not set");
		
		try {
			Date date = new Date(this.endTime);
			dtime = SOSDate.getDateAsString(date, "yyyy-MM-dd'T'HH:mm:ss");
		} catch (Exception e) {
			throw new Exception("XMLReport: cannot evaluate endTime");
		}
		
		return dtime;
	}
	
}
