/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// blkfile.h
// 14. 3.92                                             (c) Joacim Zschimmer

#ifndef __BLKFILE_H
#define __BLKFILE_H

#include "../file/absfile.h"


struct Blocked_file : public Abs_record_file
{
    Blocked_file () : _opened (false),
                      _opened_by_constructor (false) {}

    Blocked_file (
        const char         *filename,
        Open_mode           open_mode
    )
      : _opened (false),
        _opened_by_constructor (true)
    {
        open (filename, open_mode);
    }

    ~Blocked_file ()  {
         close (_opened_by_constructor ? close_normal : close_error);
    }

    void open (
        const char         *filename,
        Open_mode           open_mode,
        File_spec          &file_spec
    );

    void close (
        Close_mode = close_normal
    );

    void put (
        const void     *buffer,
        Record_length   size
    );

    void get (
        void           *buffer,
        Record_length   size,
        Record_length  *length
    );

    void update(
        const void     *buffer,
        Record_length   length
    );

    void set(long record_number);
    void del();

  private:
    struct Block {
        Block() : _f(0), _buffer(0) {}
        ~Block()  { delete _buffer; }

        void init( fstream*, int blksize );
        void exit();
        void create( long seek_value );
        void create_next()  { create( _seek_value + _blksize ); }
        void read( long seek_value );
        void read_next() { read( _seek_value + _blksize ); }
        void flush() { if (_is_modified)  _write(); }
        long seek_value() const { return _seek_value; }
        void set_append_mode();
        void set_block_length( uint l )  { *(uint2*)_buffer = l; }

        uint2 length() const { return *(uint2*)_buffer; }
        int end_of_block() const { return long2int( _ptr - _buffer ) == length(); };
        uint space_left() const { return long2int( _buffer + _blksize - _ptr ); }
        int fits_general(Record_length l) const { return 2 + 3 + l < _blksize; }
        int fits( Record_length l ) const { return space_left() >= 3 + l; }
        int is_append_mode() const { return _mode == append_mode; }
        void put( const void*, Record_length );
        void get( void*, Record_length, Record_length* );
        void update( const void*, Record_length );
        void del();

      private:
        void _write();
        Record_length record_length(Byte*) const;

        fstream    *_f;
        Byte       *_buffer;
        Byte       *_ptr;
        Byte       *_prev_ptr;
        int         _blksize;
        long        _seek_value;
        enum {read_mode=1, append_mode} _mode;
        Bit         _is_modified;

    };

    struct Header {
        char i_am_a_blkfile [16];   // "Blocked_file\n^Z"
        Byte version;
        Byte blksize512;

        void set_standards();
        void set_blksize(long blksize)   { blksize512 = (blksize+511) / 512; }
        long blksize() const { return blksize512 * 512; }
        int is_blocked() const { return blksize512 != 0; };
    };

    fstream     _f;
    Block       _block;
    Header      _hdr;
    enum {is_get = 1, is_set, is_put} _mode;
    Bool        _opened : 1;
    Bool        _opened_by_constructor : 1;

}; // struct Blocked_file

#endif

