/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
//#define MODULE_NAME "nullfile"
//#define AUTHOR      "Joacim Zschimmer"
//#define COPYRIGHT   "1997 SOS GmbH Berlin"


#include "precomp.h"

#include "../kram/sos.h"
#include "../file/absfile.h"

namespace sos {

//---------------------------------------------------------------------------Null_file

struct Null_file : Abs_file
{
    void                        open                    ( const char*, Open_mode, const File_spec& );
  //void                        close                   ( Close_mode );

    void                        update                  ( const Const_area& )       {}
    void                        update_direct           ( const Const_area& )       {}
    void                        insert                  ( const Const_area& )       {}
    void                        store                   ( const Const_area& )       {}

    void                        get_record_key          ( Area&, const Key& );
    void                        del                     ()                          {}
    void                        del_key                 ( const Key& )              {}
    void                        set                     ( const Key& )              {}
    void                        rewind                  ( Key::Number )             {}

  protected:
    void                        get_record              ( Area& );                   
    void                        put_record              ( const Const_area& )       {}
};

//----------------------------------------------------------------------Null_file_type

struct Null_file_type : Abs_file_type
{
    virtual const char*         name                    () const  { return "null"; }

    virtual Sos_ptr<Abs_file>   create_base_file        () const
    {
        Sos_ptr<Null_file> f = SOS_NEW_PTR( Null_file() );
        return +f;
    }
};

const Null_file_type           _null_file_type;
const Abs_file_type&            null_file_type = _null_file_type;

//---------------------------------------------------------------------Null_file::open

void Null_file::open( const char* parameter, Open_mode open_mode, const File_spec& file_spec )
{
    // Parameter werden ignoriert

    if( file_spec.key_specs().key_length() ) 
    {
        _key_len = file_spec.key_specs().key_length();
        _key_pos = file_spec.key_specs().key_position();
    }
}

//---------------------------------------------------------------Null_file::get_record

void Null_file::get_record( Area& buffer )
{
    throw_eof_error();
}

//-----------------------------------------------------------Null_file::get_record_key

void Null_file::get_record_key( Area& buffer, const Key& key )
{
    throw_not_found_error();
}

//--------------------------------------------------------------Null_file::put_record

} //namespace sos
