/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#define MODULE_NAME "sqlcopy"
#define AUTHOR      "Andreas Pschel"
#define COPYRIGHT   "1995 (C) Sos GmbH"

#include <sysdep.h>

#include <svwin.h>
#include <sv.hxx>
#include <sysdep.hxx>

#include <sosstrng.h>
#include <sos.h>
#include <sosfield.h>
#include <anyfile.h>
#include <sosopt.h>

#if defined SYSTEM_WIN
extern HINSTANCE _hinstance;
#endif


/*

             Eingabe                            Ausgabe
     sqlcopy file:c:/tmp/test.sql               oracle:
     sqlcopy -l c:/tmp/test.lst -prefix=<file:> oracle:

     sqlcopy file:c:/tmp/test.sql               odbc: -db <DNS>
     sqlcopy -f c:/tmp/test.lst -prefix=<file:> odbc: -db <DNS>

     sqlcopy odbc: -db SAB -table <tabelle>     sql -commit-at-end -table <tabelle> oracle:
     sqlcopy odbc: -db SAB -table <tabelle>     sql -commit-at-end -table <tabelle> oracle:


*/


int sos_main( int argc, char* argv[] )
{
#if defined SYSTEM_WIN
    _hinstance = Sysdepen::GethInst();
#endif

    Sos_string  input_list;
    Sos_string  input_prefix;
    Sos_string  output_file;
    Sos_string  files[100];
    int         count = 0;

    for ( Sos_option_iterator opt( argc, argv ); !opt.end(); opt.next() )
    {
        if ( opt.with_value( 'f', "file" ) )    files[count++] = opt.value();
        else
        if ( opt.with_value( 'l', "list" ) )    input_list = opt.value();
        else
        if ( opt.with_value( 'p', "prefix" ) )  input_prefix = opt.value();
        else
        if ( opt.param() )                      output_file = opt.value();
        else
        if ( opt.flag( "?" ) || opt.flag( "h" ) )
        {
            SHOW_MSG( "Usage: sqlcopy  [ [-l <filelist>] | [-f File] ] [-p=<dateityp>]\n" 
                      "       <ausgabedatei>\n" );
            return 0;
        }
        else throw Sos_option_error( opt );
    }

    if ( input_list != "" )
    {
      try {
        Dynamic_area area( 100 );
        Any_file f;
        f.open( c_str(input_list), File_base::Open_mode( File_base::in|File_base::seq ) );
        while (1)
        {
            try { f.get( &area ); } catch( const Eof_error& ) { break; }
            if ( count >= 100 ) { SHOW_MSG( "Mehr als 100 Dateien angegeben" ) break; }
            files[count++] = as_string( area ) + ".sql" ;
        }
        f.close();
      }
      catch( const Xc& x )
      {
        SHOW_ERR( "Fehler beim Lesen der Dateiliste: " << x );
      }
    }

    if ( count==0 ) return 0; // Alle Dateien gelesen

    for ( int i=0; i<count; i++ )
    {
      try {
        Sos_string source = input_prefix + files[i];
        Sos_string dest   = output_file;
        copy_file( c_str(source), c_str(dest) );
      }
      catch ( const Xc& x ) {
        x.insert( c_str(files[i]) ); throw x; // oder weiter ???
      }
    }

    SHOW_MSG( "SQL-Dateien erfolgreich bertragen." );

    return 0;
}

