/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// tabbed.h

#ifndef __TABBED_H
#define __TABBED_H

#ifndef __SOSARRAY_H
#include <sosarray.h>
#endif

#ifndef __SOSFIELD_H
#include <sosfield.h>
#endif

namespace sos {

struct Tabbed_file : Abs_file
{
    ~Tabbed_file();
    Tabbed_file();
    Tabbed_file( const char* szdatasource, Open_mode open_mode, const File_spec& file_spec );

    void open( const char* filename, Open_mode open_mode, const File_spec& file_spec );
    void close( Close_mode close_mode );

    void                            insert( const Const_area& area );
    void                            update( const Const_area& area );

    void                            set( const Key& key );
    void                            del( const Key& key );
    void                            del();

    Record_position                 key_position (Key::Number = 0) { return _key_position; };
    Record_length                   key_length   (Key::Number = 0) { return _key_length; };

    static const Abs_file_type&     static_file_type();

  private:
    void                            _init();
    void                            _convert( const Const_area& area, Area* area_ptr );
    uint                            _lookup_named_field( const char* field_name );
    void                            _init_field_order();
    void                            _insert_field_order();

    void                            _replace_select_all( Sos_string* );
    void                            print_field_names( std::ostream* s, const Record_type& record_type, const char* sep = ", ", const char* = "" );

    Any_file                        _file;
    Dynamic_area                    _current_key;
    Open_mode                       _open_mode;

    Record_position                 _key_position;
    Record_position                 _key_length;

    Sos_ptr<Record_type>            _record_type_ptr;
    Sos_ptr<Field_descr>            _key_descr_ptr;
    Text_format                     _format;

    Bool                            _field_names;
    Bool                            _with_field_order;
    Bool                            _transparent_write;
    Sos_simple_array<int>           _field_order;
  protected:

    void                            get_record( Area& area );
    void                            get_record_key( Area& area, const Key& key );

    void                            put_record( const Const_area& area ) { insert( area ); }

};

} //namespace sos

#endif


