/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: tb.cxx 3546 2005-04-03 08:30:29Z jz $
// tb.cxx                                     1997 SOS Software Gmbh
// Joacim Zschimmer

#include "precomp.h"

#include <limits.h>

#include <ctype.h>
#include "../kram/sysdep.h"
#include "../kram/sos.h"
#include "../file/absfile.h"
#include "../kram/sosopt.h"
#include "../kram/soslimtx.h"

namespace sos {

//------------------------------------------------------------------------------------Tb_file

struct Tb_file : Abs_file
{
                                    Tb_file             ();
                                   ~Tb_file             ();

    void                            open                ( const char*, Open_mode, const File_spec& );
    void                            close               ( Close_mode );
    void                            rewind              ( Key::Number );

  protected:
    void                            get_record          ( Area& );
    void                            put_record          ( const Const_area& );

  private:
    Fill_zero                      _zero_;
    Any_file                       _file;
    Dynamic_area                   _buffer;
};

//--------------------------------------------------------------------------------Tb_file_type

struct Tb_file_type : Abs_file_type
{
    virtual const char*         name                    () const { return "tb"; }
  //virtual const char*         alias_name              () const { return "nl"; }
    virtual Sos_ptr<Abs_file>   create_base_file        () const
    {
        Sos_ptr<Tb_file> f = SOS_NEW( Tb_file() );
        return +f;
    }
};

const Tb_file_type      _tb_file_type;
const Abs_file_type&     tb_file_type = _tb_file_type;

// --------------------------------------------------------------------Tb_file::Tb_file

Tb_file::Tb_file()
:
    _zero_(this+1)
{
}

// -------------------------------------------------------------------Tb_file::~Tb_file

Tb_file::~Tb_file()
{
}

// -------------------------------------------------------------------------Tb_file::open

void Tb_file::open( const char* param, Open_mode open_mode, const File_spec& file_spec )
{
    Sos_string filename;

    if( ( open_mode & inout ) == inout )  throw_xc( "SOS-1236" );

    for( Sos_option_iterator opt ( param ); !opt.end(); opt.next() )
    {
        if( opt.pipe() )                                filename = opt.rest();
        else throw_sos_option_error( opt );
    }

    _file.obj_owner( this );
    _buffer.allocate_min( 2000 );

    _file.open( filename, open_mode, file_spec );
}

// --------------------------------------------------------------------------Tb_file::close

void Tb_file::close( Close_mode close_mode )
{
    _file.close( close_mode );
}

// -------------------------------------------------------------------------Tb_file::rewind

void Tb_file::rewind( Key::Number )
{
    _file.rewind();
}

// ----------------------------------------------------------------------Tb_file::put_record

void Tb_file::put_record( const Const_area& record )
{
    int         soll_spalte = 0;
    int         ist_spalte  = 0;
    const char* p           = record.char_ptr();
    const char* p_end       = p + record.length();

    _buffer.allocate_min( record.length() + 1000 );
    _buffer.length( 0 );

    while( p < p_end )
    {
        switch( *p )
        {
            case ' ':
            {
                soll_spalte++;
                p++;
                break;
            }

            case '\r':
            case '\n':
            {
                soll_spalte = 0;
                _buffer.append( *p++ );
                ist_spalte = 0;
                break;
            }

            case '\t':
            {
                soll_spalte = ( soll_spalte + 8 ) & ~7;
                p++;
                break;
            }

            default:
            {
                while( ( soll_spalte & ~7 ) > ist_spalte )  
                {
                    _buffer.append( '\t' );  
                    ist_spalte = ( ist_spalte + 8 ) & ~7; 
                }
                _buffer.append( "        ", soll_spalte - ist_spalte );
                _buffer.append( *p++ ); 
                soll_spalte++;
                ist_spalte = soll_spalte;
            }
        }
    }

    // Blanks und Tabs am Ende werden ignoriert!

    _file.put( _buffer );
}

// ---------------------------------------------------------------------Tb_file::get_record

void Tb_file::get_record( Area& buffer )
{
    _file.get( &_buffer );

    buffer.allocate_min( _buffer.length() );

    const char* p = _buffer.char_ptr();
    const char* z = p + _buffer.length();

    while(1) 
    {
        const char* p2 = (char*)memchr( p, '\t', z - p );
        if( !p2 )  p2 = z;

        buffer.append( p, p2 - p );

        if( p2 == z )  break;

        uint n = ( buffer.length() + 8 ) & ~7;
        buffer.resize_min( n );
        memset( buffer.char_ptr() + buffer.length(), ' ', n - buffer.length() );
        buffer.length( n );

        p = p2 + 1;  // \t berspringen
        if( p == z )  break;
    }
}

} //namespace sos
