/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#define MODULE_NAME "checkfs"
#define COPYRIGHT   "1996 SOS GmbH Berlin"
#define AUTHOR      "Jrg Schwiemann"

#include <stdlib.h>
#include <errno.h>

#include "../kram/sosstrng.h"
#include "../kram/sos.h"
#include "../kram/sosopt.h"
#include "../file/anyfile.h"
#include "../kram/log.h"

#define EX_TRUE         0
#define EX_FALSE        1
#define EX_FSUNEXPECTED 2
#define EX_USAGE        3


#define USAGE \
    SHOW_ERR( "usage: checkfs -server=<server> [-running|-notrunning] [-log=<logfile]" ); \
    return EX_USAGE

int sos_main( int argc, char** argv )
{
    Sos_string fileserver;
    Bool check_running = true; // Default ist Prfen, ob Filserver luft

    //sos_static_ptr()->_licence->set_par( licence_fs );                 // FS erlauben
    Sos_option_iterator opt ( argc, argv );
    for(; !opt.end(); opt.next() ) {
        if( opt.with_value( 's', "server" ))  fileserver = opt.value();
        else
        if( opt.with_value( "log"         ))  log_start( opt.value() );
        else
        if( opt.flag(       "running"     ))  check_running = opt.set();
        else
        if( opt.flag(       "notrunning"  ))  check_running = !opt.set();
        else {
            USAGE;
        }
    }

    if( empty( fileserver ) ) {
        USAGE;
    }

    Any_file f;
    try {
        // Nur fr Unix-Fileserver
        Sos_string filename = as_string( "fs -s=" ) + fileserver + as_string( " /dev/null" );
        f.open( filename, File_base::out );
        f.put( "DUMMY-RECORD" );
        f.close();
    }
    catch ( const Xc& x ) {
        f.close(); // keine Exception, oder?
        if ( strcmpi( x.code(), "SOCKET-61" ) == 0 ) {
            return check_running ? EX_FALSE : EX_TRUE;
        }
        SHOW_ERR( "checkfs: unexpected error " << x );
        return EX_FSUNEXPECTED; // Unerwarteter Fehler ...
    }

    return check_running ? EX_TRUE : EX_FALSE;
}

