/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#include "precomp.h"
//#define MODULE_NAME "iso2ebc"
// iso2ebc.cpp                                         (c) SOS GmbH Berlin

#include <stdlib.h>

#if defined __BORLANDC__
#   include <mem.h>
#else
#   include <string.h>
#endif

#include "sos.h"

using namespace std;
namespace sos {


#define nix 0x41

Byte iso2ebc [256];
Byte iso2ebc_german [256];
char ebc2iso_german [256];
char ebc2iso_german_0_is_blank [256];

extern char mvs2iso [256];  // in ebc2iso.cxx
       Byte iso2mvs [256];
       Byte iso2mvs_german [256];
       char mvs2iso_german [256];
       char mvs2iso_german_0_is_blank [256];


/* = {
  nix , nix , nix , nix , nix , nix , nix , nix ,  // 00  Ascii-Steuerzeichen
  nix , nix , nix , nix , nix , nix , nix , nix ,  // 08
  nix , nix , nix , nix , nix , nix , nix , nix ,  // 10
  nix , nix , nix , nix , nix , nix , nix , nix ,  // 18

  0x40, 0x5A, 0x7F, 0x7B, 0x5B, 0x6C, 0x50, 0x7D,  // 20   !#"$%&'
  0x4D, 0x5D, 0x5C, 0x4E, 0x6B, 0x60, 0x4B, 0x61,  // 28  ()*+,-./
  0xF0, 0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7,  // 30  01234567
  0xF8, 0xF9, 0x7A, 0x5E, 0x4C, 0x7E, 0x6E, 0x6F,  // 38  89:;<=>?
  0x7C, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7,  // 40  @ABCDEFG
  0xC8, 0xC9, 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6,  // 48  HIJKLMNO
  0xD7, 0xD8, 0xD9, 0xE2, 0xE3, 0xE4, 0xE5, 0xE6,  // 50  PQRSTUVW
  0xE7, 0xE8, 0xE9, 0xBB, 0xBC, 0xBD, 0x6A, 0x6D,  // 58  XYZ[\]^_
  0x4A, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,  // 60  `abcdefg
  0x88, 0x89, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96,  // 68  hijklmno
  0x97, 0x98, 0x99, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6,  // 70  pqrstuvw
  0xA7, 0xA8, 0xA9, 0xFB, 0x4F, 0xfd, 0xFF, 0x7,   // 78  xyz{|}~

  0x68, 0xDC, 0x51, 0x42, 0x43, 0x44, 0x47, 0x48,  // 80  
  0x52, 0x53, 0x54, 0x57, 0x56, 0x58, 0x63, 0x67,  // 88  
  0x71, 0x9C, 0x9E, 0xCB, 0xCC, 0xCD, 0xDB, 0xC0,  // 90  
  0xDF, 0xEC, 0xFC, 0xB0, 0xB1, 0xB2, nix , nix ,  // 98  
  0x45, 0x55, 0xCE, 0xDE, 0x49, 0x69, 0x9A, 0x9B,  // A0 
  0xAB, nix , 0xBA, 0xB8, 0xB7, nix , 0x8A, 0x8B,  // A8  
  nix , nix , nix , nix , nix , nix , nix , nix ,  // B0  
  nix , nix , nix , nix , nix , nix , nix , nix ,  // B8  
  nix , nix , nix , nix , 0xBB, nix , nix , nix ,  // C0  
  nix , nix , nix , nix , nix , nix , nix , nix ,  // C8  
  nix , nix , nix , nix , nix , nix , 0xBC, nix ,  // D0  
  nix , nix , nix , nix , 0xBD, nix , nix , 0xFF,  // D8  
  nix , 0x59, nix , nix , 0xFB, nix , 0xA0, nix ,  // E0  
  nix , nix , nix , nix , nix , 0x70, nix , nix ,  // E8  
  nix , nix , nix , nix , nix , nix , 0x4F, nix ,  // F0  
  0x90, 0xB3, nix , nix , 0xFC, nix , 0xAE, 0xDF   // F8  
};*/

// static const car *AsciiCars = "[\\]{|}~";
// static const car *AnsiCars  = " ";


struct Iso2ebc_init
{
    Iso2ebc_init();
};

static Iso2ebc_init iso2ebc_init;


Iso2ebc_init::Iso2ebc_init()
{
    int i;

    memset( (void*)iso2ebc, nix, 256 );

    for( i = 0; i < 256; i++ ) {
        iso2ebc[ (Byte)ebc2iso[ i ] ] = i;
    }

#   ifdef _DEBUGxxx
    {
        for( int i = 0; i < 256; i++ ) {
            if( (Byte)ebc2iso[ (Byte)iso2ebc[ i ] ] != i )  {
                SHOW_MSG( "Zeichen " << (char)i << " (" << hex << i << dec << ") in iso2ebc wird nicht eindeutig umgesetzt." );
            }
        }
    }
#   endif

    // ebc2iso_text und iso2ebc_text kopieren
    memcpy( ebc2iso_german, ebc2iso, 256 );
    memcpy( iso2ebc_german, iso2ebc, 256 );

    // ebc2iso_german anpassen
    ebc2iso_german[iso2ebc[ (Byte) '[' ]] = '';
    ebc2iso_german[iso2ebc[ (Byte) '\\']] = '';
    ebc2iso_german[iso2ebc[ (Byte) ']' ]] = '';
    ebc2iso_german[iso2ebc[ (Byte) '{' ]] = '';
    ebc2iso_german[iso2ebc[ (Byte) '|' ]] = '';
    ebc2iso_german[iso2ebc[ (Byte) '}' ]] = '';
    ebc2iso_german[iso2ebc[ (Byte) '~' ]] = '';

    iso2ebc_german[(Byte)''] = iso2ebc[(Byte)'[' ];
    iso2ebc_german[(Byte)''] = iso2ebc[(Byte)'\\'];
    iso2ebc_german[(Byte)''] = iso2ebc[(Byte)']' ];
    iso2ebc_german[(Byte)''] = iso2ebc[(Byte)'{' ];
    iso2ebc_german[(Byte)''] = iso2ebc[(Byte)'|' ];
    iso2ebc_german[(Byte)''] = iso2ebc[(Byte)'}' ];
    iso2ebc_german[(Byte)''] = iso2ebc[(Byte)'~' ];

    memcpy( ebc2iso_german_0_is_blank, ebc2iso_german, 256 );
    ebc2iso_german_0_is_blank[ 0 ] = ' ';

// MVS:
    memset( (void*)iso2mvs, nix, 256 );

    for( i = 0; i < 256; i++ ) {
        iso2mvs[ (Byte)mvs2iso[ i ] ] = i;
    }

#   ifdef _DEBUGxxx
    {
        for( int i = 0; i < 256; i++ ) {
            if( (Byte)mvs2iso[ (Byte)iso2mvs[ i ] ] != i )  {
                SHOW_MSG( "Zeichen " << (char)i << " (" << hex << i << dec << ") in iso2mvs wird nicht eindeutig umgesetzt." );
            }
        }
    }
#   endif


    // mvs2iso_text und iso2mvs_text kopieren
    memcpy( mvs2iso_german, mvs2iso, 256 );
    memcpy( iso2mvs_german, iso2mvs, 256 );

    // mvs2iso_german anpassen
    mvs2iso_german[iso2mvs[ (Byte) '[' ]] = '';
    mvs2iso_german[iso2mvs[ (Byte) '\\']] = '';
    mvs2iso_german[iso2mvs[ (Byte) ']' ]] = '';
    mvs2iso_german[iso2mvs[ (Byte) '{' ]] = '';
    mvs2iso_german[iso2mvs[ (Byte) '|' ]] = '';
    mvs2iso_german[iso2mvs[ (Byte) '}' ]] = '';
    mvs2iso_german[iso2mvs[ (Byte) '~' ]] = '';
    mvs2iso_german[iso2mvs[ (Byte) '@' ]] = '';

    mvs2iso_german[iso2mvs[ (Byte) '' ]] = '[';
    mvs2iso_german[iso2mvs[ (Byte) '' ]] = '\\';
    mvs2iso_german[iso2mvs[ (Byte) '' ]] = ']';
    mvs2iso_german[iso2mvs[ (Byte) '' ]] = '{';
    mvs2iso_german[iso2mvs[ (Byte) '' ]] = '|';
    mvs2iso_german[iso2mvs[ (Byte) '' ]] = '}';
    mvs2iso_german[iso2mvs[ (Byte) '' ]] = '~';
    mvs2iso_german[iso2mvs[ (Byte) '' ]] = '@';

    iso2mvs_german[ (Byte)'' ] = iso2mvs[ (Byte)'[' ];
    iso2mvs_german[ (Byte)'' ] = iso2mvs[ (Byte)'\\'];
    iso2mvs_german[ (Byte)'' ] = iso2mvs[ (Byte)']' ];
    iso2mvs_german[ (Byte)'' ] = iso2mvs[ (Byte)'{' ];
    iso2mvs_german[ (Byte)'' ] = iso2mvs[ (Byte)'|' ];
    iso2mvs_german[ (Byte)'' ] = iso2mvs[ (Byte)'}' ];
    iso2mvs_german[ (Byte)'' ] = iso2mvs[ (Byte)'~' ];
    iso2mvs_german[ (Byte)'' ] = iso2mvs[ (Byte)'@' ];

    iso2mvs_german[ (Byte)'[' ] = iso2mvs[ (Byte)'' ];
    iso2mvs_german[ (Byte)'\\'] = iso2mvs[ (Byte)'' ];
    iso2mvs_german[ (Byte)']' ] = iso2mvs[ (Byte)'' ];
    iso2mvs_german[ (Byte)'{' ] = iso2mvs[ (Byte)'' ];
    iso2mvs_german[ (Byte)'|' ] = iso2mvs[ (Byte)'' ];
    iso2mvs_german[ (Byte)'}' ] = iso2mvs[ (Byte)'' ];
    iso2mvs_german[ (Byte)'~' ] = iso2mvs[ (Byte)'' ];
    iso2mvs_german[ (Byte)'@' ] = iso2mvs[ (Byte)'' ];

    memcpy( mvs2iso_german_0_is_blank, mvs2iso_german, 256 );
    mvs2iso_german_0_is_blank[ 0 ] = ' ';
}

} //namespace sos
