/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// logwin.h

#ifndef __LOGWIN_H
#define __LOGWIN_H

#if defined SYSTEM_STARVIEW  &&  defined SYSTEM_WIN 
#   define LOGWIN_USE_STARVIEW
#endif

#include <time.h>

#if defined LOGWIN_USE_STARVIEW  &&  !defined _SV_HXX
#   include <sv.hxx> // iostream wird von sv.hxx reingezogen
#endif

#include <windows.h>

using namespace std;
namespace sos
{

// --- struct Edit_streambuf --------------------------------------------------------------------

const int trace_buffer_size = 256;

struct Trace_window;


struct Edit_streambuf : streambuf
// kopiert aus log.cxx (Mswin_debug_streambuf), evtl. Integration von beiden!
{
#   if defined LOGWIN_USE_STARVIEW
                                Edit_streambuf          ( MultiLineEdit* edit_ptr, Trace_window* );
        MultiLineEdit*         _edit_ptr;
#   else
                                Edit_streambuf          ( Trace_window* );
#   endif
                               ~Edit_streambuf          ();

    virtual int _Cdecl          sync                    ();
    virtual int _Cdecl          underflow               ();
    virtual int _Cdecl          overflow                ( int = EOF );
    void                        new_line                ( char** );

    DECLARE_PUBLIC_MEMBER( Bool, with_time_prefix )

  private:
#   if defined USE_GETTIMEOFDAY
        struct timeval         _time_stamp;
#    else
        time_t                 _time_stamp;
#   endif

    Trace_window*              _trace_window;
    uint4                      _last_elapsed_msec;
    uint4                      _elapsed_msec;
    Bool                       _new_line;
    char                       _buffer [ trace_buffer_size+2 ];
};


// --- struct Edit_ostream -----------------------------------------------------------------------

struct Edit_ostream : Edit_streambuf, ostream
{
#if defined LOGWIN_USE_STARVIEW
                    Edit_ostream( MultiLineEdit* edit_ptr, Trace_window* );
#else
                    Edit_ostream( Trace_window* );
#endif
};


// --- struct TraceWindow -------------------------------------------

#if defined LOGWIN_USE_STARVIEW

struct Trace_window : WorkWindow
{
                                Trace_window            ( Window* = NULL, WinBits = WB_STDWORK );
                               ~Trace_window            ();

    virtual void                Resize                  ();
    virtual BOOL                Close                   () { if ( pApp->GetAppWindow() == (WorkWindow*)this ) pApp->Quit(); /*else Hide()*/; return TRUE; }
    void                        clear                   () { _ostream.flush(); _edit.SetText(""); }

    MultiLineEdit               _edit;

#else

struct Trace_window 
{
                                Trace_window            ();
                               ~Trace_window            ();

    BOOL                        Close                   ()          { return TRUE ; }
    void                        clear                   () { _ostream.flush(); }

#endif

    ostream*                    ostream_ptr             () { return &_ostream; }
    operator                    ostream&                () { return _ostream; }
    void                        file                    ( const char* );

 private:
    friend                      struct Edit_streambuf;

    ofstream                    _file;
    ostream*                    _f;
    Edit_ostream                _ostream;
};

/*
struct Auto_trace_window : Trace_window, Sos_self_deleting
{
                                Auto_trace_window() : Trace_window(), Abs_file(), Sos_self_deleting() {}
    virtual BOOL                Close() { if( TraceWindow::Close() ) { _anchor_ptr = NULL; return TRUE; } else return FALSE; }

 private:
    Sos_ptr<Auto_trace_window>  _anchor_ptr;
};
*/

//ostream* log_create( MultiLineEdit* _edit_ptr );

} //namespace sos

#endif
