/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#include "precomp.h"
//#define MODULE_NAME "msec"
//#define COPYRIGHT   "(c) SOS GmbH Berlin"
//#define AUTHOR      "Joacim Zschimmer"

#include "../kram/sos.h"
#include "../kram/msec.h"

#if defined SYSTEM_WIN

#   include <windows.h>
#   if defined SYSTEM_WIN16
#       include <toolhelp.h>
#   endif


    using namespace std;
    namespace sos {

    int64 elapsed_msec()
    {
#if 1 // defined __WIN32__
        return GetTickCount();
# else
        TIMERINFO timer_info;

        timer_info.dwSize         = sizeof timer_info;
        timer_info.dwmsSinceStart = 0;

        TimerCount( &timer_info );

        return timer_info.dwmsSinceStart;
#endif
    }

    } //namespace sos

#elif defined SYSTEM_SOLARIS                    // gettimeofday() scheint's nicht zu geben

#   include <sys/time.h>

    using namespace std;
    namespace sos {

    int64 elapsed_msec()
    {
        hrtime_t hrtime = gethrtime();          // high resolution time (ns)

        return (int64)( hrtime / 1000000 );
    }

    } //namespace sos

#else

    //#include <sys/types.h>
    //#include <unistd.h>
#   include <sys/time.h>

    using namespace std;
    namespace sos {

    int64 elapsed_msec()
    {
        struct timeval   tv;

        gettimeofday( &tv, 0 );

        return (int64)tv.tv_sec * 1000 + (int64)tv.tv_usec / 1000;
    }

    } //namespace sos
#endif
