/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
/* optional.h                                               (c) SOS GmbH Berlin
                                                            Joacim Zschimmer

    Opt< T > , T sollte Unterklasse von Storable_as< Sos_string > sein,
    ergibt einen von T abgeleiteten Typ, der leere String-Eingaben an ein Objekt vom Typ T erlaubt.

    Fr T mssen folgende Funktionen definiert sein:

    void empty( T* )
    void object_load( const Sos_string& )

    Fr Opt<T> werden die Funktionenen

    void convert_from_string( const Sos_string&, Opt<T>* )
    void Opt<T>::object_load( const Sos_string& )

    definiert, die bei einem leeren String (Blanks und Tabs sind erlaubt) T::make_empty() ruft,
    sonst T::object_load( Sos_string, T* ).
*/

#ifndef __OPTIONAL_H
#define __OPTIONAL_H

namespace sos
{

//#pragma implementation

#if !defined __LIMITS_H
//#   include <limits.h>
#endif

//typedef Sos_string;


struct Has_empty
{
    virtual Bool                empty                   () const            = 0;
    virtual void                make_empty              ()                  = 0;
};

template< class T >    // T mit Methoden empty() und make_empty()
struct Opt : T  //, Has_empty
{
                                Opt<T>                  ()                  {}
                                Opt<T>                  ( const T& o )      { *(T*)this = o; }

  //Bool                        empty                   () const            { return ::empty( *(T*)this ); } //{ return T::empty(); }
  //void                        make_empty              ()                  { ::empty( (T*)this );         }  //{ T::make_empty();   }

    void                        object_load             ( const Sos_string& );  // Falls T: Storable_as<Sos_string>
                              //operator Sos_date       () const;
};

template< class T >
void convert_from_string( const Sos_string& str, Opt<T>* object_ptr );

//template< class T >  Bool empty( T );    // Nicht allgemein implementiert, jeder Typ mu
//template< class T >  void empty( T* );   // besonders implementiert werden. Einige unten.

// empty() fr ein paar Standard-Datentypen:

//Bool empty( const Sos_string& );
//void empty( Sos_string* );

/*
#define IMPLEMENT_SIMPLE_EMPTY( Type, empty_value )                                             \
    inline Bool empty( Type o )     { return o == empty_value; }                                \
    inline void empty( Type* p )    { *p = empty_value;        }

IMPLEMENT_SIMPLE_EMPTY( char , ' '      );
IMPLEMENT_SIMPLE_EMPTY( short, SHRT_MIN );
IMPLEMENT_SIMPLE_EMPTY( int  , INT_MIN  );
IMPLEMENT_SIMPLE_EMPTY( long , LONG_MIN );
*/
//==========================================================================================inlines

template< class T >
inline void convert_from_string( const Sos_string& str, Opt<T>* object_ptr )
{
    object_ptr->object_load( str );
}


template< class T >
void Opt<T>::object_load( const Sos_string& str )
{
    if( ::empty( str )) {
        ::empty( (T*)this );
    } else {
        T::object_load( str );
        //convert_from_string( str, (T*) this );   ruft sich selbst auf.
    }
}

inline Bool empty( const Has_empty& object )
{
    return object.empty();
}


inline void empty( Has_empty* object_ptr )
{
    object_ptr->make_empty();
}


} //namespace sos

#endif
