/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: sosalloc.cxx 4164 2006-03-06 11:22:11Z jz $

#include "precomp.h"
//#define MODULE_NAME "sosalloc"
//#define COPYRIGHT   " 1995 SOS GmbH Berlin"
//#define AUTHOR      "Joacim Zschimmer"


//#if sizeof (void*) != 4
//#   error SOSALLOC ARBEITET NUR MIT VIER BYTE LANGEN ADRESSEN!!
//#endif

#include "../kram/sysdep.h"

#if defined SYSTEM_WIN
#   define _CRTDBG_MAPALLOC
#   include <crtdbg.h>
#   include <windows.h>
#endif

#include <stdlib.h>
#include <limits.h>             // UINT_MAX
#include <malloc.h>

#include "../kram/sos.h"
#include "../kram/sosprof.h"
#include "../kram/log.h"
#include "../kram/sosalloc.h"

using namespace std;
namespace sos {

//void check_pointer( const void* ptr, uint length );  // sysdep.cxx


void sos_alloc_check( const void* ptr, uint length );  // sosalloc.h


#ifdef USE_SOSALLOC
    static Bool       profile_read = false;

    static const int  max_cache_size    = 10;
    static const int  cache_size        = max_cache_size;
    static const uint max_cached_bytes  = 4096;      // Max. Gre fr einen versteckten Block
    static const int  new_register_size = 10000;
    Bool              sosalloc_check;
    static Bool       log;

    // Diese Variablen sind sehr schn im Debugger zu sehen:
    int4 sos_alloc_count      = 0;
    int4 sos_alloc_size       = 0;
    int4 sos_alloc_max_size   = 0;
    int4 sos_alloc_cache_hits = 0;

    struct Cached_mem
    {
        Byte*                      _ptr;
        uint                       _size;
    };

    struct Sosalloc_static
    {
                                    Sosalloc_static         ();
                                ~Sosalloc_static         ();

        void*                       alloc                   ( unsigned long size, const char* info );
        void                        free                    ( Byte*, uint size );

        void                        check_add               ( void* );
        void                        check_remove            ( void* );
        void                        check_memory            ();
        void                        assert_not_allocated    ( const void*, uint );
        void                        sos_alloc_check_allocated( const void* p, uint s, const char* info );
    //Bool                        is_allocated            ( const void*, uint );
        void                        list                    ( ostream* );
        static const char*          info                    ( const void* );
        static Bool                 overlaps                ( const void* p, uint s, const void* q );

        Fill_zero                  _zero_;
        Sos_static*                _sos_static;
        uint                       _max_cached_bytes;
        uint                       _cache_size;
        int4                       _count;                  // Statistik  Anzahl Datenbereiche
        int4                       _size;                   // Statistik  Nettogre
        int4                       _total_count;            // Statistik. Anzahl sos_alloc()
        int4                       _cache_hits;             // Statistik
        void**                     _new_register;
        int                        _new_register_count;
        int                        _last_cached;
        Bool                       _hit;

        Cached_mem             _cache[ max_cache_size+1 ];  // +1, falls max_cache_size == 0
    };
#endif


//------------------------------------------------------------------------------------sos_alloc

void* sos_alloc( int4 size, const char* info )
{
#   ifdef USE_SOSALLOC
        Sos_static* sos_static = sos_static_ptr();

        if( !sos_static ) {
#           if defined SYSTEM_WIN
                OutputDebugString( "sos_alloc() wird von Sos_static() gerufen (rekursiv)\n" );
#           else
                cerr << "sos_alloc() wird von Sos_static() gerufen (rekursiv)\n";
#           endif
            abort();
        }

        Sosalloc_static* sosalloc_static = sos_static->_sosalloc;
        if( !sosalloc_static ) {
            // Dieser malloc mu klappen:
            sosalloc_static = new( malloc( sizeof (Sosalloc_static) ) ) Sosalloc_static;
            sos_static_ptr()->_sosalloc = sosalloc_static;
            sosalloc_static->_sos_static = sos_static;
        }


        if( !profile_read ) {
            profile_read = true;
    //#       if !defined __DLL__  &&  defined SYSTEM_WIN       // In Solaris Absturz (Any_file)
                sosalloc_check = read_profile_bool( "", "debug", "check-new", false );
                if( sosalloc_check ) LOG( "sos_alloc: [debug] check-new=yes\n" );
                sosalloc_static->_cache_size       = cache_size;       //read_profile_uint( "", "sosalloc", "cachesize", 10 );
                sosalloc_static->_max_cached_bytes = max_cached_bytes; //read_profile_uint( "", "sosalloc", "maxcachedbytes", 4096 );
    //#       endif
            log = read_profile_bool( "", "debug", "log-new", false );
        }

        if( log )  LOG( "sos_alloc(" << setw(6) << size << setw(0) << ")  " );

        if( sosalloc_check )  sosalloc_static->check_memory();

        Byte* p = (Byte*)sosalloc_static->alloc( size + 16, info );

        int4 s = *(int4*)(p-8+4); // Bei Speicher aus dem Cache: s >= size
        sos_alloc_size += s;    sosalloc_static->_size += s;     
        sos_alloc_count++;      sosalloc_static->_count++;
                                sosalloc_static->_total_count++;
        if( sos_alloc_max_size < sos_alloc_size )  sos_alloc_max_size = sos_alloc_size;

        //if( log )  LOG( sosalloc_static->_count << " Datenbereiche, " << sosalloc_static->_size << " Byte total\n" );
        if( log )  { LOG( (void*)p << ( sosalloc_static->_hit? " HIT " : "     " ) ); sos_alloc_log_statistic( log_ptr, info ); }

        return p;

#   else

#       if defined _DEBUG && defined SYSTEM_MICROSOFT

            void* p = _malloc_dbg( size, _NORMAL_BLOCK, info, 0 ); 

#        else

            void* p = malloc( size );

#       endif

        if( !p )  throw_no_memory_error( size );

        memset( p, 0, size );
        return p;

#   endif
}

//-------------------------------------------------------------------------------------sos_free

void sos_free( void* ptr )
{
#   ifdef USE_SOSALLOC
        if( !ptr )  return;

        Sos_static*      sos_static = sos_static_ptr();
        Sosalloc_static* sosalloc_static = sos_static->_sosalloc;

        if( !sosalloc_static )  {
#           if defined SYSTEM_WIN
                OutputDebugString( "sos_free() nach ~Sos_static gerufen\n" );
#           endif
            return;
        }

        if( sosalloc_check &&  sosalloc_static )  sosalloc_static->check_memory();


        Byte* p = (Byte*)ptr - 8;

        sos_alloc_check( ptr );
        if( sosalloc_check )  sosalloc_static->check_remove( p );

        uint        size = *(int4*)(p+4);
	    const char* inf  = NULL;
    
        if( log )  inf = Sosalloc_static::info( p );  // Fr log

        sos_alloc_size -= size;  sosalloc_static->_size -= size;
        sos_alloc_count--;       sosalloc_static->_count--;

        p[0] = 'b'; p[1] = ''; p[2] = 'r'; p[3] = 'j';

        Byte* q = p + 8 + size;
        q[0] = 's'; q[1] = 'l'; q[2] = 'u'; q[3] = 't';

        sosalloc_static->free( p, 8 + size + 8 );

        if( log ) {
            LOG( "sos_free (" << setw(6) << size << setw(0) << ")  " << (void*)ptr << ( sosalloc_static->_hit? " hit " : "     " ) );
            sos_alloc_log_statistic( log_ptr, inf );
        }

#   elif defined _DEBUG && defined SYSTEM_MICROSOFT

        _free_dbg( ptr, _NORMAL_BLOCK );

#   else

        free( ptr );

#   endif
}

//-------------------------------------------------------------------------------sos_alloc_list

void sos_alloc_list( ostream* s )
{
#   ifdef USE_SOSALLOC
        sos_static_ptr()->_sosalloc->list( s );
#   endif
}

//------------------------------------------------------------Sos_alloc_static::Sosalloc_static
#ifdef USE_SOSALLOC

Sosalloc_static::Sosalloc_static() 
: 
    _zero_(this+1) 
{
}

#endif
//------------------------------------------------------------Sosalloc_static::~Sosalloc_static
#   ifdef USE_SOSALLOC

Sosalloc_static::~Sosalloc_static()
{
    for( int i = 0; i < max_cache_size; i++ ) {
        ::free( _cache[ i ]._ptr );
        _cache[ i ]._ptr = NULL;
    }
}

#endif
//-----------------------------------------------------------------------Sosalloc_static::alloc
#ifdef USE_SOSALLOC

void* Sosalloc_static::alloc( unsigned long size, const char* info )
{
    Byte*       p         = 0;
    uint        best_size = (uint)-1;
    Cached_mem* best      = 0;
    Cached_mem* c         = _cache + _last_cached;

    _hit = false;

    if( _cache_size ) {
        while( 1 ) {
            if( c->_size >= size  &&  c->_size <= best_size ) {
                best = c;
                best_size = best->_size;
                if( size == best_size )  break;
            }
            if( c == _cache )  c = _cache + _cache_size;
            c--;
            if( c == _cache + _last_cached )  break;
        }

        if( best &&  best_size < 2 * size ) {
            p = best->_ptr;
            best->_ptr = 0;
            best->_size = 0;
            sos_alloc_cache_hits++;   _cache_hits++;
            _hit = true;              // fr sos_alloc() LOG
        }
    }

    if( !p ) {
        if( size > UINT_MAX )  p = 0;
        else {
            best_size = size;
            p = (Byte*)malloc( size );
        }

        if( p )  check_pointer( p, best_size, "malloc" );
        else {
            if( log )  LOG( "***failed***\n" );
            throw_no_memory_error( size );    // xalloc?
        }
    }
    else 
    {
        check_pointer( p, best_size, "sos_alloc.cache" );
    }

    size -= 16;
    best_size -= 16;

    if( sosalloc_check ) {
        assert_not_allocated( p, size );
        check_add( p );
    }

    p[0] = 'B'; p[1] = ''; p[2] = 'R'; p[3] = 'J';
    *(int4*)(p+4) = best_size;      // best_size >= size
    p += 8;
    memset( p, 0x00, size );
    memset( p + size, 0xFF, best_size - size );  // berhang
    Byte* q = p + best_size;
    q[0] = 'S'; q[1] = 'L'; q[2] = 'U'; q[3] = 'T';
    
    if( !info )  info = "";

    q[4+0] = ((Byte*)&info)[0];  
    q[4+1] = ((Byte*)&info)[1];  
    q[4+2] = ((Byte*)&info)[2];  
    q[4+3] = ((Byte*)&info)[3];

    return p;
}

#endif
//------------------------------------------------------------------------Sosalloc_static::info
#ifdef USE_SOSALLOC

const char* Sosalloc_static::info( const void* q )
{
    const Byte* in = (Byte*)q + 8 + *(int4*)((Byte*)q+4) + 4;
    const char* inf;

    ((Byte*)&inf)[0] = in[0];
    ((Byte*)&inf)[1] = in[1];  
    ((Byte*)&inf)[2] = in[2];
    ((Byte*)&inf)[3] = in[3];

    if( !inf )  inf = "";

    _check_string0_pointer( inf );

    return inf;
}

#endif
//------------------------------------------------------------------------Sosalloc_static::free
#ifdef USE_SOSALLOC

void Sosalloc_static::free( Byte* p, uint size )
{
    _hit = false;

    if( size > _max_cached_bytes ) {
        ::free( p );
        return;
    }

    if( _cache[ _last_cached ]._ptr ) {
        if( ++_last_cached == _cache_size )  _last_cached = 0;
    }

    Cached_mem* c = &_cache[ _last_cached ];
    if( c->_ptr )  ::free( c->_ptr );
             else  _hit = true;               // Kein free

    c->_ptr  = p;
    c->_size = size;
}

#endif
//-------------------------------------------------------------------Sosalloc_static::check_add
#ifdef USE_SOSALLOC

void Sosalloc_static::check_add( void* p )
{
    int i;

    for( i = 0; i < _new_register_count; i++ ) {
        if( _new_register[ i ] == 0 )  break;
    }
    if( i < new_register_size ) {
        if( !_new_register ) {
            _new_register = (void**)malloc( new_register_size * sizeof (void*) );
        }
        if( i == _new_register_count )  _new_register_count++;
        _new_register[ i ] = p;
    } else {
        SHOW_MSG( "Hinweis: sos_alloc: " << new_register_size << " Datenbereiche angefordet. check-new wird abgeschaltet." );
        sosalloc_check = false;
    }
}

#endif
//----------------------------------------------------------------Sosalloc_static::check_remove
#ifdef USE_SOSALLOC

void Sosalloc_static::check_remove( void* p )
{
    int i;
    for( i = 0; i < _new_register_count; i++ )  {
        if( _new_register[ i ] == p )  break;
    }
    if( i == _new_register_count ) {
        LOG( "sos_free() " << (void*)p << "\n"
             "Interner Fehler: Speicher ist nicht mit sos_alloc() angefordert worden" << '\n' );
        SHOW_ERR( "Interner Fehler: Speicher ist nicht mit sos_alloc() angefordert worden" );
        return;
    }
    _new_register[ i ] = 0;
}

#endif
//---------------------------------------------------------------Sos_alloc_static::check_memory
#ifdef USE_SOSALLOC

void Sosalloc_static::check_memory()
{
    if( sosalloc_check ) {
        for( int i = 0; i < _new_register_count; i++ ) {
            if( _new_register[ i ] ) {
                sos_alloc_check( (Byte*)_new_register[ i ] + 8 );
            }
        }
    }
}

#endif
//----------------------------------------------------------------Sosalloc_static::is_allocated
/*
Bool Sosalloc_static::is_allocated( const void* p, uint s )
{
    int i;
    for( i = 0; i < _new_register_count; i++ )
    {
        Byte* q = (Byte*)_new_register[ i ];
        if( p >= q + 8 ) {
            sos_alloc_check( q + 8 );
            uint size = *(int4*)(q+4);
            if( (Byte*)p + s <= q + 8 + size )  return true;
        }
    }

    return false;
}
*/
//--------------------------------------------------------Sosalloc_static::assert_not_allocated
#ifdef USE_SOSALLOC

Bool Sosalloc_static::overlaps( const void* p, uint s, const void* q )
{
    if( q  &&  (unsigned long)p + 8 + s + 8 > (unsigned long)q ) {
        sos_alloc_check( (Byte*)q + 8 );
        uint size = *(int4*)((Byte*)q+4);
        if( (unsigned long)p < (unsigned long)q + 8 + size + 8 )  return true;
    }

    return false;
}

#endif
//--------------------------------------------------------Sosalloc_static::assert_not_allocated
#ifdef USE_SOSALLOC

void Sosalloc_static::assert_not_allocated( const void* p, uint s )
{
    for( int i = 0; i < _new_register_count; i++ )
    {
        const Byte* q = (Byte*)_new_register[ i ];
        if( q  &&  overlaps( p, s, q ) ) {
            uint size = *(int4*)((Byte*)q+4);
            show_msg( "Interner Fehler in der Speicherverwaltung: Speicher ist DOPPELT belegt\n" );
            LOG( "*** sosalloc.cxx: " << (void*)p << ".." << (void*)((Byte*)p+8+s+8) << " berlappt vorhandenen " << (void*)q << ".." << (void*)((Byte*)q+8+size+8) << ' ' << info( q ) << '\n' );
            abort();
        }
    }
}

#endif
//-----------------------------------------------------------------------Sosalloc_static::list
#ifdef USE_SOSALLOC

void Sosalloc_static::list( ostream* s )
{
    if( !s )  return;

    for( int i = 0; i < _new_register_count; i++ )  {
        const Byte* p = (Byte*)_new_register[ i ];
        if( p ) {
            sos_alloc_check( p + 8 );
        	const char* inf  = info( p );  // Fr log
            uint size = *(int4*)(p+4);
            *s << "sos_alloc_list: " << (void*)(p+8) << ' ' << setw(6) << size << setw(0) << " Bytes  " << inf << endl;
        }
    }
}

#endif
//---------------------------------------------------------------------sos_alloc_check_allocated
#ifdef USE_SOSALLOC
// Prft, ob ein Adressbereich in genau einem angeforderten Speicherbereich oder in gar keinem
// Speicherbereich liegt.

void Sosalloc_static::sos_alloc_check_allocated( const void* p, uint s, const char* inf )
{
    for( int i = 0; i < _new_register_count; i++ )
    {
        const Byte* q = (Byte*)_new_register[ i ];
        if( q  &&  overlaps( p, s, q ) ) 
        {
            uint size = *(int4*)((Byte*)q+4);
            if( p >= (Byte*)q+8  &&  (Byte*)p+s <= (Byte*)q+8+size ) {
                // p..p+s ist vollstndig in einem angefordertem Speicherbereich enthalten
            } else {
                // p..p+s berlappt ungltig einen angeforderten Speicherbereich
                if( !inf )  inf = "";
                show_msg( "Interner Fehler: Ein Byte auerhalb eines Puffers wird angesprochen.\nBitte beenden Sie sofort das Programm." );
                LOG( "*** sosalloc.cxx: " << inf << ' ' << (void*)p << ".." << (void*)((Byte*)p+8+s+8) << " berlappt vorhandenen: " << (void*)q << ".." << (void*)((Byte*)q+8+size+8) << ' ' << info( q ) << '\n' );
            }
        }
    }
}

#endif
//---------------------------------------------------------------------sos_alloc_check_allocated

void sos_alloc_check_allocated( const void* p, uint s, const char* info )
{
#   ifdef USE_SOSALLOC
        Sosalloc_static* sosalloc_static = sos_static_ptr()->_sosalloc;
        if( sosalloc_static )  sosalloc_static->sos_alloc_check_allocated( p, s, info );
#   endif
}

//---------------------------------------------------------------------------sos_alloc_destruct

void sos_alloc_destruct( Sosalloc_static* s )
{
#   ifdef USE_SOSALLOC
        s->~Sosalloc_static();
#   endif
}

//------------------------------------------------------------------------------sos_alloc_check

void sos_alloc_check( const void* ptr )
{
#   ifdef USE_SOSALLOC
        Byte* p = (Byte*)ptr - 8;

        if( memcmp( p, "BRJ", 4 ) != 0 ) {
            const char msg1[] = "Fatal: sos_free: Anfang eines Datenbereichs berschrieben\n"
                                "oder sos_free auf ungltige Adresse";
            const char msg2[] = "Fatal: sos_free: Datenbereich wird doppelt freigegeben.";
            const char* msg = memcmp( p, "brj", 4 ) == 0? msg2 : msg1;

#           if defined SYSTEM_WIN
                OutputDebugString( msg );
                OutputDebugString( "\n" );
#           endif
            SHOW_ERR( msg );
            abort();
        }

        uint size = *(int4*)(p+4);

        if( memcmp( p + 8 + size, "SLUT", 4 ) != 0 ) {
            const char msg[] = "Fatal: sos_free: Datenbereich geplatzt";
#           if defined SYSTEM_WIN
                OutputDebugString( msg );
                OutputDebugString( "\n" );
#           endif
            //SHOW_ERR( msg );
            show_msg( msg );

            const char* inf = Sosalloc_static::info( p );
            if( inf  &&  inf[0] != '\0' ) {
#               if defined SYSTEM_WIN
                    OutputDebugString( inf );
                    OutputDebugString( "\n" );
#               endif
                show_msg( inf );
            }

            if( log_ptr )
            {
                const uint n = 40;

                *log_ptr << "addr=" << (void*)ptr << "->";

                uint len = size;
                if( len > n ) {
                    len = n;
                    *log_ptr << Const_area( (Byte*)ptr, min( size - n, n ) );   // Die ersten Bytes
                    if( size > n+n )  *log_ptr << "...";                        // Lcke
                }

                *log_ptr << hex << Const_area( (Byte*)ptr + size - len, len );  // Die letzten Bytes
                *log_ptr << "|";
                *log_ptr << Const_area( (Byte*)ptr + size, 8 );
                *log_ptr << dec << '\n' << flush;
            }

            abort();
        }
#    else
#   endif
}

//------------------------------------------------------------------------------sos_alloc_check

void sos_alloc_check( const void* ptr, uint length )
{
#   ifdef USE_SOSALLOC
        check_pointer( (Byte*)ptr, length );
        sos_alloc_check( ptr );
#   endif
}

//------------------------------------------------------------------------------sos_alloc_check

void sos_alloc_check()
{
#   ifdef USE_SOSALLOC
        Sosalloc_static* sosalloc_static = sos_static_ptr()->_sosalloc;
        if( !sosalloc_static )  return;

        sosalloc_static->check_memory();
#   endif
}

//----------------------------------------------------------------------sos_alloc_log_statistic

void sos_alloc_log_statistic( ostream* s, const char* info )
{
#   ifdef USE_SOSALLOC
        if( !s )  return;

        if( !info )  info = "";

        Sosalloc_static* sosalloc_static = sos_static_ptr()->_sosalloc;
        if( sosalloc_static ) {
            *s << "sos_alloc:"
                  "  size=" << sosalloc_static->_size <<
                  "  max=" << sos_alloc_max_size <<
                  "  count=" << sosalloc_static->_count <<
                  "  total=" << sosalloc_static->_total_count <<
                  "  hits=" << ( 100 * sosalloc_static->_cache_hits / sosalloc_static->_total_count ) <<
                  "% " << info << '\n';
        }
#   endif
}

//-------------------------------------------------------------------------------sos_alloc_size
/*
int4 sos_alloc_size()
{
    Sosalloc_static* sosalloc_static = sos_static_ptr()->_sosalloc;
    return sosalloc_static? sosalloc_static->_size : 0;
}

//------------------------------------------------------------------------------sos_alloc_count

int4 sos_alloc_count()
{
    Sosalloc_static* sosalloc_static = sos_static_ptr()->_sosalloc;
    return sosalloc_static? sosalloc_static->_count : 0;
}
*/

} //namespace sos
