/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#include "precomp.h"
//#define MODULE_NAME "sosstat"
//#define COPYRIGHT   "1996 SOS GmbH Berlin"
//#define AUTHOR      "Joacim Zschimmer"

#include "../kram/sos.h"
#include "../kram/sosstat.h"
#include "../kram/sosclien.h"
#include "../file/absfile.h"
#include "../file/sosdb.h"
#include "../kram/log.h"


using namespace std;
namespace sos {

DEFINE_SOS_STATIC_PTR( Sos_client )

//-----------------------------------------------------------------------------------Std_client

struct Std_client : Sos_client
{
};

//------------------------------------------------------------------------------init_std_client

void init_std_client()
{
    Sos_ptr<Std_client> client = SOS_NEW_PTR( Std_client );
    client->_name = "standard-client";
    sos_static_ptr()->_std_client = +client;
}

//-----------------------------------------------------------------------Sos_client::Sos_client

Sos_client::Sos_client()
{
    _session_array.obj_const_name( "Sos_client::_session_array" );
    _session_array.first_index( 1 );
}

//----------------------------------------------------------------------Sos_client::~Sos_client

Sos_client::~Sos_client()
{
    close();
}

//----------------------------------------------------------------------------Sos_client::close

void Sos_client::close()
{
    for( int i = _session_array.last_index(); i >= _session_array.first_index(); i-- ) {
        Sos_ptr<Sos_database_session> s = _session_array[ i ];
        if( s ) {
            _session_array[ i ] = NULL;
            try {
                LOGI( "Sos_client::close()  close db=" << s->_db_name
                      << ", user=" << s->_user << '\n' );
                s->close( close_error );
            }
            catch(...) {}
        }
    }
/*
    for( int i = _static_array.last_index(); i >= _static_array.first_index(); i-- )
    {
        Sos_database_static* s = _static_array[ i ];
        if( s ) {
            _static_array[ i ] = 0;
            try {
                s->close_all_sessions();
            }
            catch(...) {}
        }
    }
*/
}

//-----------------------------------------------------------------------Sos_client::_obj_print

void Sos_client::_obj_print( ostream* s ) const
{
    *s << _name;

    for( int i = _session_array.first_index(); i <= _session_array.last_index(); i++ )
    {
        Sos_database_session* session = _session_array[ i ];
        if( session ) {
            *s << ", DB session " << i << ": ";
            session->obj_print( s );
            *s << "; ";
        }
    }
}

} //namespace sos
