/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// sosctain.h                                           1996 SOS GmbH Berlin

#ifndef __SOSCTAIN_H
#define __SOSCTAIN_H

//----------------------------------------------------------------------------Sos_key_container
//Bentigt  KEY  key        ( const OBJECT& );
//          Bool operator== ( const KEY&, const KEY& );
//          Bool null       ( const OBJECT& );
//          void close      ( OBJECT* );               // postcondition  null(object) == true

template< class OBJECT, class KEY >
struct Sos_key_container
{
    typedef int                 Id;

                                Sos_key_container       ();
                               ~Sos_key_container       ();

    Id                          add                     ( const TYPE& );
    Id                          store                   ( const TYPE& );
    Id                          search                  ( const KEY& );
    TYPE&                       get_by_key              ( const KEY& );
    TYPE&                       get_by_id               ( Id );

    static Bool                 valid_id                ( Id i )                                { return i != 0; }

  private:
    Sos_simple_array< OBJECT >  _array;
};

//---------------------------------------------Sos_key_container<OBJECT,KEY>::Sos_key_container

template< class OBJECT, class KEY >
Sos_key_container<OBJECT,KEY>::Sos_key_container()
{
    _array.obj_const_name( "Sos_key_container::_array" );
    _array.first_index( 1 );
}

//---------------------------------------------Sos_key_container<OBJECT,KEY>::Sos_key_container

template< class OBJECT, class KEY >
Sos_key_container<OBJECT,KEY>::~Sos_key_container()
{
    for( int i = _array.last_index(); i >= _array.first_index(); i-- ) {
        close( &_array[ i ] );
    }
}

//----------------------------------------------------------Sos_key_container<OBJECT,KEY>::find

template< class OBJECT, class KEY >
Sos_key_container<OBJECT,KEY>::Id Sos_key_container<OBJECT,KEY>::find( const KEY& key )
{
    for( int i = _array.first_index(); i <= _array.last_index(); i++ ) {
        if( key( _array[ i ] ) == key )  return i;
    }

    return -1;
}

//---------------------------------------------------------Sos_key_container<OBJECT,KEY>::store

template< class OBJECT, class KEY >
Sos_key_container<OBJECT,KEY>::Id Sos_key_container<OBJECT,KEY>::store( const OBJECT& o )
{
    Id id  = find( key( o ) );
    return valid_id( id )? id : add( o );
}

//-----------------------------------------------------------Sos_key_container<OBJECT,KEY>::add

template< class OBJECT, class KEY >
Sos_key_container<OBJECT,KEY>::Id Sos_key_container<OBJECT,KEY>::add( const OBJECT& o )
{
    for( i = 0; i <= _array.last_index(); i++ )  if( null( _array[ i ] ) )  break;
    if( i > _array.last_index() )  _array.last_index( _array.last_index() + 1 );
    _array[ i ] = o;
    return i;
}

#endif
