/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#include <precomp.h>
//#define MODULE_NAME "sosddeap"
//#define COPYRIGHT   "1996 SOS GmbH Berlin"
//#define AUTHOR      "Joacim Zschimmer"

//#define PROBEJAHR 1995

#if defined PROBEJAHR
#   include <time.h>
#endif

#include <sos.h>

#if defined SYSTEM_STARVIEW  &&  defined SYSTEM_WIN 
#   include <svwin.h>
#   include <sv.hxx>
#   include <sysdep.hxx>
#endif

#include <xception.h>
#include <soswin.h>

#include <sosappl.h>
#include <sosstrng.h>
#include <sosopt.h>
#include <log.h>
#include <filedde.h>
#include <logwin.h>
#include <sosdde.hrc>

//#define SOSDDE_TRACE_WINDOW 0

HINSTANCE _hinstance;
extern Bool sossock_reschedule;



struct Sos_dde_application : Sos_application
{
  protected:
    void                       _main                    ( int, char*[] );
};


Sos_dde_application sos_application;

void Sos_dde_application::_main( int argc, char** argv )
{
    Sos_appl    appl            ( false );
    Sos_string  dde_server_name = "hostDDE";
    Bool        trace           = true;

    Trace_window* win_ptr = NULL;

    //sossock soll nicht DDE-Anforderungen durchlassen, dass wird rekursiv und fhrt zum Absturz. jz 21.8.97 (???)
    sossock_reschedule = true;  // Windows-Botschaften knnen uns nicht irritieren. Oder doch? jz 21.8.97

#   if defined PROBEJAHR
    {
        int ___PROBEVERSION_MIT_BEGRENZTER_LAUFZEIT___;
        time_t t;
        time( &t );
        const char* ct = ctime( &t );
        if( ct[ 23 ] != PROBEJAHR - 1990 + '0' )  {
            SHOW_ERR( "Die kostenlose Probezeit dieses Programms ist abgelaufen.\n"
                      "SOS GmbH Berlin, Telefon (030) 864 790-0" );
            return;
        }
    }
#   endif

try {
    appl.init();

    for ( Sos_option_iterator opt( argc, argv ); !opt.end(); opt.next() )
    {
        if( opt.with_value( "log" ) )   log_start( c_str( opt.value() ) );
        else
        if( opt.flag( 't', "trace" ) )  trace = opt.set();
        else
        if( opt.param() ) { dde_server_name = opt.value(); opt.max_params(1); }
        else throw Sos_option_error( opt );
    }

    File_dde_server  file_dde_server ( c_str(dde_server_name) );

#   if defined LOGWIN_USE_STARVIEW
        win_ptr = new Trace_window( NULL, WB_APP|WB_STDWORK|WB_SIZEMOVE|WB_MINMAX );
        if ( !win_ptr ) throw_no_memory_error();

        win_ptr->ChangeIcon( Icon( ResId( ICON_SOSDDE )));
        win_ptr->SetText( String( "DDE-Server " ) + String( dde_server_name ) );
        win_ptr->Show(); // Fenster anzeigen
#   else
        win_ptr = new Trace_window;
        if ( !win_ptr ) throw_no_memory_error();
#   endif

    if ( trace )
    {
        file_dde_server.log_stream_ptr( ((Trace_window*)win_ptr)->ostream_ptr() );
        *(((Trace_window*)win_ptr)->ostream_ptr()) << "DDE-Server " << dde_server_name << " gestartet." << endl;
    } else {
        file_dde_server.log_stream_ptr( log_ptr );
        LOG( "DDE-Server " << dde_server_name << " gestartet." << endl );
    }


    execute();

    delete win_ptr;
    //sos_static_ptr()->~Sos_static();
}
catch( const Xc& x )
{
    SHOW_MSG( x );

    delete win_ptr;

    sos_static_ptr()->~Sos_static();
    throw;
}
}


