/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
//#define MODULE_NAME "sossql3"
//#define COPYRIGHT   "1995 SOS GmbH Berlin"
//#define AUTHOR      "Joacim Zschimmer"

#include "precomp.h"
#include "sos.h"
#include "sosstat.h"
#include "sosfunc.h"

using namespace std;
namespace sos {


DEFINE_SOS_STATIC_PTR( Sos_function_register )

//------------------------------------------------------Sos_function_descr::Sos_function_descr

Sos_function_descr::Sos_function_descr() 
: 
    _zero_(this+1) 
{
    obj_const_name( "Sos_function_descr" );
}

//-----------------------------------------------------Sos_function_descr::~Sos_function_descr

Sos_function_descr::~Sos_function_descr()
{
}

//-------------------------------------------------Sos_function_register::Sos_function_register

Sos_function_register::Sos_function_register()
:
    _zero_ (this+1)
{
    obj_const_name( "Sos_function_register" );
    _func_array.obj_const_name( "Sos_function_descr::_func_array" );
}

//------------------------------------------------Sos_function_register::~Sos_function_register
    
Sos_function_register::~Sos_function_register()
{
}

//------------------------------------------------------------------Sos_function_register::init

Sos_function_register* Sos_function_register::init()
{
    Sos_static* st = sos_static_ptr();

    if( !st->_function_register )  {
        Sos_ptr<Sos_function_register> reg = SOS_NEW( Sos_function_register );
        st->_function_register = +reg;
    }
     
    return st->_function_register;
}

//-------------------------------------------------------------------Sos_function_register::add

void Sos_function_register::add( Sos_function* func, const char* name, int param_count )
{
    Sos_ptr<Sos_function_descr> func_descr = SOS_NEW( Sos_function_descr );

    func_descr->_name = name;
    func_descr->_param_count = param_count;
    func_descr->_func = func;

    add( func_descr );
}

//-------------------------------------------------------------------Sos_function_register::add

void Sos_function_register::add( const Sos_ptr<Sos_function_descr>& func_descr )
{
    add( +func_descr );
}

//-------------------------------------------------------------------Sos_function_register::add

void Sos_function_register::add( Sos_function_descr* func_descr )
{
    if( function_or_0( c_str( func_descr->_name ), func_descr->_param_count ) )  throw_xc( "SOS-1312", c_str( func_descr->_name ) ); 
    _func_array.add( func_descr );
}

//---------------------------------------------------------Sos_function_register::function_or_0

Sos_function* Sos_function_register::function_or_0( const char* name, int param_count )
{
    Sos_function_descr* func_descr = function_descr_or_0( name, param_count );
    return func_descr? func_descr->_func : NULL;
}

//---------------------------------------------------------Sos_function_register::function_descr

Sos_function_descr* Sos_function_register::function_descr( const char* name, int param_count )
{
    Sos_function_descr* func_descr = function_descr_or_0( name, param_count );
    if( !func_descr )  throw_xc( "SOS-1311", name, param_count );
    return func_descr;
}

//---------------------------------------------------Sos_function_register::function_descr_or_0

Sos_function_descr* Sos_function_register::function_descr_or_0( const char* name_par,
                                                                int param_count )
{
    Sos_string name = name_par;
    for( int i = 0; i <= _func_array.last_index(); i++ )
    {
        Sos_function_descr* func_descr = _func_array[ i ];
        if( stricmp( c_str( func_descr->_name ), c_str( name ) ) == 0 
         && func_descr->_param_count == param_count )  return func_descr;
    }

    return NULL;
}

} //namespace sos
