/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#define MODULE_NAME "sosid"
#define COPYRIGHT   " SOS GmbH Berlin"
#define AUTHOR      "Joacim Zschimmer"

#if defined JZ_TEST

/* Ein eigener Zeiger-Typ Sos_id wie in Rapid.
   Fr automatische Garbage Collection.

   Kann fr persistente Objekte erweitert werden:
      Sos_id_as<T>::store( Sos_storage_location* )

      mit Sos_storage_location::Sos_storage_location( Sos_file*, Key* );

      Das Sos_register verwaltet dann den Ort und ldt das Objekt automatisch bei Bedarf.


   Ein Objekt kann als verwerfbar gekennzeichnet werden. Es wird dann bei Speichermangel gelscht
   und wieder automatisch angelegt. Fr Puffer (eine Eigenschaft von Area? Klasse Discardable).
*/

typedef int Sos_id_index;


struct Sos_id
{
                                 Sos_id                 ( void* );
                                 Sos_id                 ( const Sos_id& );
                                ~Sos_id                 ();

    T*                           ptr                    () const;

  private:
    void                         operator =             ( int );            // Nicht implementiert

    Sos_id_index                _index;
};


template< class T >
struct Sos_id_as<T> : Sos_id
{
                                 Sos_id                 ( T* );
    T*                           operator ->            () const;
    T&                           operator *             () const;
    T*                           ptr                    () const;
};


struct Sos_register
{
    void*                        ptr                    ( const Sos_id& );

  private:
    friend                       Sos_id::Sos_id         ( void* );

    int                          register               ( void* );
    void                         incr_ref               ( Sos_id_index );
    void                         decr_ref               ( Sos_id_index );   // Lscht bei 0
};

Sos_register sos_register;


inline Sos_id::Sos_id( void* ptr )
{
    _index = sos_register.register( ptr );
}

inline Sos_id::Sos_id( const Sos_id& id )
{
    _index = id._index;
    sos_register.incr_ref( _index );
}

inline Sos_id::~Sos_id()
{
    sos_register.decr_ref( _index );
}

inline void* Sos_id::ptr()
{
    return sos_register.ptr( _index );
}

template< class T >
inline Sos_id_as<T>::Sos_id_as( T* object_ptr )
:
    Sos_id( object_ptr );
{
}

template< class T >
inline Sos_id_as<T>::operator-> ()
{
    return (T*)ptr();
}

template< class T >
inline T& Sos_id_as<T>::operator* ()
{
    return *(T*)ptr();
}



#endif
