/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// Mengenoperationen                                    (c) SOS GmbH Berlin
//
// 20. 9.89     Joacim Zschimmer

#ifndef __SOSSET_H
#define __SOSSET_H

#include <assert.h>

#ifndef SYSTEM_GNU
#   include <memory.h>
#endif

namespace sos
{

inline void set_incl (Byte set [], uint4 elem)  {
    set [ (uint) (elem >> 3) ] |= (Byte) (1 << ((Byte) elem & 7));
}


inline void set_excl (Byte set [], uint4 elem)  {
    set [ (uint) (elem >> 3) ] &= ~ (Byte) (1 << ((Byte) elem & 7));
}


inline Bool in_set (const Byte set [], uint4 elem)  {
    return ( set [ (uint) (elem >> 3) ] & (Byte) (1 << ((Byte) elem & 7)) ) != 0;
}



struct Bit_set
{
                                Bit_set                 ( uint4 count );
                                Bit_set                 ( uint4 count, void* bit_map_ptr );
                               ~Bit_set                 ();

    Bool                        operator []             ( uint4 i ) const               { return is_elem( i ); }
    Bit_set&                    operator |=             ( const Bit_set& o )            { unite( o ); return *this; }
    Bit_set&                    operator &=             ( const Bit_set& o )            { cut( o ); return *this; }

    Bool                        valid                   () const;
    void                        clear                   ()                              { memset( _bit_map_ptr, 0, _size ); }
    void                        include                 ( uint4 elem );
    void                        include                 ( uint4 elem, uint4 count );
    void                        exclude                 ( uint4 elem );
    void                        exclude                 ( uint4 elem, uint4 count );
    int4                        first_excluded          () const  { return scan( 0, 0 ); }
    inline Bool                 is_elem                 ( uint4 elem ) const;
    void                        unite                   ( const Bit_set &set2 );
    void                        cut                     ( const Bit_set &set2 );
    void                        invert                  ();
    uint4                       cardinal                () const;
    Bool                        disjunct                ( const Bit_set &set2 ) const;
    int4                        scan                    ( int4 start, Bool value ) const;
    int4                        scan                    ( int4 start, Bool value, int4 anzahl ) const;

  protected:
    const uint                 _size;
    Bool                       _delete;
    Byte*                      _bit_map_ptr;
};


//---------------------------------------------------------------------------Sos_set<TYPE,SIZE>

template< int SIZE >
struct Sos_set : Bit_set
{
                                Sos_set                 ();
                                Sos_set                 ( const Sos_set<SIZE>& o );

    Sos_set<SIZE>&              operator =              ( const Sos_set<SIZE>& o ); //???       { memcpy( _bits, o._bits, sizeof _bits ); _bit_map_ptr = _bits; return *this; }

    Sos_set<SIZE>&              operator |=             ( const Sos_set<SIZE>& o )            { unite( o ); return *this; }
    Sos_set<SIZE>&              operator &=             ( const Sos_set<SIZE>& o )            { cut( o ); return *this; }
    Sos_set<SIZE>               operator |              ( const Sos_set<SIZE>& o )            { Sos_set<SIZE> set; set |= o; return set; }
    Sos_set<SIZE>               operator &              ( const Sos_set<SIZE>& o )            { Sos_set<SIZE> set; set &= o; return set; }
    Sos_set<SIZE>               operator ~              ()                                    { Sos_set<SIZE> set = *this; set.invert(); return set; }

  private:
    Byte                       _bits [ ( SIZE + 7 ) / 8 ];
};

//--------------------------------------------------------------------------Sos_set::Sos_set()

template<int SIZE>
inline Sos_set<SIZE>::Sos_set()
:
    Bit_set ( SIZE, _bits )
{
    memset( _bits, 0, sizeof _bits );
}

//--------------------------------------------------------------------------Sos_set::Sos_set()

template<int SIZE>
inline Sos_set<SIZE>::Sos_set( const Sos_set<SIZE>& o )
:
    Bit_set ( SIZE, _bits )
{
    memcpy( _bits, o._bits, sizeof _bits );
    _bit_map_ptr = _bits;
}

} //namespace sos


#include "sosset.inl"

#endif
