/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
#include "precomp.h"
//#define MODULE_NAME "sosstrea"
//#define COPYRIGHT   "(c) 1995 SOS GmbH Berlin"
//#define AUTHOR      "Joacim Zschimmer"

//#pragma implementation

#include "../kram/sos.h"
#include "../kram/xception.h"
#include "../kram/sosstrea.h"

#if !defined SYSTEM_INCLUDE_TEMPLATES && defined JZ_TEST
#   include "../kram/sosstrea.tpl"
#endif

using namespace std;
namespace sos {

//---------------------------------------------------------------------------------write_iso_string

void write_iso_string( Sos_binary_ostream* s, const char* string, uint field_size )
/*
    Schreibt den String im Code ISO 8859-1 (ASCII).
    Der String wird im lolalen Code bergeben.
    Der String wird auf die Lnge field_size abgeschnitten oder mit Blanks aufgefllt.
    Beim Abschneiden kann eine Exception auftreten.
*/
{
    int l = min( strlen( string ), field_size );

    s->write_fixed( string, l );
    s->write_byte_repeated( 0x40, field_size - l );
}

//----------------------------------------------------------------------------------read_iso_string

void read_iso_string( Sos_binary_istream* s, char* string_buffer, uint field_size )
/*
    Konvertiert einen ISO 8859-1-String fester Lnge in einen 0-terminierten String im
    lokalen Code, bei dem die Blanks abgeschnitten worden sind.
    Der Puffer fr den String mu ein Byte grer sein als field_size.
*/
{
    const Byte* p0 = (const Byte*) s->read_bytes( 0 );
    const Byte* p  = p0 + field_size;

    while( p > p0  &&  *(p-1) == 0x20 )  p--;        // Blanks abschneiden

    uint l = p - p0;

    memcpy( string_buffer, p0, l );

    string_buffer[ l ] = '\0';

    s->skip_bytes( field_size );
}

//-------------------------------------------------------------------------------read_ebcdic_string

void read_ebcdic_string( Sos_binary_istream* s, char* string_buffer, uint field_size )
/*
    Konvertiert einen EBCDIC-String fester Lnge in einen 0-terminierten String im
    lokalen Code, bei dem die Blanks abgeschnitten worden sind.
    Der Puffer fr den String mu ein Byte grer sein als field_size.
*/
{
    const Byte* p0 = (const Byte*) s->read_bytes( 0 );
    const Byte* p  = p0 + field_size;

    while( p > p0  &&  *(p-1) == 0x40 )  p--;        // Blanks abschneiden

    uint l = p - p0;

    xlat( string_buffer, p0, l, ebc2iso );

    string_buffer[ l ] = '\0';

    s->skip_bytes( field_size );
}

//------------------------------------------------------------------------------write_ebcdic_string

void write_ebcdic_string( Sos_binary_ostream* s, const char* string, uint field_size )
/*
    Schreibt den String im Code ISO 8859-1 (EBCDIC).
    Der String wird im lolalen Code bergeben.
    Der String wird auf die Lnge field_size abgeschnitten oder mit Blanks aufgefllt.
    Beim Abschneiden kann eine Exception auftreten.
*/
{
    int l = min( strlen( string ), field_size );

    xlat( s->space( l ), string, l, iso2ebc );
    s->write_byte_repeated( 0x40, field_size - l );
}

//-------------------------------------------------------------------------------read_ebcdic_string

void read_ebcdic_string( Sos_binary_istream* s, Area* buffer_ptr, uint field_size )
/*
    Konvertiert einen EBCDIC-String fester Lnge in einen 0-terminierten String im
    lokalen Code, bei dem die Blanks abgeschnitten worden sind.
    Der Puffer fr den String mu ein Byte grer sein als field_size.
*/
{
    buffer_ptr->allocate_min( field_size + 1 );
    read_ebcdic_string( s, buffer_ptr->char_ptr(), field_size );
    buffer_ptr->length( strlen( buffer_ptr->char_ptr() ));
}

//------------------------------------------------------------------------------write_ebcdic_string

void write_ebcdic_string( Sos_binary_ostream* s, const Const_area& string, uint field_size )
/*
    Schreibt den String im Code ISO 8859-1 (EBCDIC).
    Der String wird im lolalen Code bergeben.
    Der String wird auf die Lnge field_size abgeschnitten oder mit Blanks aufgefllt.
    Beim Abschneiden kann eine Exception auftreten.
*/
{
    int l = min( length( string ), field_size );

    xlat( s->space( l ), string.char_ptr(), l, iso2ebc );
    s->write_byte_repeated( 0x40, field_size - l );
}



} //namespace sos
