/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: thread_data.h 3546 2005-04-03 08:30:29Z jz $

#ifndef __SOS_THREAD_DATA_H
#define __SOS_THREAD_DATA_H

#include "thread_semaphore.h"
#include "log.h"

namespace sos {

//--------------------------------------------------------------------------------------Thread_data

template< class THREAD_DATA >
struct Thread_data
{
                                Thread_data             ();
                               ~Thread_data             ();

    void                        thread_detach           ();
    THREAD_DATA*                get                     ();

    THREAD_DATA                 operator *              ()                                      { return *get(); }
    THREAD_DATA*                operator ->             ()                                      { return get(); }
    THREAD_DATA*                operator +              ()                                      { return get(); }
  //                            operator THREAD_DATA*   ()                                      { return get(); }

                                operator bool           ()                                      { return allocated(); }
    bool                        operator !              ()                                      { return !allocated(); }
    bool                        allocated               ();

  private:

#   ifdef SYSTEM_WIN
        Thread_semaphore       _lock;
        int                    _thread_index;
#    else
        THREAD_DATA*           _data;
#   endif

};

//-------------------------------------------------------------------------------------------------

#ifdef SYSTEM_WIN

    template< class THREAD_DATA >
    Thread_data<THREAD_DATA>::Thread_data()
    : 
        _lock("Thread_data"),
        _thread_index(TLS_OUT_OF_INDEXES)
    {
        //_tls_index = TlsAlloc();
    }


    template< class THREAD_DATA >
    Thread_data<THREAD_DATA>::~Thread_data()
    {
        LOGI( "~Thread_data\n" );

        if( _thread_index != TLS_OUT_OF_INDEXES )
        {
            thread_detach();

            LOG( "TlsFree()\n" );
            TlsFree( _thread_index );
            _thread_index = TLS_OUT_OF_INDEXES;
        }
    }

    
    template< class THREAD_DATA >
    void Thread_data<THREAD_DATA>::thread_detach()
    {
        LOGI( "Thread_data::thread_detach\n" );
        
        if( _thread_index != TLS_OUT_OF_INDEXES )
        {
            THREAD_DATA* thread_data = (THREAD_DATA*)TlsGetValue( _thread_index );
            if( thread_data )
            {
                LOGI( "Thread_data::thread_detach delete " << (void*)thread_data << '\n' );
                delete thread_data;
                TlsSetValue( _thread_index, NULL );
            }
        }
    }

    
    template< class THREAD_DATA >
    THREAD_DATA* Thread_data<THREAD_DATA>::get() 
    { 
        if( _thread_index == TLS_OUT_OF_INDEXES )
        {
            THREAD_LOCK( _lock )
            if( _thread_index == TLS_OUT_OF_INDEXES )
            {
                LOG( "Thread_data::get TlsAlloc()\n" );
                _thread_index = TlsAlloc();  
                if( _thread_index == TLS_OUT_OF_INDEXES )  throw_mswin_error( "TlsAlloc" );
            }
        }

        THREAD_DATA* thread_data = (THREAD_DATA*)TlsGetValue( _thread_index );
        
        if( !thread_data )
        {
            LOG( "Thread_data::get new Thread_data\n" );
            thread_data = new THREAD_DATA;
            TlsSetValue( _thread_index, thread_data );
        }

        return thread_data;
    }


    template< class THREAD_DATA >
    bool Thread_data<THREAD_DATA>::allocated() 
    { 
        return _thread_index != TLS_OUT_OF_INDEXES  &&  TlsGetValue( _thread_index ) != NULL;
    }

#else

    template< class THREAD_DATA >
    Thread_data<THREAD_DATA>::Thread_data() 
    {
        _data = new THREAD_DATA;
    }


    template< class THREAD_DATA >
    Thread_data<THREAD_DATA>::~Thread_data()
    {
        delete _data;
        _data = NULL;
    }

    
    template< class THREAD_DATA >
    void Thread_data<THREAD_DATA>::thread_detach()
    {
    }

    
    template< class THREAD_DATA >
    THREAD_DATA* Thread_data<THREAD_DATA>::get() 
    { 
        return _data;
    }

    template< class THREAD_DATA >
    bool Thread_data<THREAD_DATA>::allocated() 
    { 
        return true;
    }

#endif

//-------------------------------------------------------------------------------------------------

} //namespace sos

#endif
