/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: folder_replication.cxx 5417 2007-12-06 12:05:42Z jz $        Joacim Zschimmer, Zschimmer GmbH, http://www.zschimmer.com

#include "spooler.h"
#include "../zschimmer/directory_lister.h"

namespace sos {
namespace scheduler {
namespace folder {
namespace replication {

using namespace zschimmer::file;

//-------------------------------------------------------------------------------------------------

/*
    Mit Supervisor verbinden

    <configuration.fetch_updated_files scheduler_id="..." signal_next_change_at_udp_port="...">
        <configuration.directory directory="/" timestamp="..">
            <configuration.file path="path" timestamp=".." md5="..."/>

            <configuration.directory directory="directory/" timestamp="..">
                <configuration.file path="path" timestamp=".." md5="..."/>
            </configuration.file>
        <configuration.directory>
    </configuration.fetch_updated_files>


    Antwort:

    <configuration.directory directory="/" timestamp="..">
        <configuration.file path="path" timestamp=".." md5="...">
            <content encoding="base64">...</content>
        </configuration.file>

        <configuration.file path="path" removed="yes"/>

        <configuration.directory directory="directory/" timestamp="..">
            <configuration.file path="path" timestamp="..">
                <content encoding="base64">...</content>
            </configuration.file>
        </configuration.file>
    </configuration.directory>


    UDP:

    <configuration.changed/>




    
    START

    Verbindung zum Supervisor aufbauen
    geht nicht?
        Normal starten
        Weiter versuchen, die Verbindung zum Supervisor aufzubauen

    Verbindung aufgebaut?
        Wenn noch nicht eigene Konfigurationsdateien geladen (state<active)
            Eigene Konfigurationsverzeichnisse lesen, Timestamp und MD5 einsammeln
        Sonst bereits geladene Informationen benutzen

        an Supervisor senden und dabei UDP-Signal fr nchste nderung bestellen


        Supervisor registriert Scheduler
        merkt sich am Scheduler gespeicherte Dateien (Zeitstempel, MD5)
        Startet Verzeichnisberwachung (sowieso durch folder.cxx)
        vergleicht Dateien und liefert nderungen



    Verzeichnisnderung am Supervisor:
        Fr jeden registrierten Scheduler mit bestelltem UDP-Signal:
            Verzeichnisse durchgehen und nderung feststellen
            Falls gendert, UDP-Signal
            UDP-Signal wiederholen, wenn Scheduler Dateien nicht abholt, aber seine TCP-Verbindung noch besteht


    ZEITSTEMPEL
        in UTC



    QUOTED-PRINTABLE, verndert fr Scheduler "scheduler-printable"
        09, 0A, 20 bis 7C -> durchlassen
        00 bis 1F -> U+7E '~', U+7E '~', U+40 '@' bis U+5F      U+00 -> ~~@   U+0D -> ~~M
        80 bis 9E -> U+7E '~', U+7E '~', U+60 '`' bis U+7E      U+80 -> ~~`
        9F bis FC -> U+7E '~', U+20 bis U+7D                    U+9F -> ~~    U+FD -> ~~}
        FD bis FF -> U+7E '~', U+7E '~', U+30 '0' bis U+32 '2'  U+FD -> ~~0   U+FF -> ~~3
        XML: Erstes und letztes Blank codieren: "~ "


    ODER VERNDERTES UTF-8 (nein!)
        09, 0A, 20 bis 7E -> durchlassen
        00 bis 1F -> wie 100 bis 11F in UTF-8
        80 bis 9F -> wie 120 bis 140 in UTF-8
        A0 bis FF -> in UTF-8
            

*/

bool XXX::adjust_with_directory( const File_path& directory_path )
{
    file::Directory_lister dir ( _log );
    
    bool ok = dir.open( root_folder_path, directory_path );

    if( ok )
    {
        while( ptr<file::File_info> file_info = dir.get() )
        {
            string filename  = file_info->path().name();

            if( file_info->is_directory() )
            {
                xml_writer->begin_element( "configuration.directory" );
                xml_writer->set_attribute( "path"     , File_path( _directory, file_name ) );
                xml_writer->set_attribute( "timestamp", file_info->timestamp() );

                f( File_path( directory_path, filename ) );

                xml_writer->end_element( "configuration.directory" );
            }
            else
            {
                string name      = object_name_of_filename( filename );
                string extension = extension_of_filename( filename );
                
                if( name != "" )
                {
                    Typed_folder_map::iterator it = _typed_folder_map.find( extension );
                    if( it != _typed_folder_map.end() ) 
                    {
                        xml_writer->begin_element( "configuration.file" );
                        xml_writer->set_attribute( "path"     , File_path( _directory, file_name ) );
                        xml_writer->set_attribute( "timestamp", file_info->timestamp() );
                        xml_writer->set_attribute( "md5"      , md5( string_from_file( File_path( _directory, file_name ) ) ) );
                        xml_writer->end_element( "configuration.file" );
                    }
                }
            }
        }
    }
}

//-----------------------------------------------------------Subfolder_folder::on_base_file_changed

bool Subfolder_folder::on_base_file_changed( File_based* file_based, const Base_file_info* base_file_info, double now )
{
    bool    something_changed = false;
    Folder* subfolder         = static_cast<Folder*>( file_based );

    if( !subfolder )
    {
        ptr<Folder> new_subfolder = Z_NEW( Folder( subsystem(), folder() ) );
        new_subfolder->set_folder_path( folder()->path() );
        new_subfolder->set_name( base_file_info->_normalized_name );
        new_subfolder->fix_name();
        new_subfolder->set_base_file_info( *base_file_info );
        add_file_based( new_subfolder );
        something_changed = true;

        new_subfolder->activate();
        if( new_subfolder->file_based_state() >= File_based::s_loaded )  new_subfolder->adjust_with_directory( now );
    }
    else
    {
        subfolder->_file_is_removed = base_file_info == NULL;
        subfolder->_remove_xc       = zschimmer::Xc();

        if( base_file_info )
        {
            subfolder->set_base_file_info( *base_file_info );
            subfolder->set_to_be_removed( false ); 
            something_changed = subfolder->adjust_with_directory( now );    
        }
        else
        if( !subfolder->is_to_be_removed() ) 
        {
            string p = folder()->make_path( subfolder->base_file_info()._filename );
            subfolder->log()->info( message_string( "SCHEDULER-898" ) );

            subfolder->adjust_with_directory( now );
            subfolder->remove();

            something_changed = true;
        }
        else
        {
            // Verzeichnis ist gelscht, aber es leben vielleicht noch Objekte, die gelscht werden mssen.
            // adjust_with_directory() wird diese mit handle_replace_or_remove_candidates() lschen
            something_changed = subfolder->adjust_with_directory( now );    
        }
    }

    return something_changed;
}

//--------------------------------------------------------------Typed_folder::adjust_with_directory
    
bool Typed_folder::adjust_with_directory( const list<Base_file_info>& file_info_list, double now )
{
    bool                          something_changed  = false;
    vector<const Base_file_info*> ordered_file_infos;     // Geordnete Liste der vorgefundenen Dateien    
    vector<File_based*>           ordered_file_baseds;    // Geordnete Liste der bereits bekannten (geladenen) Dateien

    ordered_file_infos.reserve( file_info_list.size() );
    Z_FOR_EACH_CONST( list<Base_file_info>, file_info_list, it )  ordered_file_infos.push_back( &*it );
    sort( ordered_file_infos.begin(), ordered_file_infos.end(), Base_file_info::less_dereferenced );

    ordered_file_baseds.reserve( _file_based_map.size() );
    Z_FOR_EACH( File_based_map, _file_based_map, fb )  ordered_file_baseds.push_back( &*fb->second );
    sort( ordered_file_baseds.begin(), ordered_file_baseds.end(), File_based::less_dereferenced );


    vector<const Base_file_info*>::iterator fi = ordered_file_infos.begin();
    vector<File_based*          >::iterator fb = ordered_file_baseds.begin();      // Vorgefundene Dateien mit geladenenen Dateien abgleichen

    while( fi != ordered_file_infos.end()  ||
           fb != ordered_file_baseds.end() )
    {
        /// Dateinamen gleich Objektnamen?

        while( fb != ordered_file_baseds.end()  &&
               fi != ordered_file_infos.end()  &&
               (*fb)->_base_file_info._normalized_name == (*fi)->_normalized_name )
        {
            something_changed |= on_base_file_changed( *fb, *fi, now );
            fi++, fb++;
        }



        /// Dateien hinzugefgt?

        while( fi != ordered_file_infos.end()  &&
               ( fb == ordered_file_baseds.end()  ||  (*fi)->_normalized_name < (*fb)->_base_file_info._normalized_name ) )
        {
            something_changed |= on_base_file_changed( (File_based*)NULL, (*fi), now );
            fi++;
        }

        assert( fi == ordered_file_infos.end()  || 
                fb == ordered_file_baseds.end() ||
                (*fi)->_normalized_name >= (*fb)->_base_file_info._normalized_name );
        


        /// Dateien gelscht?

        while( fb != ordered_file_baseds.end()  &&
               ( fi == ordered_file_infos.end()  ||  (*fi)->_normalized_name > (*fb)->_base_file_info._normalized_name ) )  // Datei entfernt?
        {
            something_changed |= on_base_file_changed( *fb, NULL, now );
            fb++;
        }

        assert( fb == ordered_file_baseds.end()  ||
                fi == ordered_file_infos.end()  ||
                (*fi)->_normalized_name <= (*fb)->_base_file_info._normalized_name );
    }

    return something_changed;
}

//-------------------------------------------------------------------------------------------------

} //namespace replication
} //namespace folder
} //namespace scheduler
} //namespace sos
