/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: spooler_job.h 6007 2009-05-07 05:52:24Z jz $        Joacim Zschimmer, Zschimmer GmbH, http://www.zschimmer.com 

#ifndef __SPOOLER_JOB_H
#define __SPOOLER_JOB_H

namespace sos {
namespace scheduler {

//-------------------------------------------------------------------------------------------------

struct Task;
struct Module_task;
struct Job_folder;
struct Job_schedule_use;

//------------------------------------------------------------------------------Combined_job_nodes

struct Combined_job_nodes : Object
{
                                Combined_job_nodes          ( Job* );
                               ~Combined_job_nodes          ();

    void                        close                       ();
    bool                        is_empty                    () const                                { return _job_node_set.empty(); }
    Order_queue*                any_order_queue             () const;
    bool                        request_order               ( const Time& now, const string& cause );
    void                        withdraw_order_requests     ();
    Time                        next_time                   ();
    Order*                      fetch_and_occupy_order      ( const Time& now, const string& cause, Task* occupying_task );
    void                        connect_with_order_queues   ();
    void                        connect_job_node            ( job_chain::Job_node* );
    void                        disconnect_job_node         ( job_chain::Job_node* );
    xml::Element_ptr            dom_element                 ( const xml::Document_ptr&, const Show_what&, Job_chain* );
    Spooler*                    spooler                     () const                                { return _spooler; }

  private:
    Fill_zero                  _zero_;
    Job*                       _job;
    Spooler*                   _spooler;
    typedef stdext::hash_set< job_chain::Job_node* >  Job_node_set;
    Job_node_set               _job_node_set;
};

//----------------------------------------------------------------------------------------------Job

struct Job : file_based< Job, Job_folder, Job_subsystem_interface >,
             Object
{
    enum State
    {
        // Mgliche Verbesserung: Alle Zustnde, die auch File_based kennt, hier streichen. 
        // Stattdessen s_inactive, ein Zustand, der sich in die File_based-Zustnde aufspaltet
        // Die brigen Zustnde gelten fr s_active.
        // s_stopped durch _is_stopped und s_pending ersetzen. Auch mit anderen Zustnden kombinierbar.
        // s_stopping streichen? stop(false) lsst Tasks weiterlaufen. Ist das klug? Sollten ein gestoppte Job nicht immer die Tasks beenden?
        // s_running streichen? Besagt nur, dass eine Task luft, whrend bei s_pending keine Task luft. s_pending, s_running -> s_ready?

        s_not_initialized,
        s_initialized,
        s_loaded,   
        s_stopping,             // Wird gestoppt (Zustand, solange noch Tasks laufen, danach s_stopped). Sollte durch _is_stopped ersetzt werden!
        s_stopped,              // Gestoppt (z.B. wegen Fehler). Keine Task wird gestartet.
      //s_read_error,           // Skript kann nicht aus Datei (include) gelesen werden
        s_error,                // Ein Fehler ist aufgetreten (nicht vom Skript), der Job ist nicht mehr aufrufbar.
      //s_incomplete,           // Eine Resource fehlt, Lock oder Process_class
        s_pending,              // Warten auf Start
        s_running,              // Mindestens eine Task luft (die Tasks knnen aber ausgesetzt, also gerade nicht aktiv sein: s_suspended etc.)
        s__max
    };

    enum State_cmd
    {
        sc_none,
        sc_stop,                // s_running || s_suspended  -> s_stopped
        sc_unstop,              // s_stopped                 -> s_pending
        sc_start,               // s_pending                 -> s_running
        sc_wake,                // s_pending || s_running    -> s_running
        sc_end,                 // s_running                 -> s_pending
        sc_suspend,             // s_running                 -> s_suspended
        sc_continue,            // s_suspended               -> s_running
        sc_reread,              // Null-Operation, Jira JS-208
        sc_remove,              // Spooler::remove_job
        sc__max
    };


    struct Task_queue : Object
    {
        typedef list< ptr<Task> >       Queue;
        typedef Queue::iterator         iterator;

        enum Why_remove
        {
            w_task_killed,
            w_task_started
        };

        
                                Task_queue                  ( Job* job )                            : _job(job), _spooler(job->_spooler) {}
        
        void                    clear                       ()                                      { _queue.clear(); }
        int                     size                        () const                                { return _queue.size(); }
        bool                    empty                       () const                                { return _queue.empty(); }
        iterator                begin                       ()                                      { return _queue.begin(); }
        iterator                end                         ()                                      { return _queue.end(); }

        void                    enqueue_task                ( const ptr<Task>& );
        bool                    remove_task                 ( int task_id, Why_remove );
        void                    remove_task_from_db         ( int task_id );
      //bool                    has_task_waiting_for_period ();
        void                    move_to_new_job             ( Job* );
        Time                    next_start_time             ();
        void                    append_calendar_dom_elements( const xml::Element_ptr&, Show_calendar_options* );

      private:
        Job*                   _job;
        Spooler* const         _spooler;
        Queue                  _queue;
    };


    typedef list< ptr<Task> >                   Task_list;
    typedef list< ptr<Directory_watcher> >      Directory_watcher_list;
    typedef map< int, Time >                    Delay_after_error;
    typedef map< int, Time >                    Delay_order_after_setback;

    static const bool           force_start_default;


                                Job                         ( Scheduler*, const string& name = "", const ptr<Module>& = NULL );
    virtual                    ~Job                         (); 

    STDMETHODIMP_(ULONG)        AddRef                      ()                                      { return Object::AddRef(); }
    STDMETHODIMP_(ULONG)        Release                     ()                                      { return Object::Release(); }


    // Scheduler_object:
    virtual string              obj_name                    () const                                { return "Job " + path().without_slash(); }
    virtual IDispatch*          idispatch                   ()                                      { return _com_job; }
    virtual void                write_element_attributes    ( const xml::Element_ptr& element ) const { element.setAttribute( "job", path().with_slash() ); }


    // File_based:
    bool                        on_initialize               (); 
    bool                        on_load                     (); 
  //void                        on_load                     ( Transaction* );
    bool                        on_activate                 ();
    list<Requisite_path>        missing_requisites          ();
    bool                        can_be_removed_now          ();


    // Dependant:
    bool                        on_requisite_loaded         ( File_based* );
    bool                        on_requisite_to_be_removed  ( File_based* );
  //void                        on_requisite_removed        ( File_based* );


    // Has_includes:
  //void                        on_include_changed          ();


    Job_folder*                 job_folder                  () const                                { return typed_folder(); }


    void                    set_dom                         ( const xml::Element_ptr& );
    Time                        get_step_duration_or_percentage( const string& value, const Time& deflt );
    Time                        average_step_duration       ( const Time& deflt );
    void                        add_on_exit_commands_element( const xml::Element_ptr& commands_element );

    bool                        is_visible_in_xml_folder    ( const Show_what& ) const;
    xml::Element_ptr            dom_element                 ( const xml::Document_ptr& document, const Show_what& show_what )  { return dom_element( document, show_what, (Job_chain*)NULL ); }
    xml::Element_ptr            dom_element                 ( const xml::Document_ptr&, const Show_what&, Job_chain*  );
    void                        append_calendar_dom_elements( const xml::Element_ptr&, Show_calendar_options* );


    void                        init_start_when_directory_changed( Task* = NULL );
    void                        prepare_on_exit_commands    ();
    void                        set_log                     ();
    Schedule_use*               schedule_use                () const;
    void                        on_schedule_loaded          ();
    void                        on_schedule_modified        ();
    bool                        on_schedule_to_be_removed   ();
  //void                        on_schedule_removed         ();


    State_cmd                   state_cmd                   () const                                { return _state_cmd; }
    State                       state                       () const                                { return _state; }
    string                      job_state                   ();
    string                      include_path                () const;
    string                      title                       ()                                      { return _title; }
    string                      profile_section             ();
    void                    set_visible                     ()                                      { if( _visible == visible_no )  _visible = visible_yes; }
    void                    set_visible                     ( Visibility v )                        { _visible = v; }
    bool                     is_visible                     () const                                { return _visible == visible_yes; }
    bool                        temporary                   () const                                { return _temporary; }

    void                        on_prepare_to_remove        ();
    void                        on_remove_now               ();
    zschimmer::Xc               remove_error                ();

  //void                        prepare_to_replace          ();
  //bool                        can_be_replaced_now         ();
  //Job*                        on_replace_now              ();

    void                        set_delay_after_error       ( int error_steps, const string& delay );
    void                        set_delay_after_error       ( int error_steps, const Time& delay )  { _log->debug9( "delay_after_error["        +as_string(error_steps)+"]="+delay.as_string() ); _delay_after_error[ error_steps ] = delay; }
    void                        set_stop_after_error        ( int error_steps )                     { _log->debug9( "delay_after_error["        +as_string(error_steps)+"]=\"STOP\""           ); _delay_after_error[ error_steps ] = Time::never; }
    void                        clear_delay_after_error     ()                                      { _log->debug9( "clear_delay_after_error()" ); _delay_after_error.clear(); }
    void                        set_delay_order_after_setback( int setbacks, const string& delay );
    void                        set_delay_order_after_setback( int setbacks, const Time& delay )    { _log->debug9( "delay_order_after_setback["+as_string(setbacks   )+"]="+delay.as_string() ); _delay_order_after_setback[setbacks   ] = delay; }
    Time                        get_delay_order_after_setback( int setback_count );
    void                        set_max_order_setbacks      ( int n )                               { _log->debug9( "max_order_setbacks="+as_string(n) ); _max_order_setbacks = n; }
    int                         max_order_setbacks          () const                                { return _max_order_setbacks; }
    void                        database_record_store       ();
    void                        database_record_remove      ();
    void                        database_record_load        ( Read_transaction* );
    void                        load_tasks_from_db          ( Read_transaction* );
    xml::Element_ptr            read_history                ( const xml::Document_ptr& doc, int id, int n, const Show_what& show ) { return _history.read_tail( doc, id, n, show ); }

    void                        close                       ();

    ptr<Task>                   start                       ( const ptr<spooler_com::Ivariable_set>& params, const string& task_name, const Time& = 0 );
    void                        enqueue_task                ( Task* );
    void                        start_when_directory_changed( const string& directory_name, const string& filename_pattern );
    void                        clear_when_directory_changed();
    bool                        check_for_changed_directory ( const Time& now );
    void                        update_changed_directories  ( Directory_watcher* );
    string                      trigger_files               ( Task* task = NULL );
    void                        interrupt_script            ();
    void                        select_period               ( const Time& );
    bool                        is_in_period                ( const Time& );
    bool                        queue_filled                ()                                      { return !_task_queue->empty(); }
    
    void                        on_task_finished            ( Task* );                              // Task::finished() ruft das
    void                        check_min_tasks             ( const string& cause );                // Setzt _start_min_tasks
    bool                        above_min_tasks             () const;
    bool                        below_min_tasks             () const;
    bool                        should_start_task_because_of_min_tasks();
    int                         not_ending_tasks_count      () const;

    ptr<Task>                   create_task                 ( const ptr<spooler_com::Ivariable_set>& params, const string& task_name, bool force = force_start_default, const Time& = Time::never );
    ptr<Task>                   create_task                 ( const ptr<spooler_com::Ivariable_set>& params, const string& task_name, bool force, const Time&, int id );
    ptr<Task>                   get_task_from_queue         ( const Time& now );
    void                        run_task                    ( const ptr<Task>&  );

    void                        remove_running_task         ( Task* );
    void                        stop                        ( bool end_all_tasks );
    void                        stop_simply                 ( bool end_all_tasks );
    void                        stop_after_task_error       ( const string& error_message );   // _ignore_error verhindert stop()
    bool                        stops_on_task_error         ()                                      { return _stop_on_error; }
    void                        reset_scheduling            ();
    void                        set_next_start_time         ( const Time& now, bool repeat = false );
    void                        set_next_time               ( const Time& );
    void                        calculate_next_time         ( const Time& now );
    void                        signal_earlier_order        ( Order* );
    void                        signal_earlier_order        ( const Time& next_time, const string& order_name, const string& function );

    Time                        next_time                   ()                                      { return _next_time; }
    Time                        next_start_time             ();
    bool                        has_next_start_time         ()                                      { return next_start_time() < Time::never; }
    bool                     is_machine_resumable           () const                                { return _machine_resumable; }
    void                    set_machine_resumable           ( bool b )                              { _machine_resumable = b; }

    lock::Requestor*            lock_requestor_or_null      () const                                { return _lock_requestor; }
  //void                        on_removing_lock            ( lock::Lock* );

    bool                        execute_state_cmd           ();
    ptr<Task>                   task_to_start               ();
    bool                        do_something                ();

    void                    set_repeat                      ( double seconds )                      { _log->debug( "repeat=" + as_string(seconds) ),  _repeat = seconds; }
    Time                        repeat                      ()                                      { return _repeat; }

    void                        set_state                   ( State );
    void                        set_state_cmd               ( State_cmd );
    void                        end_tasks                   ( const string& task_warning );
    void                        kill_task                   ( int task_id, bool immediately = false );
    void                        kill_queued_task            ( int task_id );

    string                      state_name                  ()                                      { return state_name( _state ); }
    static string               state_name                  ( State );
    static State                as_state                    ( const string& );
    
    string                      state_cmd_name              ()                                      { return state_cmd_name( _state_cmd ); }
    static string               state_cmd_name              ( State_cmd );
    static State_cmd            as_state_cmd                ( const string& );

    void                        set_state_text              ( const string& text )                  { _state_text = text, _log->debug9( "state_text = " + text ); }

    void                        set_error_xc                ( const Xc& );
    void                        set_error_xc_only           ( const Xc& );
    void                        set_error                   ( const exception& );
    void                        reset_error                 ()                                      { _error = NULL,  _log->reset_highest_level(); }
    void                        set_job_error               ( const exception& );

    void                        signal                      ( const string& signal_name );
    void                        notify_a_process_is_idle    ();                                     // Vielleicht wird bald ein Prozess frei?
    void                        remove_waiting_job_from_process_list();
    void                        on_process_class_active     ( Process_class* );

    ptr<Com_job>&               com_job                     ()                                      { return _com_job; }

  //Order_queue*                order_queue                 () const                                { return _order_queue; }
    Order_queue*                any_order_queue             () const;
    Combined_job_nodes*         combined_job_nodes          () const                                { return _combined_job_nodes; }
    bool                        connect_job_node            ( job_chain::Job_node* );
    void                        disconnect_job_node         ( job_chain::Job_node* );
    bool                        is_in_job_chain             () const                                { return _combined_job_nodes && !_combined_job_nodes->is_empty(); }
    bool                     is_order_controlled            () const                                { return _is_order_controlled; }
    void                    set_order_controlled            ();
    void                    set_idle_timeout                ( const Time& );
    bool                        request_order               ( const Time& now, const string& cause );   // Fordert einen Auftrag fr die _order_queue an
    void                        withdraw_order_request      ();
    void                        set_job_chain_priority      ( int pri )                             { if( _job_chain_priority < pri )  _job_chain_priority = pri; }
    static bool                 higher_job_chain_priority   ( const Job* a, const Job* b )          { return a->_job_chain_priority > b->_job_chain_priority; }

    Module*                     module                      ()                                      { return _module; }
    ptr<Module_instance>        create_module_instance      ();
    Module_instance*            get_free_module_instance    ( Task* );
    void                        release_module_instance     ( Module_instance* );

    void                        increment_running_tasks     ()                                      { InterlockedIncrement( &_running_tasks_count ); }
    void                        decrement_running_tasks     ()                                      { InterlockedDecrement( &_running_tasks_count ); }

    void                        count_task                  ()                                      { InterlockedIncrement( &_tasks_count ); }
    void                        count_step                  ()                                      { InterlockedIncrement( &_step_count ); }


    friend struct               Object_set;
    friend struct               Task;
    friend struct               Module_task;
    friend struct               Job_module_task;
    friend struct               Object_set_task;
    friend struct               Com_job;
    friend struct               Task_subsystem;


    Fill_zero                  _zero_;
    bool                       _waiting_for_process;        // Task kann nicht gestartet werden, weil kein Prozess in der Prozessklasse verfgbar ist
    bool                       _waiting_for_process_try_again;  
    string                     _description;                // <description>

  protected:
    friend struct               Job_history;

    string                     _title;                      // <job title="">
    string                     _state_text;                 // spooler_job.state_text = "..."

    long32                     _tasks_count;                // Anzahl gestarteter Tasks seit Spooler-Start
    long32                     _step_count;                 // Anzahl spooler_process() aller Tasks

    State                      _state;
    State_cmd                  _state_cmd;
    bool                       _is_permanently_stopped;     // s_stopped wird zum Beenden verwendet und gilt nicht dauerhaft. Das sollte vereinfacht werden!
    bool                       _reread;                     // <script> neu einlesen, also <include> erneut ausfhren
    Time                       _task_timeout;               // Frist fr einen Schritt einer Task
    Time                       _idle_timeout;               // Frist fr den Zustand Task::s_running_waiting_for_order
    bool                       _force_idle_timeout;         // _idle_timeout wirkt beendet auch Tasks, wenn _min_tasks unterschritten wird
    Visibility                 _visible;
    bool                       _temporary;                  // Job nach einem Lauf entfernen
    bool                       _start_once_for_directory;
    bool                       _machine_resumable;          // Test
    bool                       _stop_on_error;              // Nach Task-Fehler Job stoppen (default)
    bool                       _ignore_every_signal;        // Nur Unix: Nach Beendigung mit Signal (kill, Absturz) den Job nicht stoppen 
    stdext::hash_set<int>      _ignore_signals_set;

    bool                       _log_append;                 // Jobprotokoll fortschreiben <job log_append=(yes|no)>

    ptr<Job_schedule_use>      _schedule_use;
    bool                       _start_once;                 // <run_time start_once="">, wird false nach Start
    Period                     _period;                     // Derzeitige oder nchste Period
    Time                       _next_single_start;
    Time                       _next_start_time;
    Time                       _next_time;                  // Fr Task_subsystem::wait(): Um diese Zeit soll Job::do_something() gerufen werden.

    Time                       _repeat;                     // spooler_task.repeat
    Time                       _delay_until;                // Nach Fehler verzgern

    xml::Document_ptr          _commands_document;          // <commands>...
    typedef map<int,xml::Element_ptr>  Exit_code_commands_map;
    Exit_code_commands_map     _exit_code_commands_map;
    xml::Element_ptr           _exit_code_commands_on_signal;
    Delay_after_error          _delay_after_error;
    long32                     _error_steps;                // Zahl aufeinanderfolgender Fehler

    Directory_watcher_list     _directory_watcher_list;
    Time                       _directory_watcher_next_time;
    bool                       _directory_changed;
    string                     _changed_directories;
    Xc_copy                    _error;

    ptr<Com_variable_set>      _default_params;


    ptr<Module>                _module;                     // Job hat ein eigenes Skript
    xml::Element_ptr           _script_element;             // <script> (mit <include>) fr <modify_job cmd="reload"/>

    typedef vector< ptr<Module_instance> >  Module_instance_vector;
    Module_instance_vector     _module_instances;
    ptr<Com_job>               _com_job;

    ptr<Task_queue>            _task_queue;                 // Warteschlange der nchsten zu startenden Tasks
    Task_list                  _running_tasks;              // Alle laufenden Tasks (auch die gestarteten, aber wartenden, z.B. s_running_waiting_for_order)
    long32                     _running_tasks_count;        // Anzahl der Tasks, die tatschlich laufen (und nicht gerade warten)
    int                        _min_tasks;                  // Min. Anzahl Tasks, die der Scheduler stets laufen lassen soll
    bool                       _start_min_tasks;            // Starte Tasks solange _running_tasks.count() < _min_tasks
    int                        _max_tasks;                  // Max. Anzahl gleichzeitig laufender Tasks. _running_tasks.size() <= _max_tasks!

    Job_history                _history;

  //ptr<Order_queue>           _order_queue;
    bool                       _is_order_controlled;

    ptr<Combined_job_nodes> _combined_job_nodes;

    int                        _job_chain_priority;         // Maximum der Prioritten aller Jobkettenknoten mit diesem Job. 

    Delay_order_after_setback  _delay_order_after_setback;
    int                        _max_order_setbacks;

    typedef list< pair<string,string> > Start_when_directory_changed_list;  
    Start_when_directory_changed_list  _start_when_directory_changed_list;      // Fr <start_when_directory_changed>

    ptr<lock::Requestor>       _lock_requestor;

    Time                       _db_next_start_time;
    bool                       _db_stopped;

    string                     _warn_if_shorter_than_string;
    string                     _warn_if_longer_than_string;
};

//-------------------------------------------------------------------------------------Internal_job

struct Internal_job : Job
{
                                Internal_job                ( Scheduler*, const string& name, const ptr<Module>& );
};

//------------------------------------------------------------------------------------------Job_map

#define FOR_EACH_JOB( JOB )  FOR_EACH_FILE_BASED( Job, JOB )

//---------------------------------------------------------------------------------------Job_folder

struct Job_folder : typed_folder< Job >
{
                                Job_folder                  ( Folder* );


  //void                        set_dom                     ( const xml::Element_ptr& );
  //void                        load_job_from_xml           ( const xml::Element_ptr&  );
  //void                        add_or_replace_job_from_xml ( const string& name, const xml::Element_ptr&, bool activate = false );
    void                        add_job                     ( const ptr<Job>& job )                 { add_file_based( +job ); }
    Job*                        job                         ( const string& name )                  { return file_based( name ); }
    Job*                        job_or_null                 ( const string& name )                  { return file_based_or_null( name ); }
  //xml::Element_ptr            dom_element                 ( const xml::Document_ptr&, const Show_what& );
    xml::Element_ptr            new_dom_element             ( const xml::Document_ptr& doc, const Show_what& )  { return doc.createElement( "jobs" ); }
};

//---------------------------------------------------------------------------Job_subsystem_interface

struct Job_subsystem_interface: Object, 
                                file_based_subsystem< Job >
{
                                Job_subsystem_interface     ( Scheduler*, Type_code );

    virtual ptr<Job_folder>     new_job_folder              ( Folder* )                             = 0;
    virtual int                 remove_temporary_jobs       ()                                      = 0;
    virtual bool                is_any_task_queued          ()                                      = 0;
    virtual void                append_calendar_dom_elements( const xml::Element_ptr&, Show_calendar_options* ) = 0;
    virtual Schedule*           default_schedule            ()                                      = 0;

    Job*                        job                         ( const Absolute_path& job_path )       { return file_based( job_path ); } 
    Job*                        job_or_null                 ( const Absolute_path& job_path )       { return file_based_or_null( job_path ); }
    xml::Element_ptr            new_file_baseds_dom_element ( const xml::Document_ptr& doc, const Show_what& ) { return doc.createElement( "jobs" ); }
};

ptr<Job_subsystem_interface>    new_job_subsystem           ( Scheduler* );

//-------------------------------------------------------------------------------------------------

} //namespace scheduler
} //namespace sos

#endif
