/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: subsystem.cxx 5445 2007-12-27 10:49:14Z jz $        Joacim Zschimmer, Zschimmer GmbH, http://www.zschimmer.com

#include "spooler.h"

namespace sos {
namespace scheduler {

//----------------------------------------------------------------------string_from_subsystem_state
    
string string_from_subsystem_state( Subsystem_state state )
{
    switch( state )
    {
        case subsys_not_initialized:    return "not_initialized";
        case subsys_initialized:        return "initialized";
        case subsys_loaded:             return "loaded";
        case subsys_active:             return "active";
        case subsys_stopped:            return "stopped";

        default: 
            Z_DEBUG_ONLY( assert("subsystem_state"==NULL) );
            return "Subsystem_state:" + as_string( state );
    }
}

//-----------------------------------------------------------Subsystem::throw_subsystem_state_error

void Subsystem::throw_subsystem_state_error( Subsystem_state state, const string& message_text )
{
    z::throw_xc( "SUBSYSTEM-STATE-ERROR", string_from_subsystem_state( state ), string_from_subsystem_state( _subsystem_state ), message_text );
}

//----------------------------------------------------------------Subsystem::assert_subsystem_state

void Subsystem::assert_subsystem_state( Subsystem_state expected_state, const string& message_text )
{
    if( _subsystem_state != expected_state )  throw_subsystem_state_error( expected_state, message_text );
}

//----------------------------------------------------------------Subsystem::switch_subsystem_state

bool Subsystem::switch_subsystem_state( Subsystem_state new_state )
{
    bool result = false;
    
    if( _subsystem_state != new_state )
    {
        Z_LOGI2( "scheduler", obj_name() << ": switch_subsystem_state " << string_from_subsystem_state ( new_state ) << "\n" );

        try
        {
            switch( new_state )
            {
                case subsys_initialized:
                {
                    assert_subsystem_state( subsys_not_initialized, Z_FUNCTION );

                    result = subsystem_initialize();
                    break;
                }

                case subsys_loaded:
                {
                    if( subsystem_state() < subsys_initialized )  switch_subsystem_state( subsys_initialized );
                    assert_subsystem_state( subsys_initialized, Z_FUNCTION );

                    result = subsystem_load();
                    break;
                }

                case subsys_active:
                {
                    if( subsystem_state() < subsys_loaded )  switch_subsystem_state( subsys_loaded );
                    assert_subsystem_state( subsys_loaded, Z_FUNCTION );

                    result = subsystem_activate();
                    break;
                }

                case subsys_stopped:
                {
                    close();
                    break;
                }

                default:
                    throw_subsystem_state_error( new_state, Z_FUNCTION );
            }

            Z_LOG2( "scheduler", obj_name() << ": state=" << string_from_subsystem_state( _subsystem_state ) << "\n" );
        }
        catch( exception& x )
        {
            //_log->error( message_string( "SCHEDULER-332", obj_name(), string_from_subsystem_state( new_state ) ) );
            z::throw_xc( "SCHEDULER-332", obj_name(), string_from_subsystem_state( new_state ), x );
        }
    }

    return result;
}

//------------------------------------------------------------------Subsystem::subsystem_initialize

bool Subsystem::subsystem_initialize()
{
    z::throw_xc( Z_FUNCTION, obj_name(), "not implemented" );
}

//------------------------------------------------------------------------Subsystem::subsystem_load

bool Subsystem::subsystem_load()
{
    z::throw_xc( Z_FUNCTION, obj_name(), "not implemented" );
}

//--------------------------------------------------------------------Subsystem::subsystem_activate

bool Subsystem::subsystem_activate()
{
    z::throw_xc( Z_FUNCTION, obj_name(), "not implemented" );
}

//-------------------------------------------------------------------------------------------------

} //namespace scheduler
} //namespace sos
