/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: embedded_files.cxx 5419 2007-12-06 14:15:42Z jz $

#include "zschimmer.h"
#include "embedded_files.h"

using namespace std;


namespace zschimmer {

//-------------------------------------------------------------------------------------------static

static Message_code_text error_codes[] =
{
    { "Z-NO-EMBEDDED-FILE", "Datei $1 ist nicht eingebettet" },
    {}
};

//-------------------------------------------------------------------------------------------Z_INIT

Z_INIT( embedded_files )
{
    add_message_code_texts( error_codes ); 
}

//--------------------------------------------------------------------------------get_embedded_file
    
const Embedded_file* Embedded_files::get_embedded_file( const string& filename ) const
{
    const Embedded_file* result = get_embedded_file_or_null( filename );
    if( !result )  throw_xc( "Z-NO-EMBEDDED-FILE", filename );
    return result;
}

//--------------------------------------------------------------------------------get_embedded_file
    
const Embedded_file* Embedded_files::get_embedded_file_or_null( const string& filename ) const
{
    
    for( const Embedded_file* f = _embedded_files; f->_filename; f++ )
    {
         if( f->_filename == filename )  return f;
    }

    return NULL;
}

//------------------------------------------------------------------------string_from_embedded_file

string Embedded_files::string_from_embedded_file( const string& filename ) const
{
    const Embedded_file* embedded_file = get_embedded_file( filename );
    return string( embedded_file->_content, embedded_file->_length );
}

//-------------------------------------------------------------------------------------------------

} //namespace zschimmer
