/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: java.h 5531 2008-01-18 13:39:38Z jz $

#ifndef __ZSCHIMMER_JAVA_H_
#define __ZSCHIMMER_JAVA_H_


#include <vector>
#include <jni.h>
#include "threads.h"
#include "z_com.h"
#include "log.h"


namespace zschimmer {
namespace java {

const int                       max_params                  = 10;

struct Vm;
struct Method;

//-------------------------------------------------------------------------------------------------

JNIEnv*                         jni_env                     ();
JavaVM*                         javavm_from_jnienv          ( JNIEnv* );
Vm*                             create_vm                   ( bool init = true );
Z_NORETURN void                 throw_java_ret              ( int return_value, const string& method_name, const string& = "" );
string                          expand_class_path           ( const string& class_path_with_wildcards );

//---------------------------------------------------------------------------------Java_thread_data

struct Java_thread_data
{
                                Java_thread_data            ()                                      : _zero_(this+1) {}


    Fill_zero                  _zero_;
    JNIEnv*                    _jni_env;
};

//-----------------------------------------------------------------------------------Java_exception

struct Java_exception : Xc
{
                                Java_exception              ( const char* code )                    : Xc(code) {}
    Z_GNU_ONLY(                ~Java_exception              () throw()                              {}  )               // gcc 3.2.2

    string                      name                        () const                                { return _exception_name; }
    string                      message                     () const                                { return _message; }

    string                     _exception_name;
    string                     _message;
};

//----------------------------------------------------------------------------------------------Env

struct Env 
{
                                Env                         ( JNIEnv* jenv = java::jni_env() )      : _jni_env( jenv ) {}
                                Env                         ( Vm* );

                                operator JNIEnv*            ()                                      { return jni_env(); }
                              //operator JavaVM*            ()                                      { return vm(); }
    JNIEnv*                     operator ->                 ()                                      { return jni_env(); }

    JNIEnv*                     jni_env                     ()                                      { return _jni_env; }
    JavaVM*                     vm                          ()                                      { return javavm_from_jnienv( _jni_env ); }
    Vm*                         get_vm                      ();

    Z_NORETURN void             throw_java                  ( const string&, const string& = "" );
  //Z_NORETURN void             throw_java                  ( jclass, const string& = "" );
    
  //void                        add_object                  ( Java_idispatch* );
  //void                        release_objects             (); //?                                      { _java_idispatch_list.clear(); }

    string                      string_from_jstring         ( const jstring& );
    jstring                     jstring_from_string         ( const char* str, size_t length );
    inline jstring              jstring_from_string         ( const char* str )                     { return jstring_from_string( str, strlen(str) ); }
    inline jstring              jstring_from_string         ( const string& str )                   { return jstring_from_string( str.data(), str.length() ); }

    string                      get_class_name              ( jclass );

    void                        set_java_exception          ( const char* what );
    void                        set_java_exception          ( const exception& );
    void                        set_java_exception          ( const _com_error& );
    void                        set_NullPointerException    ( const string& text = "" );

    bool                        is_byte_array_class         ( jobject );
    bool                        is_string_array_class       ( jobject );
    jclass                      get_spooler_idispatch_class_if_is_instance_of( jobject );


    // Einige Java-Methoden mit Fehlerprfung:
    jclass                      find_class                  ( const string& name );
    jmethodID                   get_method_id               ( jclass cls, const string& name, const string& signature, bool is_static = false ) { return get_method_id( cls, name.c_str(), signature.c_str(), is_static ); }
    jmethodID                   get_method_id               ( jclass, const char* name, const char* signature, bool is_static = false );
  //jclass                      get_object_class            ( jobject );


    JNIEnv*                    _jni_env;
    //?std::list< ptr<Java_idispatch> >  _java_idispatch_list;     // Hlt alle in einer native Methode erzeugten IDispatchs, bis release_objects()
    //std::list< Java_idispatch* >  _java_idispatch_list;     // Hlt alle in einer native Methode erzeugten IDispatchs, bis release_objects()
};

//-------------------------------------------------------------------------------------------Has_vm
// inlines am Ende des Headers
/*
struct Has_vm
{
                                Has_vm                      ( Vm* );

    JNIEnv*                     jenv                        ();
    Env                         env                         ();
    Vm*                         vm                          ();

    void                    set_vm                          ( Vm* );

    ptr<Vm>                    _java_vm;
};
*/
//--------------------------------------------------------------------------------------Local_frame

struct Local_frame
{
                                Local_frame                 ( int capacity );
                               ~Local_frame                 ()                                      { _jni_env->PopLocalFrame(NULL); }

    JNIEnv*                    _jni_env;
};

//--------------------------------------------------------------------------------------Jvalue_kind
/*
struct Jvalue_kind
{
    jvalue_none = 0,
    jvalue_z    = 'Z',
    jvalue_b    = 'B',
    jvalue_c    = 'C',
    jvalue_s    = 'S',
    jvalue_i    = 'I',
    jvalue_j    = 'J',
    jvalue_f    = 'F',
    jvalue_d    = 'D',
    jvalue_l
};
*/
//-------------------------------------------------------------------------------------------Jvalue

struct Jvalue
{
                                Jvalue                      ( jobject value )                       : _type( 'L' ) { _jvalue.l = value; }
                                Jvalue                      ( jboolean value )                      : _type( 'Z' ) { _jvalue.z = value; }
                                Jvalue                      ( bool value )                          : _type( 'Z' ) { _jvalue.z = value; }
                                Jvalue                      ( char value )                          : _type( 'C' ) { _jvalue.c = value; }
                                Jvalue                      ( jshort value )                        : _type( 'S' ) { _jvalue.s = value; }
                                Jvalue                      ( jint value )                          : _type( 'I' ) { _jvalue.i = value; }
                                Jvalue                      ( jlong value )                         : _type( 'j' ) { _jvalue.j = value; }
                                Jvalue                      ( jfloat value )                        : _type( 'F' ) { _jvalue.f = value; }
                                Jvalue                      ( jdouble value )                       : _type( 'D' ) { _jvalue.d = value; }

    const jvalue&               get_jvalue                  ()                                      { return _jvalue; }

    jobject                     as_jobject                  ()                                      { check( 'L' );  return _jvalue.l; }
    bool                        as_bool                     ()                                      { check( 'Z' );  return _jvalue.b != 0; }

    void                        check                       ( char type );

    jvalue                     _jvalue;
    char                       _type;
};

//------------------------------------------------------------------------------------------Jobject

struct Jobject : Jvalue
{
    //enum Registered { reg_none, reg_local, reg_global };

  protected:
                                Jobject                     ()                                      : Jvalue( (jobject)NULL )  {}
                                Jobject                     ( jobject jo )                          : Jvalue( jo ) {}
                                Jobject                     ( const Jobject& )                      : Jvalue( (jobject)NULL )  {}   // Muss von Unterklasse implementiert werden!

    Jobject&                    operator =                  ( const Jobject& o )                    { assign_add_ref( o.get_jobject() );  return *this; } 

  public:
    virtual                    ~Jobject                     ()                                      {}


                                operator jobject            () const                                { return get_jobject(); }
    Jobject&                    operator =                  ( jobject jo )                          { assign( jo );  return *this; }
    bool                        operator !                  () const                                { return get_jobject() == NULL; }

    jobject                     take                        ()                                      { jobject result = _jvalue.l;  _jvalue.l = NULL;  return result; }

  //void                        set_global                  ();

    virtual void                assign                      ( jobject )                             = 0;    // jobject muss eine LocalRef sein, und wird danach ungltig!
    virtual void                assign_add_ref              ( jobject )                             = 0;    // jobject muss eine LocalRef sein, und wird danach ungltig!
    virtual jclass              get_jclass                  ()                                      = 0;

    jobject                     get_jobject                 () const                                { return _jvalue.l; }

    jmethodID                   method_id                   ( const char*   name, const char*   signature );
    jmethodID                   method_id                   ( const string& name, const string& signature ) { return method_id( name.c_str(), signature ); }

    jmethodID                   static_method_id            ( const char*   name, const char*   signature );
    jmethodID                   static_method_id            ( const string& name, const string& signature ) { return static_method_id( name.c_str(), signature.c_str() ); }

    void                        call_void_method            ( const char* name, const char* signature, ... );
    jobject                     call_object_method          ( const char* name, const char* signature, ... );
    jint                        call_int_method             ( const char* name, const char* signature, ... );
    jlong                       call_long_method            ( const char* name, const char* signature, ... );
    bool                        call_bool_method            ( const char* name, const char* signature, ... );
    bool                        call_bool_method            ( jmethodID, ... );
    string                      call_string_method          ( const char* name, const char* signature, ... );
    string                      call_string_method          ( jmethodID, ... );

    jobject                     call_static_object_method   ( const char* name, const char* signature, ... );


    //Registered                 _registered;
};

//-----------------------------------------------------------------------------------Global_jobject

struct Global_jobject : Jobject
{
                                Global_jobject              ()                                      : _jclass(NULL)  {}
                                Global_jobject              ( jobject jo )                          : _jclass(NULL)  { assign( jo ); }
                                Global_jobject              ( const Global_jobject& o )             : _jclass(NULL)  { assign( o.get_jobject() ); }
                               ~Global_jobject              ()                                      { assign( NULL ); }

    Global_jobject&             operator =                  ( jobject jo )                          { assign_add_ref( jo );  return *this; }
    bool                        operator !                  () const                                { return get_jobject() == NULL; }

    void                        assign                      ( jobject jo )                          { assign_add_ref( jo ); }
    void                        assign_add_ref              ( jobject );
    jclass                      get_jclass                  ();

  private:
    jclass                     _jclass;
};

//---------------------------------------------------------------------------------global_jobject<>

template< class CLASS >
struct global_jobject : Global_jobject
{
                                global_jobject              ()                                      {}
                                global_jobject              ( CLASS jo )                            { assign( jo ); }

                                operator CLASS              () const                                { return (CLASS)get_jobject(); }
    global_jobject<CLASS>&      operator =                  ( CLASS jo )                            { assign( jo );  return *this; }
    bool                        operator !                  () const                                { return get_jobject() == NULL; }
    bool                        operator ==                 ( CLASS jo ) const                      { return (CLASS)get_jobject() == jo; }
    bool                        operator !=                 ( CLASS jo ) const                      { return (CLASS)get_jobject() != jo; }
    void                        assign                      ( CLASS jo )                            { Global_jobject::assign( jo ); }
};

//------------------------------------------------------------------------------------Local_jobject

struct Local_jobject : Jobject
{
                                Local_jobject               ()                                      : _jclass(NULL), _call_close_at_end(false)  {}
                                Local_jobject               ( jobject jo )                          : _jclass(NULL), _call_close_at_end(false)  { assign( jo ); }
                                Local_jobject               ( const Local_jobject& o )              : _jclass(NULL)  { assign_add_ref( o.get_jobject() ); }
                               ~Local_jobject               ();

    Local_jobject&              operator =                  ( jobject jo )                          { assign( jo );  return *this; }
    bool                        operator !                  () const                                { return get_jobject() == NULL; }

    void                        assign_add_ref              ( jobject );
    void                        assign                      ( jobject );
    jclass                      get_jclass                  ();
    void                        call_close_at_end           ( bool b = true )                       { _call_close_at_end = b; }

  private:
    jclass                     _jclass;
    bool                       _call_close_at_end;
};

//----------------------------------------------------------------------------------local_jobject<>

template< class CLASS >
struct local_jobject : Local_jobject
{
                                local_jobject               ()                                      {}
                                local_jobject               ( CLASS jo )                            { assign( jo ); }

                                operator CLASS              () const                                { return (CLASS)get_jobject(); }
                                operator bool               () const                                { return get_jobject() != NULL; }
    bool                        operator !                  () const                                { return get_jobject() == NULL; }
    local_jobject<CLASS>&       operator =                  ( CLASS jo )                            { assign( jo );  return *this; }
    void                        assign                      ( CLASS jo )                            { Local_jobject::assign( jo ); }
};

//------------------------------------------------------------------------------------Local_jstring

struct Local_jstring : local_jobject<jstring>
{
                                Local_jstring               ()                                      {}
                                Local_jstring               ( jstring jstr )                        { assign( jstr ); }
                                Local_jstring               ( const string& str )                   { assign( str ); }
                                Local_jstring               ( const char* str )                     { assign( str ); }
                                Local_jstring               ( const char* str, size_t length )      { assign( str, length ); }

  //STDMETHODIMP                QueryInterface              ( const IID&, void** );

    Local_jstring&              operator =                  ( jstring jstr )                        { assign( jstr );  return *this; }
    Local_jstring&              operator =                  ( const string& str )                   { assign( str  );  return *this; }
    Local_jstring&              operator =                  ( const char* str )                     { assign( str  );  return *this; }
                                operator jstring            ()                                      { return get_jstring(); }

    void                        assign                      ( jstring jstr )                        { local_jobject<jstring>::assign( jstr ); }
    void                        assign                      ( const string& str )                   { assign( str.data(), str.length() ); }
    void                        assign                      ( const char* str )                     { assign( str, strlen( str ) ); }
    void                        assign                      ( const char* s, size_t l )             { assign( Env().jstring_from_string( s, l ) ); }
    jstring                     get_jstring                 ()                                      { return (jstring)get_jobject(); }
};

//----------------------------------------------------------------------------Read_jstring_critical
// Nur kurzzeitig benutzen! Keine JNI-Aufrufe bis ~Read_jstring_critical!

struct Read_jstring_critical : Non_cloneable
{
                                Read_jstring_critical       ( jstring );
                               ~Read_jstring_critical       ();

    int                         length                      () const                                { return _jni_env->GetStringLength( _jstring ); }
                                operator const jchar*       ()                                      { return _jchars; }
                              //operator const OLECHAR*     ()                                      { return (const OLECHAR*)_jchars; }
    const jchar&                operator []                 ( int i )                               { return _jchars[ i ]; }

    JNIEnv*                    _jni_env;
    jstring                    _jstring;
    const jchar*               _jchars;
};

//--------------------------------------------------------------------------------------------Class

struct Class : Global_jobject
{
                                Class                       ( const char* name = NULL );
                                Class                       ( const string& name );

    void                        load                        ( const string& name );

    Class&                      operator =                  ( jclass c )                            { assign( c ); return *this; }
                                operator jclass             () const                                { return (jclass)get_jobject(); }

    jmethodID                   method_id                   ( const char*   name, const char*   signature );
    jmethodID                   method_id                   ( const string& name, const string& signature ) { return method_id( name.c_str(), signature ); }

    jmethodID                   static_method_id            ( const char*   name, const char*   signature );
    jmethodID                   static_method_id            ( const string& name, const string& signature ) { return static_method_id( name.c_str(), signature.c_str() ); }

    jobject                     new_object                  ( const char* signature, ... );

    string                     _name;
};

//-------------------------------------------------------------------------------------------Method

struct Method
{
                                Method                      ( Class* cls, const char* name, const char* signature ) : _class(cls) { assign( name, signature ); }
                                Method                      ( Class* cls, const char* name_and_signature          ) : _class(cls) { assign( name_and_signature ); }

                                operator jmethodID          ()                                      { return id(); }

    void                        assign                      ( const char* name, const char* signature, bool is_static = false );
    void                        assign                      ( const string& name_and_signature, bool is_static = false );
    jmethodID                   id                          ()                                      { return _id; }


    Z_DEBUG_ONLY( string       _name_and_signature; )
    Class*                     _class;
    jmethodID                  _id;
};

//------------------------------------------------------------------------------------Static_method

struct Static_method : Method
{
                                Static_method               ( Class* cls, const char* name, const char* signature ) : Method( cls, "", "" ) { assign( name, signature ); }
                                Static_method               ( Class* cls, const char* name_and_signature ) : Method( cls, NULL ) { assign( name_and_signature ); }

    void                        assign                      ( const char* name, const char* signature )     { Method::assign( name, signature, true ); }
    void                        assign                      ( const string& name_and_signature )            { Method::assign( name_and_signature, true ); }
};

//-----------------------------------------------------------------------------------------------Vm
// Fr dieselbe JavaVM knnen mehrere VM vorhanden sein, fr jede DLL eine.
// Neben dem Hauptprogram zurzeit hostole.dll, spidermonkey.dll, hostjava.dll.

struct Vm : Object              // Java virtual machine
{
    struct Option
    {
                                Option                      ( const string& option, void* extra = NULL ) : _option(option), _extra(extra) {}

        string                 _option;
        void*                  _extra;
    };


    struct Instance
    {
                                Instance                    ( const string& class_name, jobject jo ) : _class_name(class_name), _jobject(jo) {}

        string                 _class_name;
        jobject                _jobject;
    };


    struct Standard_classes : Object
    {
                                Standard_classes            ()                                      : _zero_(this+1) {}
                               ~Standard_classes            ();

        void                    load                        ( JNIEnv* );

        Fill_zero              _zero_;
        global_jobject<jclass> _java_lang_string_class;
        global_jobject<jclass> _java_lang_short_class;
        global_jobject<jclass> _java_lang_integer_class;
        global_jobject<jclass> _java_lang_long_class;
        global_jobject<jclass> _java_lang_float_class;
        global_jobject<jclass> _java_lang_double_class;
        global_jobject<jclass> _java_lang_boolean_class;
        global_jobject<jclass> _java_lang_byte_class;
        global_jobject<jclass> _java_lang_runtimeexception_class;
        global_jobject<jclass> _java_lang_nullpointerexception_class;
        global_jobject<jclass> _java_util_date_class;

        jmethodID              _java_lang_short_constructor_id;
        jmethodID              _java_lang_integer_constructor_id;
        jmethodID              _java_lang_long_constructor_id;
        jmethodID              _java_lang_float_constructor_id;
        jmethodID              _java_lang_double_constructor_id;
        jmethodID              _java_lang_boolean_constructor_id;
        jmethodID              _java_lang_byte_constructor_id;
        jmethodID              _java_util_date_constructor_id;
    };

                                Vm                          ( bool do_start = true );
                                Vm                          ( JavaVM* );                            // Vorhandene JVM verwenden (fr natives Module)
                               ~Vm                          ();


    void                    set_debug                       ( bool d )                              { _debug = d; }
    bool                        debug                       ()                                      { return _debug; }

    bool                        running                     () const                                { return _vm != NULL; }

    void                        new_instances               ( const string& class_names );          // "class:class:..."
    void                        load_standard_classes       ();
    Standard_classes*           standard_classes            ();
    void                    set_log                         ( Has_log* );
    void                    set_filename                    ();
    void                    set_filename                    ( const string& module_filename )       { _filename = module_filename; }
    string                      filename                    () const                                { return _filename; }

    void                    set_class_path                  ( const string& class_path )            { _class_path = class_path; }
    string                      class_path                  ()                                      { expand_class_path(); return _complete_class_path; }
    void                        append_class_path           ( const string& path )                  { _class_path += Z_PATH_SEPARATOR; _class_path += path; }
    void                        prepend_class_path          ( const string& path )                  { _class_path = path + Z_PATH_SEPARATOR + _class_path; }
    void                        expand_class_path           ();

    void                    set_javac_filename              ( const string& filename )              { _javac_filename = filename; }
    string                      javac_filename              () const                                { return _javac_filename; }

    void                    set_work_dir                    ( const string& directory )             { _work_dir = directory; }
    string                      work_dir                    ()                                      { return _work_dir; }

    void                    set_destroy_vm                  ( bool );

    void                    set_sos_initialized             ( bool b = true )                       { _sos_initialized = b; }       // Fr kram/sos_java.h
    bool                        sos_initialized             () const                                { return _sos_initialized; }

    void                        init                        ();
    void                        start                       ();
    void                        close                       ();
    void                        log_version                 ( JNIEnv*, const string& module_filename );
    void                    set_options                     ( const string& );

    JNIEnv*                     attach_thread               ( const string& thread_name );
    void                        detach_thread               ();

    JavaVM*                     vm                          ()                                      { return _vm; }

    JNIEnv*                     jni_env                     ();

    void                        check_for_exception         ();

  //void                        add_object                  ( Java_idispatch* );
  //void                        release_objects             ()                                      { _java_idispatch_list.clear(); }


  //static void             set_jvm                         ( JavaVM* );
  //static JavaVM*              jvm                         ();
    static ptr<Vm>          get_vm                          ( bool start = true );
    static JavaVM*              request_jvm                 ();
    static void                 release_jvm                 ( JavaVM* );

    static Mutex                static_vm_mutex;
    static Vm*                  static_vm;
  //static JavaVM*              static_foreign_jvm;
  //static JavaVM*              static_jvm;                 // In java_jvm.cxx


    Fill_zero                  _zero_;
    bool                       _foreign;                    // JVM ist von auen (fr Hostjava)
    bool                       _vm_requested;
    bool                       _dont_destroy;               // DestroyJavaVM() nicht rufen (hngt manchmal)
    bool                       _debug;
    bool                       _debug_options_set;
    bool                       _sos_initialized;
    Delegated_log              _log;
    JavaVMInitArgs             _vm_args;
    std::vector<Option>        _options;
    JavaVM*                    _vm;
  //Thread_data<Java_thread_data> _thread_data;

    string                     _filename;                   // Dateiname der Java-VM
    string                     _class_path;
    string                     _last_expanded_class_path;   // Fr expand_class_path()
    string                     _complete_class_path;
    string                     _javac_filename;
    string                     _work_dir;
    string                     _new_instances;

    typedef std::vector<Instance> Instances;
    Instances                  _instances;

  //jclass                     _idispatch_jclass;
  //list< ptr<Java_idispatch> >_java_idispatch_list;        // Hlt alle in einer nativen Methode erzeugten IDispatchs, bis release_objects()

    ptr<Standard_classes>      _standard_classes;
};

//---------------------------------------------------------------------------------------------Call
/*
struct Call
{
                                Call                        ( Env& e, jclass, const char* name_and_signature );
                                Call                        ( Env& e, jobject, const char* name_and_signature );

    void                        add_param                   ( jvalue v )                            { _jvalues[ _jvalues_count++ ] = v; }
    jobject                     call_unchecked_jobject      ();

    Fill_zero                  _zero_;
    jobject                    _jobject;
    jclass                     _jclass;
    jmethodID                  _method_id;
    int                        _is_static;
    int                        _jvalues_count;
    jvalue                     _jvalues [ max_params ];
};
*/
//-------------------------------------------------------------------------------------------------


inline Vm*                      Env::get_vm                 ()                                      { return Vm::static_vm; }

//------------------------------------------------------------------------------------inline Has_vm
/*
inline                          Has_vm::Has_vm              ( Vm* vm )                              : _java_vm(vm) {}
inline void                     Has_vm::set_vm              ( Vm* vm )                              { _java_vm = vm; }
inline Vm*                      Has_vm::vm                  ()                                      { return _java_vm; }
inline JNIEnv*                  Has_vm::jenv                ()                                      { return _java_vm->jenv(); }
inline Env                      Has_vm::env                 ()                                      { return Env( _java_vm->jenv() ); }
*/
//-------------------------------------------------------------------------------------------------

} //namespace java
} //namespace zschimmer


#endif
