/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: threads_base.h 5542 2008-01-27 15:27:19Z jz $

#ifndef ZSCHIMMER_THREADS_BASE_H
#define ZSCHIMMER_THREADS_BASE_H

#include "zschimmer.h"

namespace zschimmer {


//---------------------------------------------------------------------------------------Event_base
/*
    Bei den Events knnte besser getrennt werden, welche Methoden eine Systemfunktion aufrufen
    und welche Methoden nur das Flag _signaled ndern. 
*/

struct Event_base : Object//, Has_set_signaled
{
    enum Dont_create { dont_create };

                                Event_base                  ()                                      : _zero_(this+1) {}
                                Event_base                  ( const string& name )                  : _zero_(this+1), _name(name) {}
    virtual                    ~Event_base                  ();

    virtual void                close                       () = 0;

    virtual void                set_name                    ( const string& name )                  { _name = name; }
    string                      name                        ()                                      { return _name; }

    virtual bool                valid                       () const = 0;
  //virtual void            set_signal                      () = 0;
    virtual void                signal                      ( const string& name ) = 0;
    virtual bool                signaled                    ()                                      { return _signaled; }   // Nicht const, kann Zustand ndern
    bool                        signaled_flag               () const                                { return _signaled; }   // const
    void                    set_signaled                    ( bool b = true )                       { _signaled = b; }
    void                    set_signaled                    ( const string& name )                  { _signaled = true; _signal_name = name; }
    virtual void                reset                       () = 0;
  //virtual bool                signaled_after_check        ()                                      { return _signaled; }

    void                    set_waiting_thread_id           ( Thread_id id )                        { _waiting_thread_id = id; }  // Wenn nur dieser Thread wartet

    virtual bool                wait                        ( double seconds ) = 0;
    bool                        wait_until_localtime        ( double local_time );                  // Bercksichtigt Sommerzeitumstellung
    bool                        wait_until_gmtime           ( double gm_time );
    virtual string              as_text                     () const;
    friend ostream&             operator <<                 ( ostream& s, const Event_base& e )     { s << e.as_text(); return s; }


    Fill_zero                  _zero_;
    string                     _name;
    string                     _signal_name;
    bool                       _signaled;
    bool                       _waiting;
    Thread_id                  _waiting_thread_id;


  private:
                                Event_base                  ( const Event_base& );                   // Nicht implementiert. CRITICAL_SECTION darf nicht kopiert oder verschoben werden
    void                        operator =                  ( const Event_base& );                   // Nicht implementiert.
};

//--------------------------------------------------------------------------------------Thread_base

struct Thread_base : Object
{
    typedef Thread_id           Id;


                                Thread_base                 ( const string& name )                  : _zero_(this+1), _thread_name(name) {}
    virtual                    ~Thread_base                 ()                                      {}

    virtual void                thread_close                ()                                      = 0;

    void                    set_thread_name                 ( const string& name )                  { _thread_name = name; }
    string                      thread_name                 () const                                { return _thread_name; }

    void                    set_thread_termination_event    ( Event_base* event )                   { _thread_termination_event = event; }

    int                         thread_exit_code            ()                                      { return _thread_exit_code; }
    virtual string              thread_as_text              () const                                { return "Thread"; }
    virtual string              as_text                     () const                                { return thread_as_text(); }

    int                         call_thread_main            ();                                     // Mit thread_call_main() zusammenfassen?

  protected:
    void                        thread_call_main            ();
    virtual int                 thread_main                 ()                                      = 0;
    virtual void                thread_init                 ()                                      = 0;
    virtual void                thread_exit                 ( int exit_code )                       = 0;

    Fill_zero                  _zero_;
    string                     _thread_name;
    int                        _thread_exit_code;
    Event_base*                _thread_termination_event;
};

//-------------------------------------------------------------------------------------------------

} //namespace zschimmer

#endif
