/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: z_windows_directory.cxx 4336 2006-06-06 18:03:24Z jz $

#include "zschimmer.h"
#include "z_directory.h"

namespace zschimmer {
namespace windows {

//-------------------------------------------------------------------Simple_directory_reader::close
    
void Simple_directory_reader::close() 
{ 
    if( _handle != -1 )  _findclose( _handle ),  _handle = -1; 
}

//-------------------------------------------------------------------Simple_directory_reader::first

void Simple_directory_reader::open( const string& path, Flags flags, const string& pattern ) 
{ 
    //string path_pattern = path + "/" + pattern;
    // Windows vergleicht nur die ersten drei Zeichen der Erweiterung. *.jar liefert also auch x.jar~
    // Also machen wir das mit regulren Ausdrcken

    S regex_pattern;

    regex_pattern << '^';

    for( uint i = 0; i < pattern.length(); i++ )
    {
        char c = pattern[ i ];
        switch( c )
        {
            case '*': regex_pattern << ".*";    break;
            case '?': regex_pattern << ".";     break;
            case '.':
            case '+':
            case '{':
            case '}':
            case '(':
            case ')':
            case '|':
            case '[':
            case ']':
            case '^':
            case '$':
            case '\\': regex_pattern << '\\' << c;  break;
            default: regex_pattern << c;
        }
    }

    regex_pattern << '$';

    _regex.compile( regex_pattern, Regex::rx_ignore_case );


    string path_pattern = ( path == ""? "." : path ) + "/*";
    _flags = flags;

    _handle = _findfirsti64( path_pattern.c_str(), &_entry ); 
    if( _handle == -1 )  throw_errno( errno, "_findfirst", path.c_str() );  

    _is_first_entry = true;
}

//---------------------------------------------------------------------Simple_directory_reader::get

string Simple_directory_reader::get() 
{ 
    while(1)
    {
        if( _is_first_entry )  _is_first_entry = false;
        else
        {
            int ret = _findnexti64( _handle, &_entry ); 
            if( ret == -1 )  
            {
                if( errno == ENOENT )  return "";
                throw_errno( errno, "_findnext" ); 
            }
        }

        if( !_regex.match( _entry.name ) )  continue;

        if( _entry.attrib & _A_SUBDIR )
        {
            if( _flags & no_subdirectory )  continue;
            if( strcmp( _entry.name, "."  ) == 0 )  continue;
            if( strcmp( _entry.name, ".." ) == 0 )  continue;
        }

        break;
    }

    //if( _entry.attrib & _A_SUBDIR )  return string( _entry.name ) + Z_DIR_SEPARATOR;
    return _entry.name; 
}

//-------------------------------------------------------------------------------------------------

} //namespace windows
} //namespace zschimmer
