/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: z_windows_event.cxx 4522 2006-09-18 17:38:54Z jz $

#include "zschimmer.h"

#ifdef Z_WINDOWS

#include "z_windows_event.h"
#include "threads.h"
#include "log.h"

namespace zschimmer {
namespace windows {


//-----------------------------------------------------------------------------windows_message_step

static void windows_message_step()
{
    MSG msg;

    while(1)
    {
        msg.message = WM_NULL;

        int ret = PeekMessage( &msg, NULL, 0, 0, PM_REMOVE );
        if( !ret )  break;
        if( msg.message == WM_NULL )  break;

        //TranslateMessage( &msg ); 
        DispatchMessage( &msg ); 
    }
}

//-----------------------------------------------------------------------------------wait_for_event

static bool wait_for_event( HANDLE handle, double wait_time )
{
    assert( handle );

    do
    {
        const double max_t = (double)(INT_MAX-10) / 1000.0;
        int ms = int( min( max_t, wait_time ) * 1000.0 );
        if( ms < 0 )  ms = 0;

        if( ms > 0 )  Z_LOG( "MsgWaitForMultipleObjects(" << (void*)handle << "," << ( ms * 1000.0 ) << "s)\n" );
        int ret = MsgWaitForMultipleObjects( 1, &handle, FALSE, ms, QS_ALLINPUT ); 

        if( ret == WAIT_OBJECT_0 )  return true;
        
        if( ret == WAIT_OBJECT_0 + 1 )  windows_message_step();
        else
        if( ret != WAIT_TIMEOUT )  throw_mswin( "MsgWaitForMultipleObjects" );

        wait_time -= max_t;
    }
    while( wait_time > 0 );

    return false;
}

//-------------------------------------------------------------------------------------Event::Event

Event::Event( const string& name )
: 
    _zero_(this+1),
    _mutex( "Event " + name)
{
    set_name( name );
    create();
}

//----------------------------------------------------------------------------------- Event::~Event

Event::~Event()
{
}

//------------------------------------------------------------------------------------Event::create

void Event::create()
{
    //Z_LOG( "CreateEvent " << _name << "\n" );
    _handle = CreateEvent( NULL, FALSE, FALSE, NULL );
    if( !_handle )  throw_mswin( "CreateEvent", _name.c_str() );
}

//--------------------------------------------------------------------------------Event::set_signal
/*
void Event::set_signal()
{
    Z_MUTEX( _mutex )
    {
        Z_LOG( "Event(" << _name << "," << _signal_name << ").set_signal()\n" );

        _signaled = true;
    }
}
*/
//------------------------------------------------------------------------------------Event::signal

void Event::signal( const string& name )
{
    Z_MUTEX( _mutex )
    {
        _signaled = true;
        _signal_name = name;

        if( current_thread_id() != _waiting_thread_id )
        {
            //Z_LOG2( "event", "Event(" << _name << "," << _signal_name << ").signal()  SetEvent()\n" );

            BOOL ok = SetEvent( _handle );  
            if( !ok )  throw_mswin( "SetEvent", _name.c_str() );
        }
    }
}

//------------------------------------------------------------------------------Event::async_signal
// Keinen System-Aufruf machen! (Auer SetEvent)

void Event::async_signal( const char* )
{
    _signaled = true;
    SetEvent( _handle );  
}

//------------------------------------------------------------------------Event::signaled_then_reset

bool Event::signaled_then_reset()
{
    if( !_signaled )  return false;
    
    bool signaled = false;

    Z_MUTEX( _mutex )
    {
        signaled = _signaled;
        reset();
    }

    return signaled;
}

//-------------------------------------------------------------------------------------Event::reset

void Event::reset()
{
    Z_MUTEX( _mutex )
    {
        //Z_DEBUG_ONLY( Z_LOG( "Event(" << _name << "," << _signal_name << ").reset()  ResetEvent()\n" ); )

        _signaled = false;
        _signal_name = "";

        //Es ist ein Autoreset-Event. Das ist besser, denn dann muss nicht immer wieder ResetEvent() gerufen werden.
        //BOOL ok = ResetEvent( _handle );
        //if( !ok )  throw_mswin( "ResetEvent", _name.c_str() );
    }
}

//--------------------------------------------------------------------------------------Event::wait

bool Event::wait( double wait_time )
{
    if( _signaled ) 
    {
        reset();
        return true;
    }

    if( wait_for_event( _handle, wait_time ) )
    {
        _signaled = true;
        return true;
    }

    return false;
}

//-------------------------------------------------------------------------------------------------

} //namespace windows
} //namespace zschimmer

#endif

