/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: z_windows_mutex.cxx 3546 2005-04-03 08:30:29Z jz $

#include "zschimmer.h"

#ifdef Z_WINDOWS

#include "z_windows_mutex.h"
#include <process.h>
#include "log.h"

namespace zschimmer {
namespace windows {

//-------------------------------------------------------------------------------------------static

typedef BOOL (WINAPI *Proc)(LPCRITICAL_SECTION);

static volatile Proc      tryentercriticalsection = (Proc)-1;
static volatile HINSTANCE kernel32;

//---------------------------------------------------------------------------------Mutex::try_enter

bool Mutex::try_enter()
{
    bool ok = false;

    if( tryentercriticalsection == (Proc)-1 )
    {
        OSVERSIONINFO v;  memset( &v, 0, sizeof v ); v.dwOSVersionInfoSize = sizeof v;

        GetVersionEx( &v );
        if( v.dwPlatformId == VER_PLATFORM_WIN32_NT )  // Windows NT oder Windows 2000?
        {
            kernel32 = LoadLibrary( "kernel32.dll" );
            if( kernel32 )  tryentercriticalsection = (Proc)GetProcAddress( kernel32, "TryEnterCriticalSection" );
        }

        if( tryentercriticalsection == (Proc)-1 )  tryentercriticalsection = NULL;
    }

    Proc t = tryentercriticalsection;
    if( t != NULL  &&  t != (Proc)-1 )  
    {
        ok = t( &_system_mutex ) != 0;
    }

    return ok;
}

//-------------------------------------------------------------------------Mutex::locking_thread_id

Thread_id Mutex::locking_thread_id()
{
    //return //Erst ab Windows 2003 Server: GetThreadId( _system_mutex.OwningThread );
    return 0;
}

//-------------------------------------------------------------------------------------------------

} //namespace windows
} //namespace zschimmer

#endif

