/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: z_windows_process.h 5986 2009-03-20 16:10:47Z jz $

#ifndef __ZSCHIMMER_WINDOWS_PROCESS_H
#define __ZSCHIMMER_WINDOWS_PROCESS_H


#include "z_windows.h"


namespace zschimmer {
namespace windows {

//-------------------------------------------------------------------------------------------------
    
bool                            try_kill_process_immediately( Process_handle, const string& debug_string = "" );
void                            kill_process_immediately    ( Process_handle, const string& debug_string = "" );
void                            kill_process_immediately    ( pid_t         , const string& debug_string = "" );
void                            try_kill_process_with_descendants_immediately( pid_t, Has_log* log, const Message_string* msg, const string& debug_string );
int                             priority_class_from_string  ( const string& );

//------------------------------------------------------------------------------------------Process

struct Process : Process_base, Handle
{
                                Process                     ();
                                Process                     ( int pid )                             : _zero_(this+1) { init(); attach_pid( pid ); }

    void                        init                        ();

    STDMETHODIMP            get_Priority_class              ( BSTR* result )                        { Z_COM_IMPLEMENT( hr = com::String_to_bstr( priority_class(), result ) ); } 

    void                        close                       ();
    void                        start                       ( const string& command_line );
    void                        start                       ( const std::vector<string>& program_and_parameters );
  //void                    set_startup_info                ( const STARTUP_INFO si )               { _startup_info = si; );
    void                        attach_handle               ( HANDLE handle, int pid )              { _handle = handle;  _pid = pid; }
    void                        attach_pid                  ( int pid );
    bool                    set_priority                    ( int );          //, Has_log* = NULL );
    int                         priority                    ();
  //virtual void            set_with_console_window         ( bool b )                              { _with_console_window = b; }
    bool                    set_priority_class              ( int );          //, Has_log* = NULL );// NORMAL_PRIORITY_CLASS etc.
    bool                    set_priority_class              ( const string& );//, Has_log* = NULL );
    string                      priority_class              ();               //  Has_log* = NULL );
  //bool                        raise_priority              ( int difference );
    void                        wait                        ();
    bool                        wait                        ( double seconds );                     // Nicht in Unix
    bool                        terminated                  ();
    int                         exit_code                   ();
    int                         termination_signal          ()                                      { return 0; }   // Windows kennt nur exit_code
    void                        kill                        ( int signal = 99 );
    bool                        try_kill                    ( int signal = 99 );
    HANDLE                      take_handle                 ()                                      { HANDLE result = _handle;  _handle = NULL, _pid = 0;  return result; }
    bool                        opened                      () const                                { return _handle != 0; }


    Fill_zero                  _zero_;
    STARTUPINFO                _startup_info;
    
  private:
    string                      make_environment_string     ();

    int                        _priority_class;
  //bool                       _with_console_window;
  //bool                       _raise_priority_set;
  //int                        _raise_priority;
};

//--------------------------------------------------------------------------------------Process_ref
/*
struct Process_ref : Process
{
                                Process_ref                 ( HANDLE handle )                       { attach_handle( handle ); }
                               ~Process_ref                 ()                                      { _handle = NULL; }
};
*/
//-------------------------------------------------------------------------------------------------

} //namespace windows

//-------------------------------------------------------------------------------------------------
} //namespace zschimmer

#endif
