/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: z_windows_service.h 5419 2007-12-06 14:15:42Z jz $

#ifndef __ZSCHIMMER_WINDOWS_SERVICE_H
#define __ZSCHIMMER_WINDOWS_SERVICE_H


#include "z_windows.h"
#include "threads.h"


namespace zschimmer {
namespace windows {

//------------------------------------------------------------------------------------------Service

struct Service : Object
{
  //typedef ptr<Service>        Constructor                 ( int argc, char** argv );


                                Service                     ()                                      : _zero_(this+1) {}
                               ~Service                     ()                                      {}

    int                         run_service                 ( const string& service_name, int argc, char** argv );

    void                    set_service_emulation_allowed   ( bool b )                              { _service_emulation_allowed = b; }
    void                    set_self_destruction_timeout    ( double t )                            { _self_destruction_timeout = t; }
    void                    set_service_status              ( int state, int checkpoint_value = 0, int wait_hint_seconds = 10 );
  //void                    set_start_duration_hint         ( double t )                            { _start_duration_hint = t; }
  //void                    set_stop_duration_hint          ( double t )                            { _stop_duration_hint = t; }
    string                      name                        () const                                { return _name; }

    virtual void                run                         ( int argc, char** argv )               = 0;
    virtual DWORD               service_control_handler     ( DWORD, DWORD, void* )                 = 0;
  //virtual void                service_start               ()                                      = 0;
  //virtual void                service_stop                ()                                      = 0;
  //virtual void                service_shutdown            ()                                      { service_stop(); }
  //virtual void                service_pause               ()                                      = 0;
  //virtual void                service_continue            ()                                      = 0;
    virtual void                abort_immediately           ()                                      {}

  private:
    static DWORD __stdcall      service_control_handler_static( DWORD, DWORD, void*, void* );       // HandlerEx
    DWORD                       service_control_handler_super ( DWORD, DWORD, void* );                    

    static void __stdcall       ServiceMain                 ( DWORD argc, char** argv );
    void                        service_main                ( DWORD argc, char** argv );

    static uint __stdcall       service_thread_static       ( void* );
    uint                        service_thread              ();

    void                        start_self_destruction      ();
    static uint __stdcall       self_destruction_thread_static( void* );
    uint                        self_destruction_thread     ();

    Fill_zero                  _zero_;
    string                     _name;
    bool                       _service_emulation_allowed;
    double                     _self_destruction_timeout;
    Thread_id                  _self_destruction_thread_id;
    bool                       _service_stop;               // STOP-Kommando von der Dienstesteuerung (und nicht von TCP-Schnittstelle)

    bool                       _abort_immediately;

    SERVICE_STATUS_HANDLE      _service_status_handle;
    Handle                     _thread_handle;
    string                     _service_name;
    int                        _argc;
    char**                     _argv;

    // Zustnde SERVICE_START_PENDING und SERVICE_STOP_PENDING nicht lnger als pending_timeout Sekunden:
    Event                      _service_main_terminated;
    windows::Handle            _pending_watchdog_signal;
    bool                       _pending_timed_out;
  //double                     _start_duration_hint;
  //double                     _stop_duration_hint;
    int                        _state;
};

//-------------------------------------------------------------------------------------------------

void                            install_service             ( const string& service_name, const string& service_display, const string& service_description, 
                                                              const string& dependencies, const string& params );
void                            remove_service              ( const string& service_name );
void                            start_service               ( const string& service_name );

string                          service_state_name          ( int state );

//-------------------------------------------------------------------------------------------------


} //namespace windows
} //namespace zschimmer

#endif
