/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: z_windows_system_information.cxx 5419 2007-12-06 14:15:42Z jz $

#include "zschimmer.h"
#include "system_information.h"
#include "z_com_server.h"

namespace zschimmer {

using namespace zschimmer::com;

//--------------------------------------------------------------------------------get_memory_status

static MEMORYSTATUSEX get_memory_status()
{
    MEMORYSTATUSEX memory_status;
    // dwMemoryLoad
    // dwTotalPhys
    // dwAvailPhys
    // dwTotalPageFile
    // dwAvailPageFile
    // dwTotalVirtual
    // dwAvailVirtual

    //  DWORD dwLength;  DWORD dwMemoryLoad;  DWORDLONG ullTotalPhys;  DWORDLONG ullAvailPhys;  DWORDLONG ullTotalPageFile;  DWORDLONG ullAvailPageFile;  DWORDLONG ullTotalVirtual;  DWORDLONG ullAvailVirtual;  DWORDLONG ullAvailExtendedVirtual;

    memory_status.dwLength = sizeof memory_status;

    BOOL ok = GlobalMemoryStatusEx( &memory_status );
    if( !ok )  throw_mswin( "GlobalMemoryStatusEx" );

    return memory_status;
}

//---------------------------------------------------------------------------------------set_result

static void set_result( VARIANT* result, uint64 value )
{
    // Windows lsst ein VT_I8 verschwinden (jedenfalls JScript)
    //result->vt = VT_I8;
    //V_I8( result ) = (int64)value;

    result->vt = VT_R8;
    V_R8( result ) = (double)value;
}

//----------------------------------------------------------------zschimmer::get_System_information

STDMETHODIMP zschimmer::get_System_information( BSTR what_bstr, VARIANT* parameter_vt, VARIANT* result )
{
    HRESULT hr = S_OK;;

    try
    {
        string what      = string_from_bstr( what_bstr );
        string parameter = string_from_variant( *parameter_vt );

        if( what == "free_disk_space" )
        {
            if( parameter_vt->vt != VT_BSTR )  return DISP_E_TYPEMISMATCH;

            ULARGE_INTEGER large_integer;

            BOOL ok = GetDiskFreeSpaceExW( V_BSTR( parameter_vt ), &large_integer, NULL, NULL );
            if( !ok )  throw_mswin( "GetDiskFreeSpaceExU", parameter );

            set_result( result, large_integer.QuadPart );
        }
        else
        if( what == "disk_space" )
        {
            if( parameter_vt->vt != VT_BSTR )  return DISP_E_TYPEMISMATCH;

            ULARGE_INTEGER large_integer;

            BOOL ok = GetDiskFreeSpaceExW( V_BSTR( parameter_vt ), NULL, &large_integer, NULL );
            if( !ok )  throw_mswin( "GetDiskFreeSpaceExU", parameter );

            set_result( result, large_integer.QuadPart );
        }
        else
        if( what == "total_free_disk_space" )
        {
            if( parameter_vt->vt != VT_BSTR )  return DISP_E_TYPEMISMATCH;

            ULARGE_INTEGER large_integer;

            BOOL ok = GetDiskFreeSpaceExW( V_BSTR( parameter_vt ), NULL, NULL, &large_integer );
            if( !ok )  throw_mswin( "GetDiskFreeSpaceExU", parameter );

            set_result( result, large_integer.QuadPart );
        }
        else
        if( what == "free_memory" )
        {
            set_result( result, get_memory_status().ullAvailPhys );
        }
        else
        if( what == "free_swap" )
        {
            set_result( result, get_memory_status().ullAvailPageFile );
        }
        else
        if( what == "free_virtual" )
        {
            set_result( result, get_memory_status().ullAvailVirtual );
        }
        else
            hr = E_INVALIDARG;
    }
    catch( const exception & x )  { hr = Set_excepinfo( x, Z_FUNCTION ); }

    return hr;
}

//-------------------------------------------------------------------------------------------------

} //namespace zschimmer
