/********************************************************* begin of preamble
**
** Copyright (C) 2003-2009 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: z_windows_thread.h 5542 2008-01-27 15:27:19Z jz $

#ifndef __ZSCHIMMER_WINDOWS_THREAD_H
#define __ZSCHIMMER_WINDOWS_THREAD_H

#include "base.h"

#ifdef Z_WINDOWS

#include "z_windows_event.h"


namespace zschimmer {
namespace windows {

//-------------------------------------------------------------------------------------------Thread

struct Thread : Thread_base, Non_cloneable
{
  //typedef uint                Id;


                                Thread                      ( const string& name = "" )             : Thread_base(name), _zero_(this+1), _thread_handle( Event::dont_create, name ) {}
                               ~Thread                      ();

    void                        thread_close                ();
    void                        thread_start                ();

    bool                        thread_wait_for_termination ( double wait_time = INT_MAX );

    bool                        thread_is_running           ();
    Id                          thread_id                   ()                                      { return _thread_id; }
  //void                    set_thread_id                   ( Id id )                               { _thread_id = id; }            // Nur sinnvoll, wenn das Objekt mal nicht als Thread luft (z.B. Task_subsystem)
    HANDLE                      thread_handle               ()                                      { return _thread_handle; }

    void                    set_thread_name                 ( const string& name );

    bool                        try_set_thread_priority     ( int pri )                             { return SetThreadPriority( _thread_handle._handle, pri ) != 0; }
    int                         thread_priority             ()                                      { return GetThreadPriority( _thread_handle._handle ); }
    virtual string              thread_as_text              () const;

  protected:
    friend uint __stdcall       thread_function             ( void* );
    void                        thread_init                 ();
    void                        thread_exit                 ( int exit_code );
  //virtual int                 thread_main                 () = 0;


  public:
    Fill_zero                  _zero_;
    Event                      _thread_handle;              // Trgt den Thread-Handle und signalisiert Ende des Threads
    Id                         _thread_id;
};


inline Thread::Id               current_thread_id           ()                                      { return GetCurrentThreadId(); }

//-----------------------------------------------------------------------------Thread_local_storage

struct Thread_local_storage
{
                                Thread_local_storage        ()                                      : _index(TLS_OUT_OF_INDEXES) {}
                               ~Thread_local_storage        ()                                      { free(); }

    void                        allocate                    ();
    void                        free                        ();
    void                        set                         ( void* value ) const                   { TlsSetValue( _index, value ); }
    void*                       get                         () const                                { return TlsGetValue( _index ); }

    bool                        is_allocated                () const                                { return _index != TLS_OUT_OF_INDEXES; }

    int                        _index;
};

//-------------------------------------------------------------------------------------------------

} //namespace windows
} //namespace zschimmer

#endif
#endif
